;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: b1.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;
;;; $Revision: 1.4 $
;;; $Date: 1996/05/07 16:47:40 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: b1.lisp,v $
;;; Revision 1.4  1996/05/07  16:47:40  rich
;;; Changes for clisp.
;;;
;;; Revision 1.3  1995/01/24  23:59:13  rich
;;; Release of tca 7.9.  Mostly speed improvements.
;;; The cvs binaries may now be located in /usr/local.
;;; Reformatting to less than 80 character lines.
;;;
;;; Revision 1.2  1993/06/13  23:25:38  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;

(unless (boundp 'SampleDataForm) (load "sample"))

(defun SelfQueryHnd (Ref x)

  (format T "SelfQueryHnd: Start.~%")

  (format T "x = ~d~%" x)

  (tca::tcaReply Ref 17)

  (format T "SelfQueryHnd: Done.~%"))

(defun SampleHnd (Ref Sample)
  
  (format T "SampleHnd: Start.~%")

  ;; You can now call a goal from a query
  (tca::tcaExpandGoal "GoalMsg" Sample)

;;;  (format T "press return") (read-char)

  (let (ReplyData)
    (cond (Sample
	   (format T "Sample: ~A~%" (Sample-s Sample))
	   (setq ReplyData (Sample-x Sample)))
	  (T (format T "Sample: NULL~%")
	     (setq ReplyData 51)))

    (tca::tcaReply Ref ReplyData)

    (format T "SampleHnd: Done.~%")))

(defun GoalHnd (Ref Sample)
  (declare (ignore Sample))

  (format T "GoalHnd: Start~%")

;;;  (format T "press return~%")
;;;  (read-char)

  (let (b)
    (format T "Calling tcaQuery~%")
    (tca::tcaQuery "SelfQueryMsg" 42 b)

    (format T "b = ~d~%" b)

;;;;    (tca::tcaFailure Ref "Random Failure Message" Sample)

    (tca::tcaSuccess Ref)

    (format T "GoalHnd: Done.~%")))

(defun PolyHnd (Ref Poly)
  (let (ReplyData)

    (format T "PolyHnd: Start.~%")

    (format T "Poly->silly: ~D~%" (poly-silly Poly))
    (format T "Poly->n: ~D~%" (Poly-npoly Poly))
    (dotimes (i (Poly-npoly Poly))
       (format T "Poly->x[~D]: ~F~%" i (aref (Poly-poly_x Poly) i))
       (format T "Poly->y[~D]: ~F~%" i (aref (Poly-poly_y Poly) i)))

    (setq ReplyData 7)

    (tca::tcaReply Ref ReplyData)

    (format T "PolyHnd: Done.~%")))

(defun MapHnd (Ref Map)
  (let (ReplyData)

    (format T "MapHnd: Start.~%")

    (format T "~F~%" (tca::aref-map Map 1 5))
    (format T "~F~%" (tca::aref-map Map 1 6))
    (format T "~F~%" (tca::aref-map Map 1 7))
    (format T "~F~%" (tca::aref-map Map 2 5))
    (format T "~F~%" (tca::aref-map Map 2 6))
    (format T "~F~%" (tca::aref-map Map 2 7))
  
    (setq ReplyData 7)

    (tca::tcaReply Ref ReplyData)

    (format T "MapHnd: End.~%")))

(defun LinkHnd (Ref s2)
  (let (ReplyData)

    (format T "LinkHnd: Start.~%")

    (do ((tmp s2 (s2-next tmp)))
	((null tmp))
      (format T "s2->a: ~D~%" (s2-a tmp))
      (format T "s2->b: ~D~%" (s2-b tmp))
      (format T "s2->s: ~A~%" (s2-s tmp))
      (format T "~%"))

    (setq ReplyData 9)

    (format T "LinkHnd: End.~%")

    (tca::tcaReply Ref ReplyData)))

(defun FixedArrayHnd (Ref fixed_array)
  (let (ReplyData)

    (format T "FixedArrayHnd: Start.~%")

    (dotimes (i FIXED_ARRAY_DIM1)
       (dotimes (j FIXED_ARRAY_DIM2)
          (format T "~D " (aref fixed_array i j)))
       (format T "~%"))

    (setq ReplyData 10)

    (format T "FixedArrayHnd: End.~%")

    (tca::tcaReply Ref ReplyData)))

(defun VarArrayHnd (Ref var_array)
  (let (ReplyData)

    (format T "VarArrayHnd: Start.~%")

    (dotimes (i (var_two_d_array-dim1 var_array))
       (dotimes (j (var_two_d_array-dim2 var_array))
          (format T "~D " (aref (var_two_d_array-elements var_array) i j)))
       (format T "~%"))

    (setq ReplyData 10)

    (format T "VarArrayHnd: End.~%")

    (tca::tcaReply Ref ReplyData)))

(defun set_point (point x1 y1 z1)
  (setf (point-x point) x1 (point-y point) y1 (point-z point) z1))

(defun set_two_points (two_points Leg_No Name x1 y1 z1 x2 y2 z2)
  (setf (two_points-leg_no two_points) Leg_No
	(two_points-name two_points) Name)
  (set_point (two_points-pt1 two_points) x1 y1 z1)
  (set_point (two_points-pt2 two_points) x2 y2 z2))

(defun print_point (apoint)
  (cond (apoint (format T "Point: ~F ~F ~F~%"
			(point-x apoint) (point-y apoint) (point-z apoint)))
	(T (format T "No Point~%"))))

(defun print_two_points (twopoints)
  (cond (twopoints
	 (format T "Two Points: ~D ~A~%" (two_points-leg_no twopoints)
		 (two_points-name twopoints))
	 (format T "     ") (print_point (two_points-pt1 twopoints))
	 (format T "     ") (print_point (two_points-pt2 twopoints)))
	(T (format T "No Points~%"))))

(defun print_complex_points (complexpoints)

  (format T "Complex Point:~%")
  (dotimes (i 6)
    (format T "~D. " i) 
    (print_point (aref (complex_points-points complexpoints) i)))
  (cond ((complex_points-length complexpoints)
	 (dotimes (i (complex_points-length complexpoints))
	    (format T "~D. " i) 
	    (print_two_points (aref (complex_points-more_points complexpoints) i))))
	(T (format T "No Points~%")))
  (print_point (complex_points-pt_ptr complexpoints)))

(defun PointsHnd (Ref Points)
  (let (reply_point)

    (print_complex_points Points)

    (cond ((complex_points-more_points Points) 
	   (setq reply_point (aref (complex_points-more_points Points) 1)))
	  (T (setq reply_point (make-two_points))
	     (set_two_points reply_point -1 "Null Reply" 0.0 0.0 0.0 0.0 0.0 0.0)))

    (tca::tcaReply Ref reply_point)))

(defun b1 (&optional (server (tca::tcaServerMachine)))
  (Format T "Connect ...~%")

  (tca::tcaConnectModule "Module B" server)

  (tca::tcaRegisterMessage "SelfQueryMsg" tca::QueryClass "int" "int")
  (tca::tcaRegisterHandler "SelfQueryMsg" "SelfQueryHnd" 'SelfQueryHnd)

  (tca::tcaRegisterMessage "LinkTestMsg" tca::QueryClass s2TypeForm
			  ReplyDataForm)
  (tca::tcaRegisterHandler "LinkTestMsg" "LinkHnd" 'LinkHnd)
#|
  (tca::tcaAddHndToResource "LinkHnd" "Module B")
|#
  (tca::tcaRegisterMessage "GoalMsg" tca::GoalClass SampleDataForm "")
  (tca::tcaRegisterHandler "GoalMsg" "GoalHnd" 'GoalHnd)

  (tca::tcaRegisterMessage "SampleMsg" tca::QueryClass SampleDataForm
			  ReplyDataForm)
  (tca::tcaRegisterHandler "SampleMsg" "SampleHnd" 'SampleHnd)

  (tca::tcaRegisterResource "Miasma" 1)
  (tca::tcaRegisterResource "Vapor" 1)
  (tca::tcaAddHndToResource "SampleHnd" "Miasma")
  (tca::tcaAddHndToResource "SampleHnd" "Vapor")

  (tca::tcaRegisterMessage "PolyMsg" tca::QueryClass PolyDataForm ReplyDataForm)
  (tca::tcaRegisterHandler "PolyMsg" "PolyHnd" 'PolyHnd)

  (tca::tcaRegisterMessage "MapMsg" tca::QueryClass MapDataForm ReplyDataForm)  
  (tca::tcaRegisterHandler "MapMsg" "MapHnd" 'MapHnd)

  (tca::tcaRegisterMessage "FixedArrayMsg" tca::QueryClass two_d_array_form 
		      ReplyDataForm)
  (tca::tcaRegisterHandler "FixedArrayMsg" "FixedArrayHnd" 'FixedArrayHnd)

  (tca::tcaRegisterMessage "VarArrayMsg" tca::QueryClass var_two_d_array_form 
		      ReplyDataForm)
  (tca::tcaRegisterHandler "VarArrayMsg" "VarArrayHnd" 'VarArrayHnd)

  (tca::tcaRegisterNamedFormatter "point" point_format)
  (tca::tcaRegisterNamedFormatter "two_points" two_points_format)
  (tca::tcaRegisterMessage "StructuredFormatters" tca::QueryClass 
		   complex_points_format "two_points")
  (tca::tcaRegisterHandler "StructuredFormatters" "PointsHnd" 'PointsHnd)

  (tca::tcaWaitUntilReady)

  (tca::tcaModuleListen))
