;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: b4.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;
;;; $Revision: 1.3 $
;;; $Date: 1995/01/24 23:59:15 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: b4.lisp,v $
;;; Revision 1.3  1995/01/24  23:59:15  rich
;;; Release of tca 7.9.  Mostly speed improvements.
;;; The cvs binaries may now be located in /usr/local.
;;; Reformatting to less than 80 character lines.
;;;
;;; Revision 1.2  1993/06/13  23:25:46  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;

(tca::defstruct_tca (BBsample)
  (object "" :type  string)
  (slot "" :type string)
  (value nil))

(defun ModifBBHnd (ref message)
  (format t "ModifBBHnd: start~%")

  (format t "Message from interface: ~s ~s ~a~%" 
	  (BBsample-object message)
	  (BBsample-slot message)
	  (BBsample-value message))

  (let ((resourceref (tca::tcaReserveResource "Module INTERFACE")))
    (send-data-to-interface)
    (tca::tcaCancelReservation resourceref))

  (tca::tcaSuccess ref)

  (format t "ModifBBHnd: done.~%"))

(defun ModifBBConstHnd (ref message)
  (format t "ModifBBConstHnd: start~%")

  (format t "Message from interface: ~s ~s ~a~%" 
	  (BBsample-object message)
	  (BBsample-slot message)
	  (BBsample-value message))

;  (let ((resourceref (tca::tcaReserveResource "Module INTERFACE")))
;  (send-data-to-interface)
  (send-const-to-interface)
;    (tca::tcaCancelReservation resourceref))
  
  (format t "ModifBBConstHnd: done.~%"))

(defun ModifBBQueryHnd (ref message)
  (format t "ModifBBQueryHnd: start~%")

  (format t "Query from interface: ~s ~s ~a~%" 
	  (BBsample-object message)
	  (BBsample-slot message)
	  (BBsample-value message))
  
  (let ((resourceref (tca::tcaReserveResource "Module INTERFACE")))
    (send-data-to-interface)
    (let ((replydata 1))
      (tca::tcaReply ref replydata))
    (tca::tcaCancelReservation resourceref))
  
  (format t "ModifBBQueryHnd: done.~%"))

(defun init-connect-bb (&optional (server (tca::tcaServerMachine)))
  (format t "Connect ...~%")

  (tca::tcaConnectModule "Module BB" server)

  (tca::tcaRegisterCommandMessage "ModifBBMsg" "{string, string, boolean}")
  (tca::tcaRegisterHandler "ModifBBMsg" "ModifBBHnd" 'ModifBBHnd)  

  (tca::tcaRegisterConstraintMessage "ModifBBConst"
				     "{string, string, boolean}")
  (tca::tcaRegisterHandler "ModifBBConst" "ModifBBConstHnd" 'ModifBBConstHnd)  

  (tca::tcaRegisterQueryMessage "ModifBBQuery" "{string, string, boolean}"
				"int")
  (tca::tcaRegisterHandler "ModifBBQuery" "ModifBBQueryHnd" 'ModifBBQueryHnd)  

  (tca::tcaWaitUntilReady)
  )

(defun send-data-to-interface ()
  (let ((message (make-BBsample :object "Lois-TL"
			      :slot "select"
			      :value t)))
    (format t "Message to interface: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaExecuteCommand "ModifInterfaceMsg" message)
    )

  (let ((message (make-BBsample :object "Loi-AB"
			      :slot "select"
			      :value t)))
    (format t "Message to interface: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaExecuteCommand "ModifInterfaceMsg" message)
    )

  (let ((message (make-BBsample :object "Gain-GA"
			      :slot "select"
			      :value t)))
    (format t "Message to interface: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaExecuteCommand "ModifInterfaceMsg" message)
    )
  )

(defun send-const-to-interface ()
  (let ((message (make-BBsample :object "Lois-TL"
			      :slot "select"
			      :value t)))
    (format t "Message to interface: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaAddConstraint "ModifInterfaceConst" message))

  (let ((message (make-BBsample :object "Loi-AB"
			      :slot "select"
			      :value t)))
    (format t "Message to interface: ~s ~s ~a~%" 
	    (BBsample-object message)
	    (BBsample-slot message)
	    (BBsample-value message))
    (tca::tcaAddConstraint "ModifInterfaceConst" message)
    )
  )

(defun get-data-from-interface ()
  (loop (let ((result (tca::tcaHandleMessage 1)))
	  (format t "~a~%" result)
	  (when (eql result TimeOut)
		(return))))
  )

(defun close-connect-bb ()
  (tca::tcaClose))


(defun b4 ()
  (init-connect-bb))
