;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: clispMacros.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;
;;; $Revision: 1.1 $
;;; $Date: 1996/05/07 16:54:16 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: clispLoadMacros.lisp,v $
;;; Revision 1.1  1996/05/07  16:54:16  rich
;;; dded macro files for clisp.
;;;
;;;

(defpackage :Foreign-Function
  (:nicknames "FF")
  (:export defforeign convert-to-lang defun-c-callable register-value
	   register-function *lisp-encode-value*  *lisp-decode-value*
	   ;; Rest are only exported for debugging.
	   formatType formatArgs extractLispArgs typeConvertArgs
	   )
  )


(in-package "FF" :USE '("FFI" "USER" "LISP" ))

;;; Try creating macros to allow clisp to use the allegro foreign function
;;; definitions that we already have.

;;(ff:defforeign 'returnLispFlagGlobal
;;  :entry-point (ff::convert-to-lang "returnLispFlagGlobal")
;;  :arguments '(integer integer)
;;  :return-type :integer)

(defun formatType (type)
  (case type
    ((character :character) 'character)
    ((fixnum :fixnum) 'uint32)
    ((integer :integer) 'uint32)
    ((double-float :double-float) 'double-float)
    ((string :string) 'c-string)
    ((unsigned-long :unsigned-long tca::unsigned-long) 'uint32)
    ;;((lisp :lisp) 'c-pointer)
    ((lisp :lisp) 'uint32)
    ((array :array) 'c-pointer)
    ((void :void) nil)
    (otherwise 
     type)
    )
  )

(defun formatArgs (args &optional (count 1))
  (if (null args)
      nil
    (append (list
	     (list (intern 
		    (concatenate 'string "parameter" (format nil "~A" count)))
		   (formatType (first args))))
	    (formatArgs (rest args) (1+ count)))
    ))

;; This is the version needed when loading the function to use.
(defmacro defforeign (name &key entry-point arguments return-type)
  (eval
   `(DEF-CALL-OUT ,(eval name)
      (:name ,(eval entry-point))
      (:arguments ,@(formatArgs (eval arguments)))
      (:return-type ,(formatType (eval return-type)))
      (:language :STDC)
      )
   )
  nil
  )

(defmacro convert-to-lang (args)
  `,args
  )

(defmacro register-function (args)
  `,args
  )

(defun extractLispArgs (args)
  (if (null args)
      nil
    (append (list (first (first args)))
	    (extractLispArgs (rest args)))
    ))

(defun typeConvertArgs (args)
  (if (null args)
      nil
    (append (list (list (first (first args)) 
			(formatType (second (first args)))))
	    (typeConvertArgs (rest args)))
    ))

(defmacro defun-c-callable (name args &body code)
  `(and 
    (defun ,name ,(extractLispArgs args) ,@code)
    (DEF-CALL-IN ,name 
      (:arguments ,@(typeConvertArgs args))
      (:return-type int)
      (:language :STDC)
      )
    )
  )

(defvar *lisp-encode-value* nil)
(defvar *lisp-decode-value* nil)

(defun register-value (value offset)
  (declare (ignore offset))
  (setf *lisp-encode-value* value)
  (values nil *lisp-encode-value*)
  )

(defmacro defstruct_TCA ((name &rest args) &body fields)
  (declare (ignore args))
  (eval
   `(defstruct (,name ),@fields)
  ))

;;(trace ff:formatType ff:formatArgs ff:extractlispargs ff:typeconvertargs ffi:def-call-out ffi:def-call-in ff:defforeign ffi::note-c-fun)
;;(trace defstruct defstruct_tca)

;;(trace ff:register-value)
