;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: map.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;
;;; $Revision: 1.6 $
;;; $Date: 1996/05/07 16:47:45 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: map.lisp,v $
;;; Revision 1.6  1996/05/07  16:47:45  rich
;;; Changes for clisp.
;;;
;;; Revision 1.5  1995/08/05  17:20:52  reids
;;; Brought the LISP version into sync with the C version (pretty much);
;;; Added some more of the "matrix" functions to the LISP side;
;;; Fixed a bug in sending a null string.
;;;
;;; Revision 1.4  1995/01/24  23:59:17  rich
;;; Release of tca 7.9.  Mostly speed improvements.
;;; The cvs binaries may now be located in /usr/local.
;;; Reformatting to less than 80 character lines.
;;;
;;; Revision 1.3  1994/10/13  03:20:37  rich
;;; Changes needed for Allegro V4.2 on sunOS machines.
;;;
;;; Revision 1.2  1993/06/13  23:25:57  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;

#-(OR CLTL2 CMU CLISP) (in-package :USER :USE '(:TASK-CONTROL))
#-(OR CLTL2 CMU CLISP)
(export '(make-tca_map tca_map-lb1 tca_map-ub1 tca_map-lb2 tca_map-ub2
		       tca_map-el aref-map newcmat newimat newfmat newdmat))
#+(OR CLTL2 CMU CLISP) (in-package :TC)
#+ALLEGRO-V4.1 (setf (excl:package-definition-lock
		      (find-package :COMMON-LISP)) NIL)

(defstruct_TCA (TCA_MAP)
  (lb1 0 :type integer)
  (ub1 0 :type integer)
  (lb2 0 :type integer)
  (ub2 0 :type integer)
  (el NIL :type (or array NULL)))


(defun newmat (lb1 ub1 lb2 ub2 type)
  (let ((row_length (1+ (- ub1 lb1)))
	(col_length (1+ (- ub2 lb2))))
    (make-tca_map :LB1 lb1 :UB1 ub1 :LB2 lb2 :UB2 ub2
	      :EL (make-array (list row_length col_length) 
			      :ELEMENT-TYPE type))))

(defun newcmat (lb1 ub1 lb2 ub2)  (newmat lb1 ub1 lb2 ub2 'CHARACTER))
(defun newimat (lb1 ub1 lb2 ub2)  (newmat lb1 ub1 lb2 ub2 'INTEGER))
(defun newfmat (lb1 ub1 lb2 ub2)  (newmat lb1 ub1 lb2 ub2 'SINGLE-FLOAT))
(defun newdmat (lb1 ub1 lb2 ub2)  (newmat lb1 ub1 lb2 ub2 'DOUBLE-FLOAT))

(defun aref-map (map i j)
  (when (and (>= i (Tca_Map-Lb1 map)) (<= i (Tca_Map-Ub1 map))
	     (>= j (Tca_Map-Lb2 map)) (<= j (Tca_Map-Ub2 map)))
     (aref (tca_map-el map) (- i (Tca_Map-Lb1 map)) (- j (Tca_Map-Lb2 map)))))

(defsetf aref-map (map i j) (store-variable)
  `(when (and (>= ,i (Tca_Map-Lb1 ,map)) (<= ,i (Tca_Map-Ub1 ,map))
	      (>= ,j (Tca_Map-Lb2 ,map)) (<= ,j (Tca_Map-Ub2 ,map)))
      (setf (aref (tca_map-el ,map) (- ,i (Tca_Map-Lb1 ,map))
		  (- ,j (Tca_Map-Lb2 ,map)))
	    ,store-variable)))

;; (declaim (ftype (function (string map) NULL) map_print))
(defun map_print (message map)

  (format T "~s~%Size: [~d:~d][~d:~d]~%" message 
	  (tca_map-lb1 map) (tca_map-ub1 map) (tca_map-lb2 map) (tca_map-ub2 map) )
  (do ((i (tca_map-lb1 map) (1+ i)))
      ((> i (tca_map-ub1 map)))
    (do ((j (tca_map-lb2 map) (1+ j)))
	((> j (tca_map-ub2 map)))
      (format T " ~6,2f" (aref-map map i j)))
    (terpri)))
