;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: .lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;/******************************************************************
;;; * Duplicate the task tree of Figure 2 in the tca manual
;;; ****************************************************************/
;;;
;;; $Revision: 1.3 $
;;; $Date: 1995/01/24 23:59:19 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: mt.lisp,v $
;;; Revision 1.3  1995/01/24  23:59:19  rich
;;; Release of tca 7.9.  Mostly speed improvements.
;;; The cvs binaries may now be located in /usr/local.
;;; Reformatting to less than 80 character lines.
;;;
;;; Revision 1.2  1993/06/13  23:25:59  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;

;;;#include "stdio.h"
;;;#include "tca.h"
;;;/*
;;;#include <datamsg.h>
;;;#include <dispatch.h>
;;;*/

(defun print_ref (msg ref)
 (format t "~a" msg)
 (cond 
  ((NULL ref) (format t ": NULL REFERENCE~%"))
  ((tca:tcaIsSameReference ref tca:tcaRootNodeGlobal)
   (format t ": The Root Node~%"))
  (t (format t ": ~a~%" (tca:tcaReferenceName ref)))))

(defun g1Hnd (ref data)
 (format t "~%Handling G1~%")
 (incf data)
 (tca:tcaExpandGoal "G2" data)
 (incf data)
 (tca:tcaExpandGoal "G3" data)
 (tca:tcaSuccess ref))

(defun g2Hnd (ref data)
 (format t "~%Handling G2~%")
 (incf data)
 (tca:tcaExpandGoal "G4" data)
 (incf data)
 (tca:tcaExecuteCommand "C2" data)
 (tca:tcaSuccess ref))

(defun g3Hnd (ref data)
 (format t "~%Handling G3~%")
 (let* ((c3Ref (tca:tcaCreateReference "C3"))
	(c4Ref (tca:tcaCreateReference "C4"))
	(pm1Ref (tca:tcaCreateReference "PM1"))
	(doubleData NIL))

;;;	(pm1Ref (tca:tcaCreateReference "A1"))
	 
  (tca:TPL_ORDER (tca:tcaAchievementOf c3Ref) (tca:tcaAchievementOf pm1Ref))
  (tca:TPL_ORDER (tca:tcaAchievementOf c4Ref) (tca:tcaAchievementOf pm1Ref))
  (incf data)
  (tca:tcaExecuteCommandWithConstraints c3Ref "C3" data 0)  
  (incf data)
  (tca:tcaExecuteCommandWithConstraints c4Ref "C4" data 0)
  (incf data)
;;;  (setq doubleData data)
  (tca:tcaPointMonitorWithConstraints pm1Ref "PM1" data 0))
;;;/*  tca:tcaExecuteCommandWithConstraints(pm1Ref, "A1", data, 0);*/

  (tca:tcaSuccess ref))

(defun g4Hnd (ref data)
 (format t "~%Handling G4~%")
 (incf data)
 (tca:tcaExecuteCommand "C1" data)
 (tca:tcaSuccess ref))

(defun c1Hnd (ref data)
 (format t "~%Handling C1~%")
 (print_ref "Parent Node" (tca:tcaFindParentReference ref))
 (print_ref "Top Level Node" (tca:tcaFindTopLevelReference ref))
 (print_ref "Next Child" (tca:tcaFindNextChild ref))
 (print_ref "Previous Child" (tca:tcaFindPreviousChild ref))
 (tca:tcaSuccess ref))

(defun c2Hnd (ref data)
 (format t "~%Handling C2~%")
 (print_ref "Parent Node" (tca:tcaFindParentReference ref))
 (print_ref "Top Level Node" (tca:tcaFindTopLevelReference ref))
 (print_ref "Next Child" (tca:tcaFindNextChild ref))
 (print_ref "Previous Child" (tca:tcaFindPreviousChild ref))
 (tca:tcaSuccess ref))

(defun c3Hnd (ref data)
 (format t "~%Handling C3~%")
 (print_ref "Parent Node" (tca:tcaFindParentReference ref))
 (print_ref "Top Level Node" (tca:tcaFindTopLevelReference ref))
 (print_ref "Next Child" (tca:tcaFindNextChild ref))
 (print_ref "Previous Child" (tca:tcaFindPreviousChild ref))
 (tca:tcaSuccess ref))

(defun c4Hnd (ref data)
 (format t "~%Handling C4~%")
 (let ((parentRef (tca:tcaFindParentReference ref)))
  (print_ref "Parent Node" parentRef)
  (print_ref "Top Level Node" (tca:tcaFindTopLevelReference ref))
  (print_ref "Next Child" (tca:tcaFindNextChild ref))
  (print_ref "Previous Child" (tca:tcaFindPreviousChild ref))
  (print_ref "First Child" (tca:tcaFindFirstChild ref))
  (print_ref "Last Child" (tca:tcaFindLastChild ref))
  (print_ref "First Child of Parent" (tca:tcaFindFirstChild parentRef))
  (print_ref "Last Child of Parent" (tca:tcaFindLastChild parentRef))

  (print_ref "C3 Child of parent" (tca:tcaFindChildByName parentRef "c3"))
  (print_ref "PM1 Child of parent" (tca:tcaFindChildByName parentRef "pm1"))
  (print_ref "C5 Child of parent" (tca:tcaFindChildByName parentRef "c5"))
;;;/*
;;;  refData = tcaReferenceData(parentRef);
;;;  printf("Data of parent: ");
;;;  if (refData) 
;;;    Print_Data("int", refData);
;;;  else
;;;    printf("No Data");
;;;  printf("\n");
;;;
;;;  refData = tcaReferenceData(tcaFindChildByName(parentRef, "pm1"));
;;;  printf("Data of PM1: ");
;;;  if (refData) 
;;;    Print_Data("int", refData);
;;;  else
;;;    printf("No Data");
;;;  printf("\n");
;;;*/

  (tca:tcaSuccess ref)))

(defun q1Hnd (ref data)
 (format t "~%Handling Q1~%")
 (tca:tcaReply ref data))

(defun a1Hnd (ref data)
 (let ((monitorRef NIL) (CRef NIL))

  (format t "~%Handling A1~%")

  (print_ref "Parent Node" (tca:tcaFindParentReference ref))
  (print_ref "Top Level Node" (tca:tcaFindTopLevelReference ref))

  (setq monitorRef (tca:tcaFindParentReference ref))
       
  (print_ref "Next Child of Monitor" (tca:tcaFindNextChild monitorRef))
  (print_ref "Previous Child of Monitor" (tca:tcaFindPreviousChild monitorRef))

;;;/*
;;;  print_ref("Next Child", tcaFindNextChild(ref));
;;;  print_ref("Previous Child", tcaFindPreviousChild(ref));
;;;*/

  (tca:tcaDisplayTaskTree (tca:tcaFindTopLevelReference ref))
  (tca:tcaDisplayTaskTree tca:tcaRootNodeGlobal)
  (tca:tcaKillSubTaskTree tca:tcaRootNodeGlobal)

  (setq CRef (tca:tcaAddChildReference tca:tcaRootNodeGlobal "c1"))
  
;;;  intData = (int)*data;

  (tca:tcaExecuteCommandWithConstraints CRef "c1" data SEQ_ACH)

  (tca:tcaDisplayTaskTree tca:tcaRootNodeGlobal)
;;;/*  tcaKillSubTaskTree(tcaRootNodeGlobal);*/

  (tca:tcaSuccess ref)))


(defun mt ()

  (setq one 1)

 (format t "Connect ...~%")

 (tca:tcaConnectModule "manTree" (tca:tcaServerMachine))

 (tca:tcaRegisterGoalMessage "G1" "int")
 (tca:tcaRegisterHandler "G1" "G1" 'g1Hnd)

 (tca:tcaRegisterGoalMessage "G2" "int")
 (tca:tcaRegisterHandler "G2" "G2" 'g2Hnd)

 (tca:tcaRegisterGoalMessage "G3" "int")
 (tca:tcaRegisterHandler "G3" "G3" 'g3Hnd)

 (tca:tcaRegisterGoalMessage "G4" "int")
 (tca:tcaRegisterHandler "G4" "G4" 'g4Hnd)
 
 (tca:tcaRegisterCommandMessage "C1" "int")
 (tca:tcaRegisterHandler "C1" "C1" 'c1Hnd)

 (tca:tcaRegisterCommandMessage "C2" "int")
 (tca:tcaRegisterHandler "C2" "C2" 'c2Hnd)

 (tca:tcaRegisterCommandMessage "C3" "int")
 (tca:tcaRegisterHandler "C3" "C3" 'c3Hnd)

 (tca:tcaRegisterCommandMessage "C4" "int")
 (tca:tcaRegisterHandler "C4" "C4" 'c4Hnd)

 (tca:tcaRegisterQueryMessage "Q1" "int" "int")
 (tca:tcaRegisterHandler "Q1" "Q1" 'q1Hnd)

 (tca:tcaRegisterCommandMessage "A1" "int")
 (tca:tcaRegisterHandler "A1" "A1" 'a1Hnd)

 (tca:tcaRegisterPointMonitor "PM1" "Q1" "A1")

 (tca:tcaWaitUntilReady)

 (tca:tcaExpandGoal "G1" one)

 (tca:tcaModuleListen))
