;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: tca-compile.lisp
;;;
;;; ABSTRACT:
;;; 
;;; Task Control Architecture - Lisp Compile File
;;;
;;; $Revision: 1.3 $
;;; $Date: 1996/05/07 16:47:50 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: tca-compile.lisp,v $
;;; Revision 1.3  1996/05/07  16:47:50  rich
;;; Changes for clisp.
;;;
;;; Revision 1.2  1995/07/08  18:24:06  rich
;;; Change all /afs/cs to /afs/cs.cmu.edu to get ride of conflict problems.
;;;
;;; Revision 1.1  1994/03/27  22:48:55  rich
;;; Loads are not done relative to the tca.lisp file and not with a
;;; hard coded pathname.
;;; The tca-compile file can be used to compile the list files.  Just load it.
;;;
;;;

;;; Get the tca root directory.

(defun parentDir (dir)
  (subseq dir 
	  0
	  (1+ (position #\/ dir
			:from-end T :end (1- (length dir))))))

(defparameter *tcaDir* nil
  "The tca root directory")


(eval-when (eval load compile)
	   (setf *tcaDir* 				 
		 (parentDir (namestring *load-truename*)))
	   )
(defun loadCompOrLisp (file directory)
  "
  If necessary, compile file and quietly load the compile version.
  "
  (let ((lfile (concatenate 'string "/" directory file ".lisp"))
	(cfile (concatenate 'string "/" directory file 
			    #+sun4 ".sparcl"
			    #+sun3 ".fasl"
			    #+DEC3100 ".pmaxl"
			    #+CLISP ".fas"
			    ))
	)
    (cond ((or (not (probe-file cfile))
	       (> (file-write-date lfile)
		  (file-write-date cfile)))
	   (load lfile))
	  (t
	   (load cfile))
	  )))

(defmacro tcaLoadFile (filename)
  (loadCompOrLisp filename 
		  *tcaDir*
		  ))

;;; Load the lisp version of the code.
(tcaLoadFile "tca")

;;; compile the relevant files.
(compile-file (concatenate 'string *tcaDir* "tca.lisp"))
(compile-file (concatenate 'string *tcaDir* "formatters.lisp"))
(compile-file (concatenate 'string *tcaDir* "map.lisp"))
(compile-file (concatenate 'string *tcaDir* "primFmttrs.lisp"))
#+CLISP (compile-file (concatenate 'string *tcaDir* "clispMacros.lisp"))
(compile-file (concatenate 'string *tcaDir* "tcaForeignCalls.lisp"))

;;; Load the compiled version of the code.
(tcaLoadFile "tca")
 
;;; recompile the relevant files.
(compile-file (concatenate 'string *tcaDir* "tca.lisp"))
(compile-file (concatenate 'string *tcaDir* "formatters.lisp"))
(compile-file (concatenate 'string *tcaDir* "map.lisp"))
(compile-file (concatenate 'string *tcaDir* "primFmttrs.lisp"))
#+CLISP (compile-file (concatenate 'string *tcaDir* "clispMacros.lisp"))
(compile-file (concatenate 'string *tcaDir* "tcaForeignCalls.lisp"))
