;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: tcaForeignCalls.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Task Control Architecture
;;;
;;; $Revision: 1.10 $
;;; $Date: 1996/05/07 16:47:55 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: tcaForeignCalls.lisp,v $
;;; Revision 1.10  1996/05/07  16:47:55  rich
;;; Changes for clisp.
;;;
;;; Revision 1.9  1995/10/29  18:25:37  rich
;;; Initial creation of 8.3.
;;;
;;; Revision 1.8  1995/10/25  22:43:52  rich
;;; Maintain the original definition of _tcaRegisterHandler and add a new
;;; function _tcaRegisterHandlerL.
;;;
;;; Revision 1.7  1995/10/17  17:31:02  reids
;;; Added a "language" flag to tcaRegisterHandler, to tell it whether the
;;; handler is C or LISP (or ...)
;;;
;;; Revision 1.6  1995/10/07  19:05:38  rich
;;; Pre-alpha release of tca-8.2.
;;; Added PROJECT_DIR. Added tcaWillListen.
;;;
;;; Revision 1.5  1995/08/08  05:42:53  rich
;;; Fixed broadcast message interface in lisp.  Added test routines
;;; broadcast.lisp and listener.lisp.
;;;
;;; Revision 1.4  1995/07/24  16:29:15  reids
;;; Fix some bugs in STR_Trans, and bring the LISP version up to TCA V8.0
;;;
;;; Revision 1.3  1994/10/13  03:20:46  rich
;;; Changes needed for Allegro V4.2 on sunOS machines.
;;;
;;; Revision 1.2  1993/06/13  23:26:12  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;
;;; 25-Mar-91 Christopher Fedor, School of Computer Science, CMU
;;; Temporarily save value from ff:register-value call to allow
;;; recursive calls to tca functions. This is sometimes triggered
;;; by msg info queries.
;;;
;;; 25-Mar-91 Christopher Fedor, School of Computer Science, CMU
;;; Created tca lisp for version 5.x
;;;

#+CLISP (in-package "USER" :USE '("FFI" "LISP"))
#+CLISP 
(ff::defun-c-callable lispDecode ((format :unsigned-long) 
				  (buffer :unsigned-long))
		      (setf ff:*lisp-decode-value*
			    (cond ((eql format 0) NIL)
				  (T (tca::DecodeData format buffer))))
		      0)

#+CLISP 
(ff::defun-c-callable lispEncode ((format :unsigned-long)
				  (data :unsigned-long)
				  (buffer :unsigned-long))
   (tca::EncodeData format data buffer))


#+CLISP 
(ff::defun-c-callable lispBufferSize ((msgTotal uint32)
				      (format uint32)
				      (data uint32))
		      (declare (ignore data))
		      (let ((x (tca::BufferSize format 
						ff:*lisp-encode-value*
						)))
			(tca::setIntValue msgTotal x)
			x))

#+CLISP 
(ff:defun-c-callable lispReply ((data uint32))
  (setq tca::*lisp-reply-value* data))

#+CLISP 
(ff::defun-c-callable lispExit ()
  (progn 
    (format T "~% Returning to lisp top level \n")
    (abort)
    )
  )

#+CLISP
(def-c-struct tcaRef
  (refId long)
  (msg (c-ptr char))
  (name c-string)
  )

#+CLISP 
(ff::defun-c-callable lispDispatch ((hndName :string) 
				    ;;(ref  (c-ptr tcaRef))
				    (ref  :unsigned-long)
				    (buffer :unsigned-long))
		      (let ((hnd (gethash hndName tca::*tcaHandlers*)))
			(format t "~% Calling ~A ~A " hndName hnd)
			(cond (hnd (funcall hnd ref ff:*lisp-decode-value*))
			      (t (format t "Handler ~A not found" hndName)
				 (abort))
			      ))
		      0
		      )

#-(OR CLTL2 CMU CLISP) (in-package "TASK-CONTROL" :nicknames '("TC"))
#+(OR CLTL2 CMU CLISP) (in-package "TC")

(ff:defforeign 'returnLispFlagGlobal
  :entry-point (ff::convert-to-lang "returnLispFlagGlobal")
  :arguments '()
  :return-type :integer)

#+ALLEGRO
(ff:defforeign 'setIntValue
  :entry-point (ff::convert-to-lang "setIntValue")
  :arguments '(integer integer)
  :return-type :void)

#+CLISP
(ff:defforeign 'setIntValue
  :entry-point (ff::convert-to-lang "setIntValue")
  :arguments '(integer fixnum)
  :return-type :void)

(ff:defforeign 'tcaSetSizeEncodeDecode
  :entry-point (ff::convert-to-lang "tcaSetSizeEncodeDecode")
  :arguments '(integer integer integer integer integer)
  :return-type :void)

(defparameter *LISP-FLAG-GLOBAL* 0)

;;; if dataTotal is 0 then return null as well - have c code set format to 0
#+ALLEGRO 
(ff::defun-c-callable lispDecode ((format :unsigned-long) 
				  (buffer :unsigned-long))
  (cond ((eql format 0) NIL)
	(T (DecodeData format buffer))))

#+ALLEGRO 
(ff::defun-c-callable lispEncode ((format :unsigned-long)
				  (data :lisp)
				  (buffer :unsigned-long))
   (EncodeData format data buffer))

#+ALLEGRO 
(ff::defun-c-callable lispBufferSize ((msgTotal :unsigned-long)
				      (format :unsigned-long)
				      (data :lisp))
  (let ((x (BufferSize format data)))
    (setIntValue msgTotal x)))

(defvar *LISP-REPLY-VALUE* nil)

#+ALLEGRO 
(ff:defun-c-callable lispReply ((data :lisp))
  (setq *LISP-REPLY-VALUE* data))

#+ALLEGRO
(ff::defun-c-callable lispExit ()
  (progn 
    (format T "~% Returning to lisp top level \n")
    (top-level:do-command "reset")
    )
  )

;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; *** Constant and Structure Definitions ***
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defconstant *WAITFOREVER* -1)

;;; *** Communciation Return Values ***

;;;typedef enum {Success, TimeOut, Fail, WrongVersion, MsgUndefined,
;;;	      WrongMsgClass, NullReply} TCA_RETURN_VALUE_TYPE;

;;; needs to be constent with c enums

(defconstant Success       0)
(defconstant TimeOut       1)
(defconstant Failure       2)
(defconstant WrongVersion  3)
(defconstant MsgUndefined  4)
(defconstant WrongMsgClass 5)
(defconstant NullReply     6)

;;; *** Low Level Communication Status Values ***

;;;typedef enum {StatEOF, StatError, StatOK, StatSendEOF, StatSendError,
;;;		StatRecvEOF, StatRecvError} TCA_RETURN_STATUS_TYPE;

(defconstant StatEOF       0)
(defconstant StatError     1)
(defconstant StatOK        2)
(defconstant StatSendEOF   3)
(defconstant StatSendError 4)
(defconstant StatRecvEOF   5)
(defconstant StatRecvError 6)

;;; *** Class Type ***

;;;typedef enum {UNKNOWN, HandlerRegClass, ExecHndClass, QueryClass, 
;;;		GoalClass, CommandClass, ConstraintClass, ExceptionClass,
;;;		PollingMonitorClass, PointMonitorClass, DemonMonitorClass,
;;;		ReplyClass, SuccessClass, FailureClass, 
;;;		FireDemonClass,BroadcastClass, MultiQueryClass}
;;;             TCA_MSG_CLASS_TYPE;

(defconstant UNKNOWN              0)
(defconstant HandlerRegClass      1)
(defconstant ExecHndClass         2)
(defconstant QueryClass           3)
(defconstant GoalClass            4)
(defconstant CommandClass         5)
(defconstant InformClass          6)
(defconstant ConstraintClass      InformClass)
(defconstant ExceptionClass       7)
(defconstant PollingMonitorClass  8)
(defconstant PointMonitorClass    9)
(defconstant DemonMonitorClass   10)
(defconstant ReplyClass          11)
(defconstant SuccessClass        12)
(defconstant FailureClass        13)
(defconstant FireDemonClass      14)
(defconstant BroadcastClass      15)
(defconstant MultiQueryClass      16)

;/****************************************************************
;* The various conditions under which a message can be tapped.
;* See the User's Manual or the file "tca/src/msgTapMon.c" for
;*  the complete semantics of the various conditions.
;*****************************************************************/
;typedef enum {UnknownCondition, WhenSent, BeforeHandling, WhileHandling, 
;		AfterHandled, AfterReplied, AfterSuccess, AfterFailure,
;		BeforeAchieving, WhileAchieving, AfterAchieved, WhenAchieved,
;		BeforePlanning, WhilePlanning, AfterPlanned, WhenPlanned,
;		WhenKilled} TAP_CONDITION_TYPE;

(defconstant UnknownCondition 0)
(defconstant WhenSent         1)
(defconstant BeforeHandling   2)
(defconstant WhileHandling    3)
(defconstant AfterHandled     4)
(defconstant AfterReplied     5)
(defconstant AfterSuccess     6)
(defconstant AfterFailure     7)
(defconstant BeforeAchieving  8)
(defconstant WhileAchieving   9)
(defconstant AfterAchieved   10)
(defconstant WhenAchieved    11)
(defconstant BeforePlanning  12)
(defconstant WhilePlanning   13)
(defconstant AfterPlanned    14)
(defconstant WhenPlanned     15)
(defconstant WhenKilled      16)

;;;typedef struct _TCA_REF *TCA_REF_PTR;

;;;#define NO_TPLCONSTR     0
(defconstant NO_TPLCONSTR 0)

;;; *** End(Achievement(Last_Node)) <= Start(Achievement(Node)) ***
;;;#define SEQ_ACH         01
(defconstant SEQ_ACH 1)

;;; *** End(Planning(Last_Node)) <= Start(Planning(Node)) ***
;;;#define SEQ_PLANNING    02
(defconstant SEQ_PLANNING 2)

;;; *** End(Planning(Node)) <= Start(Achievement(Node)) ***
;;;#define PLAN_FIRST      04
(defconstant PLAN_FIRST 4)

;;; *** Start(Planning(Node)) = Start(Achievement(Node)) ***
;;;#define DELAY_PLANNING 010
(defconstant DELAY_PLANNING 8)


(defconstant NO_REF -1)

;;typedef enum {UnallocatedDispatch, AllocatedDispatch, InactiveDispatch, 
;;		PendingDispatch, AttendingDispatch, HandledDispatch,
;;		KilledDispatch, 
;;		ReplyDispatch, SuccessDispatch, FailureDispatch,
;;		CentralPendingDispatch
;;		} DISPATCH_STATUS_TYPE;

(defconstant UnallocatedDispatch 0 )
(defconstant AllocatedDispatch 1 )
(defconstant InactiveDispatch 2 )
(defconstant PendingDispatch 3 )
(defconstant AttendingDispatch 4 )
(defconstant HandledDispatch 5 )
(defconstant KilledDispatch 6 )
(defconstant ReplyDispatch 7 )
(defconstant SuccessDispatch 8 )
(defconstant FailureDispatch 9 )
(defconstant CentralPendingDispatch 10 )

(defconstant *StatusSymbols*
  (vector  'UnallocatedDispatch 'AllocatedDispatch 'InactiveDispatch 
	   'PendingDispatch 'AttendingDispatch 'HandledDispatch
	   'KilledDispatch 
	   'ReplyDispatch 'SuccessDispatch 'FailureDispatch
	   'CentralPendingDispatch
	   ))


;;; For representing temporal constraints within modules 
;;; (central uses a different representation).

(defconstant NoTime     0)
(defconstant StartPoint 1)
(defconstant EndPoint   2)

(defconstant NoInterval          0)
(defconstant HandlingInterval    1)
(defconstant AchievementInterval 2)
(defconstant PlanningInterval    3)

(defstruct_TCA (TCA_INTERVAL_TYPE)
  (class NoInterval :type integer)
  (msgRef 0 :type integer)
  )

(defstruct_TCA (TCA_TIME_POINT_TYPE)
  (class NoTime :type integer)
  (interval 0 :type TCA_INTERVAL_TYPE))

(defstruct_TCA (TCA_TIME_LIST_TYPE)
  (timePoint 0 :type integer)
  (next NIL))

(defstruct_TCA (TCA_TPL_CONSTRAINT_TYPE)
  (timeP1 NIL)
  (relationship "" :type string)
  (timeP2 NIL))

(defstruct_TCA (INTERVAL_MONITOR_OPTIONS)
  (maxFire 0 :type integer)
  (fireEvery 0 :type integer)
  (duration 0 :type integer)
  (period 0 :type integer)
  (initialWait))

(defstruct_TCA (EXC_INFO_TYPE)
  (ERef 0 :type integer)
  (CRef 0 :type integer)
  (numOfRetries 0 :type integer))

(defconstant UNKNOWN_LANGUAGE 0)
(defconstant C_LANGUAGE       1)
(defconstant LISP_LANGUAGE    2)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; *** Library Functions and Macros ***
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmacro tcaRegisterGoalMessage (name format)
  `(tcaRegisterMessage ,name GoalClass ,format NIL))

(defmacro tcaRegisterCommandMessage (name format)
  `(tcaRegisterMessage ,name CommandClass ,format NIL))

(defmacro tcaRegisterInformMessage (name format)
  `(tcaRegisterMessage ,name InformClass ,format NIL))

(defmacro tcaRegisterConstraintMessage (name format)
  `(tcaRegisterMessage ,name ConstraintClass ,format NIL))

(defmacro tcaRegisterQueryMessage (name msg_format res_format)
  `(tcaRegisterMessage ,name QueryClass ,msg_format ,res_format))

(defmacro tcaRegisterExceptionMessage (name format)
  `(tcaRegisterMessage ,name ExceptionClass ,format NIL))

(defmacro tcaRegisterBroadcastMessage (name format)
  `(tcaRegisterMessage ,name BroadcastClass ,format NIL))

(defmacro tcaRegisterMultiQueryMessage (name msg_format res_format)
  `(tcaRegisterMessage ,name MultiQueryClass ,msg_format ,res_format))

(defmacro VALID_TIME (timePoint)
  `(not (eql (TCA_TIME_POINT_TYPE-class , timePoint) NoTime)))

(defmacro TPL_ORDER(interval1 interval2)
  `(tcaTplConstrain (tcaEndOf ,interval1) "<=" (tcaStartOf ,interval2)))

;;; ************************************************************************

(defun tcaInform (msg data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaInformC msg *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))

(ff:defforeign 'tcaInformC
  :entry-point (ff::convert-to-lang "tcaInform")
  :arguments '(string integer)
  :return-type :integer)

(defmacro tcaAddConstraint (msg data) `(tcaInform ,msg ,data))

;;; ************************************************************************

(ff:defforeign 'tcaQueryC
  :entry-point (ff::convert-to-lang "tcaQuery")
  :arguments '(string integer integer)
  :return-type :integer)

(defmacro tcaQuery (msg query reply)
  `(let ((retVal NIL))
     (multiple-value-bind (index value) (ff:register-value ,query 0)
       (declare (ignore index))	      
       (setq retVal (tcaQueryC ,msg *LISP-FLAG-GLOBAL* *LISP-FLAG-GLOBAL*))
       (if (not (eql retVal NullReply))
	   #+CLISP (setf ,reply ff::*LISP-DECODE-VALUE*)
	   #+ALLEGRO (setf ,reply *LISP-REPLY-VALUE*)
	 )
       (ff:register-value value 0))
     retVal))

;;; ************************************************************************

(ff:defforeign 'tcaReplyC
  :entry-point (ff::convert-to-lang "tcaReply")
  :arguments '(integer integer)
  :return-type :integer)

(defun tcaReply (ref data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaReplyC ref *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))  

(ff:defforeign 'tcaNullReply
  :entry-point (ff::convert-to-lang "tcaNullReply")
  :arguments '(integer)
  :return-type :integer)

;;; ************************************************************************

(ff:defforeign 'tcaEnableDistributedResponses
  :entry-point (ff:convert-to-lang "tcaEnableDistributedResponses")
  :arguments NIL
  :return-type :void)

(ff:defforeign 'tcaDisableDistributedResponses
  :entry-point (ff:convert-to-lang "tcaDisableDistributedResponses")
  :arguments NIL
  :return-type :void)

;;; ************************************************************************

;;;extern TCA_RETURN_VALUE_TYPE tcaQuerySend();
;;;extern TCA_RETURN_VALUE_TYPE tcaQueryReceive();

;;; ************************************************************************

(ff:defforeign 'tcaExpandGoalC
  :entry-point (ff::convert-to-lang "tcaExpandGoal")
  :arguments '(string integer)
  :return-type :integer)

(defun tcaExpandGoal (msg data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaExpandGoalC msg *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))  

;;; ************************************************************************

(ff:defforeign 'tcaExpandGoalWithConstraintsC
  :entry-point (ff::convert-to-lang "tcaExpandGoalWithConstraints")
  :arguments '(integer string integer integer)
  :return-type :integer)

(defun tcaExpandGoalWithConstraints (ref msg data tplConstr)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaExpandGoalWithConstraintsC (or ref 0) msg *LISP-FLAG-GLOBAL* tplConstr)
  (ff:register-value value 0)))  

;;; ************************************************************************

(ff:defforeign 'tcaExecuteCommandC
  :entry-point (ff::convert-to-lang "tcaExecuteCommand")
  :arguments '(string integer)
  :return-type :integer)

(defun tcaExecuteCommand (msg data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaExecuteCommandC msg *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))  

;;; ************************************************************************

(ff:defforeign 'tcaExecuteCommandWithConstraintsC
  :entry-point (ff::convert-to-lang "tcaExecuteCommandWithConstraints")
  :arguments '(integer string integer integer)
  :return-type :integer)

(defun tcaExecuteCommandWithConstraints (ref msg data tplConstr)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaExecuteCommandWithConstraintsC (or ref 0) msg
				     *LISP-FLAG-GLOBAL* tplConstr)
  (ff:register-value value 0)))

;;; ************************************************************************

(ff:defforeign 'tcaWaitForCommandC
  :entry-point (ff::convert-to-lang "tcaWaitForCommand")
  :arguments '(string integer)
  :return-type :integer)

(defun tcaWaitForCommand (msg data)
 (let ((retVal NIL))
  (multiple-value-bind (index value) (ff:register-value data 0)
   (declare (ignore index))	      
   (setq retVal (tcaWaitForCommandC msg *LISP-FLAG-GLOBAL*))
   (ff:register-value value 0))
  retVal))

;;; ************************************************************************

(ff:defforeign 'tcaWaitForCommandWithConstraintsC
  :entry-point (ff::convert-to-lang "tcaWaitForCommandWithConstraints")
  :arguments '(integer string integer integer)
  :return-type :integer)

(defun tcaWaitForCommandWithConstraints (ref msg data tplConstr)
 (let ((retVal NIL))
  (multiple-value-bind (index value) (ff:register-value data 0)
   (declare (ignore index))	      
   (setq retVal (tcaWaitForCommandWithConstraintsC
		 (or ref 0) msg *LISP-FLAG-GLOBAL* tplConstr))
  (ff:register-value value 0))
  retVal))

;;; ************************************************************************

(ff:defforeign 'tcaExecuteC
  :entry-point (ff::convert-to-lang "tcaExecute")
  :arguments '(string integer)
  :return-type :integer)

(defun tcaExecute (msg data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaExecuteC msg *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))  

;;; ************************************************************************

(ff:defforeign 'tcaExecuteWithConstraintsC
  :entry-point (ff::convert-to-lang "tcaExecuteWithConstraints")
  :arguments '(integer string integer integer)
  :return-type :integer)

(defun tcaExecuteWithConstraints (ref msg data tplConstr)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaExecuteWithConstraintsC (or ref 0) msg *LISP-FLAG-GLOBAL* tplConstr)
  (ff:register-value value 0)))

(defmacro tcaExecuteCommandAsync (name data) 
  `(tcaExecuteCommandWithConstraints NIL ,name ,data NO_TPLCONSTR))

(defmacro tcaExecuteAsync (name data) 
  `(tcaExecuteWithConstraints NIL ,name ,data NO_TPLCONSTR))

(defmacro tcaMaybeExecute (name data)
  `(if (tcaMessageHandlerRegistered ,name) (tcaExecute ,name ,data)))

(defmacro tcaMaybeExecuteWithConstraints (ref name data tplConstraints)
  `(if (tcaMessageHandlerRegistered ,name)
       (tcaExecuteWithConstraints ,ref ,name ,data ,tplConstraints)))

(defmacro tcaMaybeExecuteAsync (name data)
  `(if (tcaMessageHandlerRegistered ,name) 
       (tcaExecuteWithConstraints NIL ,name ,data NO_TPLCONSTR)))

;;; ************************************************************************

;; ignore null ref test because it better have a value
(ff:defforeign 'tcaSuccess
  :entry-point (ff::convert-to-lang "tcaSuccess")
  :arguments '(integer)
  :return-type :void)

;;; ************************************************************************

(ff:defforeign 'tcaFailureC
  :entry-point (ff::convert-to-lang "tcaFailure")
  :arguments '(integer string integer)
  :return-type :void)

(defun tcaFailure (ref msg data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaFailureC ref msg *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))  

(defmacro tcaRaiseException (ref msg data) `(tcaFailure ,ref ,msg ,data))  

;;; ************************************************************************

(defun tcaBroadcast (msg data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaBroadcastC msg *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))

(ff:defforeign 'tcaBroadcastC
  :entry-point (ff::convert-to-lang "tcaBroadcast")
  :arguments '(string integer)
  :return-type :integer)

;;; TCA_RETURN_VALUE_TYPE tcaMultiQuery (const char *name, void *query,
;;;                                      int32 max, TCA_REF_PTR *refPtr)
;;; TCA_RETURN_VALUE_TYPE tcaMultiReceive(TCA_REF_PTR ref, void *reply, 
;;;                                       long timeout)

;;;*************************************************************************

(defun tcaServerMachine()
  #+allegro(system:getenv "CENTRALHOST")
  #+clisp ""
  )

(ff:defforeign 'tcaRootNode
  :entry-point (ff::convert-to-lang "returnRootGlobal")
  :arguments '()
  :return-type :integer)

(defun tcaDefaultTime () 
  (make-TCA_TIME_POINT_TYPE :class NoTime 
			    :interval (make-TCA_INTERVAL_TYPE 
				       :class NoInterval
				       :msgRef NO_REF)))

(ff:defforeign 'tcaGetServer
  :entry-point (ff::convert-to-lang "tcaGetServer")
  :arguments '()
  :return-type :integer)

;;; fd_set *tcaGetConnections (void)

;;;************************************************************************

(ff:defforeign 'tcaWaitUntilReady
  :entry-point (ff::convert-to-lang "tcaWaitUntilReady")
  :arguments '()
  :return-type :void)

(ff:defforeign 'tcaWillListen
  :entry-point (ff:convert-to-lang "tcaWillListen")
  :arguments '(integer)
  :return-type :void)

(ff:defforeign 'tcaModuleListen
  :entry-point (ff::convert-to-lang "tcaModuleListen")
  :arguments '()
  :return-type :void)

(ff:defforeign 'tcaHandleMessage
  :entry-point (ff::convert-to-lang "tcaHandleMessage")
  :arguments '(integer)
  :return-type :integer)

;;; void tcaModuleProvides (const char *resourceName, ...)
;;; void tcaModuleProvidesArgv (const char **resourceNames)
;;; void tcaModuleRequires (const char *resourceName, ...)
;;; void tcaModuleRequiresArgv (const char **resourceNames)

;;;************************************************************************

(ff:defforeign 'tcaConnectModuleC
  :entry-point (ff::convert-to-lang "tcaConnectModule")
  :arguments '(string string)
  :return-type :void)

(defun tcaConnectModule (module host)
  (tcaLispInit)
  (tcaConnectModuleC module host)
  (setf *LISP-FLAG-GLOBAL* (returnLispFlagGlobal))
  #+ALLEGRO
  (tcaSetSizeEncodeDecode (ff:register-function 'lispBufferSize)
			  (ff:register-function 'lispEncode)
			  (ff:register-function 'lispDecode)
			  (ff:register-function 'lispReply)
			  (ff:register-function 'lispExit))
  )

(ff:defforeign 'tcaIgnoreLogging
  :entry-point (ff::convert-to-lang "tcaIgnoreLogging")
  :arguments '(string)
  :return-type :void)

(ff:defforeign 'tcaResumeLogging
  :entry-point (ff::convert-to-lang "tcaResumeLogging")
  :arguments '(string)
  :return-type :void)

;;;************************************************************************

(ff:defforeign 'tcaClose
  :entry-point (ff::convert-to-lang "tcaClose")
  :arguments '()
  :return-type :void)

;;;**************************************************************************

(ff:defforeign 'tcaRegisterNamedFormatter
  :entry-point (ff::convert-to-lang "tcaRegisterNamedFormatter")
  :arguments '(string string)
  :return-type :void)

;;;extern void tcaRegisterLengthFormatter();

(ff:defforeign 'tcaRegisterMessageC
  :entry-point (ff::convert-to-lang "tcaRegisterMessage")
  :arguments '(string integer string string)
  :return-type :void)

(defun tcaRegisterMessage (name class msgFormat replyForm)
  (let ((mForm msgFormat) (rForm replyForm))
    (if (null mForm) 
	(setq mForm ""))
    (if (null rForm)
	(setq rForm ""))
    (tcaRegisterMessageC name class mForm rForm)))

(ff:defforeign 'tcaPreloadMessage
  :entry-point (ff::convert-to-lang "tcaPreloadMessage")
  :arguments '(string)
  :return-type :void)

;;;************************************************************************

#+ALLEGRO
(ff:defforeign 'tcaRegisterHandlerC
  :entry-point (ff::convert-to-lang "_tcaRegisterHandlerL")
  :arguments '(string string integer integer)
  :return-type :void)

#+ALLEGRO
(defun tcaRegisterHandler (msgName hndName hnd)
  (let* ((func (gensym hndName)) (f (ff:register-function func)))
    (tcaRegisterHandlerC msgName hndName f LISP_LANGUAGE)
    (eval `(ff::defun-c-callable ,func ((a :signed-long) (b :lisp))
	    (,hnd a b)))))

#+ALLEGRO
(ff:defforeign 'tcaDeregisterHandler
  :entry-point (ff::convert-to-lang "tcaDeregisterHandler")
  :arguments '(string string)
  :return-type :void)

#+CLISP
(defvar *tcaHandlers*
  (make-hash-table :size 17
		   :rehash-size 1 
		   :rehash-threshold 1.0
		   :test #'equal
		   ))

#+CLISP
(ff:defforeign 'tcaRegisterHandlerC
  :entry-point (ff::convert-to-lang "tcaRegisterHandler")
  :arguments '(string string)
  :return-type :void)

#+CLISP
(defun tcaRegisterHandler (msgName hndName hnd)
  (tcaRegisterHandlerC msgName hndName)
  (setf (gethash msgName tca::*tcaHandlers*) hnd)
  )

#+CLISP
(ff:defforeign 'tcaDeregisterHandlerC
  :entry-point (ff::convert-to-lang "tcaDeregisterHandler")
  :arguments '(string string)
  :return-type :void)

#+CLISP
(defun tcaDeregisterHandler (msgName hndName)
  (tcaDeregisterHandlerC msgName hndName)
  (setf (gethash hndName tca::*tcaHandlers*) nil)
  )

;;;************************************************************************

(ff:defforeign 'tcaMessageRegisteredC
  :entry-point (ff::convert-to-lang "tcaMessageRegistered")
  :arguments '(string)
  :return-type :integer)

(defun tcaMessageRegistered (msg)
  (not (zerop (tcaMessageRegisteredC msg))))

;;;************************************************************************

(ff:defforeign 'tcaMessageHandlerRegisteredC
  :entry-point (ff::convert-to-lang "tcaMessageHandlerRegistered")
  :arguments '(string)
  :return-type :integer)

(defun tcaMessageHandlerRegistered (msg)
  (not (zerop (tcaMessageHandlerRegisteredC msg))))

;;; ************************************************************************

(defmacro tcaRegisterGoal (name format hndProc)
  `(progn (tcaRegisterGoalMessage ,name ,format)
	  (tcaRegisterHandler ,name ,name ,hndProc)))

(defmacro tcaRegisterCommand (name format hndProc)
  `(progn (tcaRegisterCommandMessage ,name ,format)
	  (tcaRegisterHandler ,name ,name ,hndProc)))

(defmacro tcaRegisterInform (name format hndProc)
  `(progn (tcaRegisterInformMessage ,name ,format)
	  (tcaRegisterHandler ,name ,name ,hndProc)))

(defmacro tcaRegisterQuery (name format reply hndProc)
  `(progn (tcaRegisterQueryMessage ,name ,format ,reply)
	  (tcaRegisterHandler ,name ,name ,hndProc)))

(defmacro tcaRegisterException (name format hndProc)
  `(progn (tcaRegisterExceptionMessage ,name ,format)
	  (tcaRegisterHandler ,name ,name ,hndProc)))

;;; ************************************************************************

;;; void tcaRegisterExitProc (void (*proc)(void))

;;; ************************************************************************

(ff:defforeign 'tcaRegisterResource
  :entry-point (ff::convert-to-lang "tcaRegisterResource")
  :arguments '(string integer)
  :return-type :void)

(ff:defforeign 'tcaDirectResource
  :entry-point (ff::convert-to-lang "tcaDirectResource")
  :arguments '(string)
  :return-type :void)

(ff:defforeign 'tcaAddHndToResource
  :entry-point (ff::convert-to-lang "tcaAddHndToResource")
  :arguments '(string string)
  :return-type :void)

(ff:defforeign 'tcaLimitPendingResource
  :entry-point (ff::convert-to-lang "tcaLimitPendingResource")
  :arguments '(string integer)
  :return-type :void)

(ff:defforeign 'tcaLimitPendingMessages
  :entry-point (ff::convert-to-lang "tcaLimitPendingMessages")
  :arguments '(string string integer)
  :return-type :void)

;;;*************************************************************************

(ff:defforeign 'tcaReserveResource
  :entry-point (ff::convert-to-lang "tcaReserveResource")
  :arguments '(string)
  :return-type :integer)

(ff:defforeign 'tcaReserveModResource
  :entry-point (ff::convert-to-lang "tcaReserveModResource")
  :arguments '(string string)
  :return-type :integer)

(ff:defforeign 'tcaCancelReservation
  :entry-point (ff::convert-to-lang "tcaCancelReservation")
  :arguments '(integer)
  :return-type :void)

(ff:defforeign 'tcaLockResource
  :entry-point (ff::convert-to-lang "tcaLockResource")
  :arguments '(string)
  :return-type :integer)

(ff:defforeign 'tcaLockModResource
  :entry-point (ff::convert-to-lang "tcaLockModResource")
  :arguments '(string string)
  :return-type :integer)

(ff:defforeign 'tcaUnlockResource
  :entry-point (ff::convert-to-lang "tcaUnlockResource")
  :arguments '(integer)
  :return-type :void)

;;;**************************************************************************
;
;typedef struct { 
;  int maxFire;     /* maximum times to fire action message;
;		      -1 indicates infinite (default is -1) */
;  int fireEvery;  /* fire action message every n times condition is satisfied.
;		     (default is 1) */
;  int duration;    /* maximum time, in seconds, that the monitor can run;
;		      -1 indicates infinite time (default is -1) */
;  int period;      /* time between polls, in seconds */
;  int initialWait; /* if TRUE, don't poll immediately (default is FALSE) */
;} INTERVAL_MONITOR_OPTIONS_TYPE, *INTERVAL_MONITOR_OPTIONS_PTR;
;

(ff:defforeign 'tcaRegisterPointMonitor
  :entry-point (ff::convert-to-lang "tcaRegisterPointMonitor")
  :arguments '(string string string)
  :return-type :void)

(ff:defforeign 'tcaRegisterDemonMonitor
  :entry-point (ff::convert-to-lang "tcaRegisterDemonMonitor")
  :arguments '(string string string string)
  :return-type :void)

(ff:defforeign 'tcaRegisterPollingMonitor
  :entry-point (ff::convert-to-lang "tcaRegisterPollingMonitor")
  :arguments '(string string string)
  :return-type :void)

;;; ************************************************************************

(ff:defforeign 'tcaPointMonitorC
  :entry-point (ff::convert-to-lang "tcaPointMonitor")
  :arguments '(string integer)
  :return-type :integer)

(defun tcaPointMonitor (msg data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaPointMonitorC msg *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))

;;; ************************************************************************

(ff:defforeign 'tcaPointMonitorWithConstraintsC
  :entry-point (ff::convert-to-lang "tcaPointMonitorWithConstraints")
  :arguments '(integer string integer integer)
  :return-type :integer)

(defun tcaPointMonitorWithConstraints (ref msg data tplConstr)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaPointMonitorWithConstraintsC (or ref 0) msg *LISP-FLAG-GLOBAL* tplConstr)
  (ff:register-value value 0)))

;;; ************************************************************************

(defun tcaCreateMonitorOptions ( &optional &key 
					   (maxFire -1) 
					   (fireEvery 1)
					   (duration -1)
					   (period 10)
					   (initialWait NIL))
  (make-INTERVAL_MONITOR_OPTIONS :maxFire maxFire
				 :fireEvery fireEvery
				 :duration duration
				 :period period
				 :initialWait initialWait))

;;;************************************************************************

(ff:defforeign 'tcaIntervalMonitorC
  :entry-point (ff::convert-to-lang "tcaIntervalMonitor")
  :arguments '(string integer)
  :return-type :integer)

(defun tcaIntervalMonitor (name data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaIntervalMonitorC name *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))

;;; ************************************************************************

(ff:defforeign 'tcaIntMonWithConstraintsLISPC
  :entry-point (ff:convert-to-lang "tcaIntMonWithConstraintsLISPC")
  :arguments '(integer string integer integer integer integer integer
	       integer integer integer integer integer integer integer)
  :return-type :integer)

(defun tcaIntervalMonitorWithConstraints (ref name data start expire options)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (let ((refVal ref) 
	(initialWait (INTERVAL_MONITOR_OPTIONS-initialWait options))
	(startInt (if (not (null start)) (TCA_TIME_POINT_TYPE-interval start)))
	(startClass (if (not (null start)) (TCA_TIME_POINT_TYPE-class start)))
	(expireInt (if (not (null expire)) (TCA_TIME_POINT_TYPE-interval expire)))
	(expireClass (if (not (null expire)) (TCA_TIME_POINT_TYPE-class expire))))
    (if (null refVal)
	(setq refVal 0))
    (if (null initialWait)
	(setq initialWait 0)
      (setq initialWait 1))
    (if (null startInt)
	(setq startInt (make-TCA_INTERVAL_TYPE :class NoInterval :msgRef 0)))
    (if (null startClass)
	(setq startClass noTime))
    (if (null expireInt)
	(setq expireInt (make-TCA_INTERVAL_TYPE :class NoInterval :msgRef 0)))
    (if (null expireClass)
	(setq expireClass noTime))
	
    (tcaIntMonWithConstraintsLISPC refVal name *LISP-FLAG-GLOBAL* 
				   startClass
				   (TCA_INTERVAL_TYPE-class startInt)
				   (TCA_INTERVAL_TYPE-msgRef startInt)
				   expireClass
				   (TCA_INTERVAL_TYPE-class expireInt)
				   (TCA_INTERVAL_TYPE-msgRef expireInt)
				   (INTERVAL_MONITOR_OPTIONS-maxFire options)
				   (INTERVAL_MONITOR_OPTIONS-fireEvery options)
				   (INTERVAL_MONITOR_OPTIONS-duration options)
				   (INTERVAL_MONITOR_OPTIONS-period options)
				   initialWait))
    (ff:register-value value 0)))

;;; ************************************************************************

(ff:defforeign 'tcaFireDemonC
  :entry-point (ff::convert-to-lang "tcaFireDemon")
  :arguments '(string integer integer)
  :return-type :integer)

(defun tcaFireDemon (name demonUserId data)
 (multiple-value-bind (index value) (ff:register-value data 0)
  (declare (ignore index))	      
  (tcaFireDemonC name demonUserId *LISP-FLAG-GLOBAL*)
  (ff:register-value value 0)))

;;; ************************************************************************

(ff:defforeign 'tcaIsSameReferenceC
  :entry-point (ff::convert-to-lang "tcaIsSameReference")
  :arguments '(integer integer)
  :return-type :integer)

(defun tcaIsSameReference (ref1 ref2)
  (not (eql (tcaIsSameReferenceC ref1 ref2) 0)))

(ff:defforeign 'tcaReferenceId
  :entry-point (ff::convert-to-lang "tcaReferenceId")
  :arguments '(integer)
  :return-type :integer)

#+ALLEGRO
(ff:defforeign 'tcaRefNameLength
  :entry-point (ff::convert-to-lang "tcaRefNameLength")
  :arguments '(integer)
  :return-type :integer) 

#+ALLEGRO
(ff:defforeign 'tcaRefNameLisp
  :entry-point (ff::convert-to-lang "tcaRefNameLisp")
  :arguments '(integer integer)
  :return-type :lisp) 

#+ALLEGRO
(defvar *LISP-C-GLOBAL-STRING-BUF*)

#+ALLEGRO
(defun tcaReferenceName(ref)
  (let ((length (tcaRefNameLength ref)))
    (cond ((zerop length) NIL)
	  (t (setq *LISP-C-GLOBAL-STRING-BUF* (make-string length))
	     (multiple-value-bind (index value)
		 (ff:register-value '*LISP-C-GLOBAL-STRING-BUF* 0)
	       (declare (ignore index) (ignore value))
	       (tcaRefNameLisp ref length)
	       )))))

#+CLISP
(defun tcaReferenceName (ref)
  (declare (:type (ref tcaRef)))
  (slot-value ref 'name))

(ff:defforeign 'tcaRefDataLisp
  :entry-point (ff::convert-to-lang "tcaRefDataLisp")
  :arguments '(integer)
  :return-type :integer)

(defun tcaReferenceData(ref)
  (if (eql (tcaRefDataLisp ref) 1)
      *LISP-REPLY-VALUE*
    NIL))

(ff:defforeign 'tcaReferenceRelease
  :entry-point (ff::convert-to-lang "tcaReferenceRelease")
  :arguments '(integer)
  :return-type :void)

(ff:defforeign 'tcaReferenceStatusC
  :entry-point (ff::convert-to-lang "tcaReferenceStatus")
  :arguments '(integer)
  :return-type :integer)

(defun tcaReferenceStatus (msgRef)
  (let ((status (tcaReferenceStatusC msgRef)))
    (if (eql status -1)
	NIL
      (aref *StatusSymbols* status))))

(ff:defforeign 'tcaRefFree
  :entry-point (ff::convert-to-lang "tcaRefFree")
  :arguments '(integer)
  :return-type :void)

;;;*************************************************************************

(ff:defforeign 'tcaAddChildReference
  :entry-point (ff::convert-to-lang "tcaAddChildReference")
  :arguments '(integer string)
  :return-type :integer)

(ff:defforeign 'tcaCreateReference
  :entry-point (ff::convert-to-lang "tcaCreateReference")
  :arguments '(string)
  :return-type :integer)

(ff:defforeign 'tcaFindParentReferenceC
  :entry-point (ff::convert-to-lang "tcaFindParentReference")
  :arguments '(integer)
  :return-type :integer)

(defun tcaFindParentReference (cRef)
  (let ((ref (tcaFindParentReferenceC cRef)))
    (if (zerop ref) nil ref)))

(ff:defforeign 'tcaFindFirstChildC
  :entry-point (ff::convert-to-lang "tcaFindFirstChild")
  :arguments '(integer)
  :return-type :integer)

(defun tcaFindFirstChild (cRef)
  (let ((ref (tcaFindFirstChildC cRef)))
    (if (zerop ref) nil ref)))

(ff:defforeign 'tcaFindLastChildC
  :entry-point (ff::convert-to-lang "tcaFindLastChild")
  :arguments '(integer)
  :return-type :integer)

(defun tcaFindLastChild (cRef)
  (let ((ref (tcaFindLastChildC cRef)))
    (if (zerop ref) nil ref)))

(ff:defforeign 'tcaFindNextChildC
  :entry-point (ff::convert-to-lang "tcaFindNextChild")
  :arguments '(integer)
  :return-type :integer)

(defun tcaFindNextChild (cRef)
  (let ((ref (tcaFindNextChildC cRef)))
    (if (zerop ref) nil ref)))

(ff:defforeign 'tcaFindPreviousChildC
  :entry-point (ff::convert-to-lang "tcaFindPreviousChild")
  :arguments '(integer)
  :return-type :integer)

(defun tcaFindPreviousChild (cRef)
  (let ((ref (tcaFindPreviousChildC cRef)))
    (if (zerop ref) nil ref)))

(ff:defforeign 'tcaFindFailedReferenceC
  :entry-point (ff::convert-to-lang "tcaFindFailedReference")
  :arguments '(integer)
  :return-type :integer)

(defun tcaFindFailedReference (cRef)
  (let ((ref (tcaFindFailedReferenceC cRef)))
    (if (zerop ref) nil ref)))

(ff:defforeign 'tcaFindTopLevelReferenceC
  :entry-point (ff::convert-to-lang "tcaFindTopLevelReference")
  :arguments '(integer)
  :return-type :integer)

(defun tcaFindTopLevelReference (cRef)
  (let ((ref (tcaFindTopLevelReferenceC cRef)))
    (if (zerop ref) nil ref)))

(ff:defforeign 'tcaFindAnteReferenceByNameC
  :entry-point (ff::convert-to-lang "tcaFindAnteReferenceByName")
  :arguments '(integer string)
  :return-type :integer)

(defun tcaFindAnteReferenceByName (cRef name)
  (let ((ref (tcaFIndAnteReferenceByNameC cRef name)))
    (if (zerop ref) nil ref)))

(ff:defforeign 'tcaFindChildByNameC
  :entry-point (ff::convert-to-lang "tcaFindChildByName")
  :arguments '(integer string)
  :return-type :integer)

(defun tcaFindChildByName (cRef name)
  (let ((ref (tcaFindChildByNameC cRef name)))
    (if (zerop ref) nil ref)))

;;;************************************************************************

(ff:defforeign 'tcaKillTaskTree
  :entry-point (ff::convert-to-lang "tcaKillTaskTree")
  :arguments '(integer)
  :return-type :void)

(ff:defforeign 'tcaKillSubTaskTree
  :entry-point (ff::convert-to-lang "tcaKillSubTaskTree")
  :arguments '(integer)
  :return-type :void)

(ff:defforeign 'tcaDisplayTaskTree
  :entry-point (ff::convert-to-lang "tcaDisplayTaskTree")
  :arguments '(integer)
  :return-type :void)

;;;************************************************************************

;;; These aren't needed: LISP does its own garbage collection
;extern void tcaFreeData(const char *msgName, void *data);
;extern void tcaFreeReply(const char *msgName, void *data);

;;;************************************************************************

(defun tcaStartOf (interval)
  (make-TCA_TIME_POINT_TYPE :class StartPoint :interval interval))

(defun tcaEndOf (interval)
  (make-TCA_TIME_POINT_TYPE :class EndPoint :interval interval))

(defun tcaHandlingOf (msgRef)
  (let ((mId NIL))
    (cond ((OR (NULL msgRef) (eql msgRef 0)) (setq mId NO_REF))
	  (T (setq mId (tcaReferenceId msgRef))))
    (make-TCA_INTERVAL_TYPE :class HandlingInterval :msgRef mId)))

(defun tcaAchievementOf (msgRef)
  (let ((mId NIL))
    (cond ((OR (NULL msgRef) (eql msgRef 0)) (setq mId NO_REF))
	  (T (setq mId (tcaReferenceId msgRef))))
    (make-TCA_INTERVAL_TYPE :class AchievementInterval :msgRef mId)))

(defun tcaPlanningOf (msgRef)
  (let ((mId NIL))
    (cond ((OR (NULL msgRef) (eql msgRef 0)) (setq mId NO_REF))
	  (T (setq mId (tcaReferenceId msgRef))))
    (make-TCA_INTERVAL_TYPE :class PlanningInterval :msgRef mId)))

(defun tcaTplConstrain (timePoint1 relationship timePoint2)
  (let ((tplConstr (make-TCA_TPL_CONSTRAINT_TYPE :timeP1 timePoint1
						 :relationship relationship
						 :timeP2 timePoint2)))
    (tcaAddConstraint "tplConstrain" tplConstr)))

;;; ************************************************************************

(ff:defforeign 'tcaRetry
  :entry-point (ff::convert-to-lang "tcaRetry")
  :arguments '(integer)
  :return-type :void)

(ff:defforeign 'tcaByPassException
  :entry-point (ff::convert-to-lang "tcaByPassException")
  :arguments '(integer)
  :return-type :void)

(ff:defforeign 'tcaGetExcepC
  :entry-point (ff::convert-to-lang "tcaGetExcepC")
  :arguments '(integer)
  :return-type :integer)

(ff:defforeign 'tcaGetERef
  :entry-point (ff::convert-to-lang "tcaGetERef")
  :arguments '(integer)
  :return-type :integer)

(ff:defforeign 'tcaGetCRef
  :entry-point (ff::convert-to-lang "tcaGetCRef")
  :arguments '(integer)
  :return-type :integer)

(ff:defforeign 'tcaGetRetries
  :entry-point (ff::convert-to-lang "tcaGetRetries")
  :arguments '(integer)
  :return-type :integer)

(defmacro tcaGetExceptionInfo (ref info)
  `(let ((cInfo (tcaGetExcepC ,ref)))
    (setf (EXC_INFO_TYPE-ERef ,info) (tcaGetERef cInfo))
    (setf (EXC_INFO_TYPE-CRef ,info) (tcaGetCRef cInfo))
    (setf (EXC_INFO_TYPE-numOfRetries ,info) (tcaGetRetries cInfo))
    (tcaFreeExcInfo cInfo)))

(ff:defforeign 'tcaAddExceptionHandler
  :entry-point (ff::convert-to-lang "tcaAddExceptionHandler")
  :arguments '(integer string)
  :return-type :void)

(ff:defforeign 'tcaFreeExcInfo
  :entry-point (ff::convert-to-lang "tcaFreeExcInfo")
  :arguments '(integer)
  :return-type :void)

;;; ************************************************************************

;/****************************************************************
;* The listening message is sent with the same data as the tapped message,
;* whenever the indicated condition of the tapped message holds.  The listening
;* message cannot be a query-class message -- it must be the name of a
;* constraint, goal, command, or monitor message.  The constraints on the class
;* of the tapped message depends on the particular condition (see below).  Note
;* that tcaTapMessage applies to all instances of the tapped message.  To tap a
;* particular instance of a message, use tcaTapReference.
;*
;* TCA_RETURN_VALUE_TYPE tcaTapMessage(condition, tappedMsg, listeningMsg)
;*  TAP_CONDITION_TYPE condition; char* tappedMsg, listeningMsg;
;*****************************************************************/

(ff:defforeign 'tcaTapMessage
  :entry-point (ff:convert-to-lang "tcaTapMessage")
  :arguments '(integer string string)
  :return-type :integer)

;/****************************************************************
;*The listening message is sent with the same data as that of the tappedRefPtr,
;* when the indicated condition of the tapped reference holds.  Note that,
;* unlike tcaTapMessage, tcaTapReference applies to just a single message.  The
;* tappedRefPtr must refer to a goal, command, or monitor message.  The
;*listening message must be the name of a constraint, goal, command, or monitor
;*message.  The effects are unpredictable if the condition has already occurred
;* when the tap is initiated (e.g., if a condition of "BeforeHandling" is given
;* when the tappedRefPtr has already been handled).
;*
;* TCA_RETURN_VALUE_TYPE tcaTapReference(condition, tappedRefPtr, listeningMsg)
;*  TAP_CONDITION_TYPE condition; TCA_REF_PTR tappedRefPtr; char* listeningMsg;
;****************************************************************/

(ff:defforeign 'tcaTapReference
  :entry-point (ff:convert-to-lang "tcaTapReference")
  :arguments '(integer integer string)
  :return-type :integer)

;/****************************************************************
;* Return a reference to the message that was tapped by the listeningRefPtr.
;* 
;* TCA_REF_PTR tcaFindTappedReference(listeningRefPtr)
;*   TCA_REF_PTR listeningRefPtr;
;*****************************************************************/

(ff:defforeign 'tcaFindTappedReference
  :entry-point (ff:convert-to-lang "tcaFindTappedReference")
  :arguments '(integer)
  :return-type :integer)

;/****************************************************************
;* Remove the listeningMsg/condition pair from the set of taps on the tapped
;* message.  Undoes the effect of tcaTapMessage.
;*
;* TCA_RETURN_VALUE_TYPE tcaRemoveTap(condition, tappedMsg, listeningMsg)
;*   TAP_CONDITION_TYPE condition; char* tappedMsg, listeningMsg;
;*****************************************************************/

(ff:defforeign 'tcaRemoveTap
  :entry-point (ff:convert-to-lang "tcaRemoveTap")
  :arguments '(integer string string)
  :return-type :integer)

(ff:defforeign 'tcaCleanUpAfterAchieved
  :entry-point (ff:convert-to-lang "tcaCleanUpAfterAchieved")
  :arguments '(string)
  :return-type :integer)

;;; Doesn't make sense for LISP -- no control over memory management
;;; void tcaRegisterFreeMemHnd (void (*func)(unsigned int), int retry)
;;; void tcaRegisterMallocHnd (void *(*func)(), int retry)

(ff:defforeign 'tcaRegisterVar
  :entry-point (ff:convert-to-lang "tcaRegisterVar")
  :arguments '(string string)
  :return-type :void)

;;; tcaSetVar
;;; tcaGetVar
;;; tcaGetSetVar
;;; tcaWatchVar
;;; tcaUnwatchVar
;;; tcaIgnoreVarLogging
;;; tcaFreeVar
;;; tcaGetVersionMajor
;;; tcaGetVersionMinor
;;; tcaSetContext
;;; tcaGetContext
