;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; PROJECT: Carnegie Mellon Planetary Rover Project
;;;          Task Control Architecture 
;;; 
;;; MODULE: tca - lisp
;;;
;;; FILE: timingReport.lisp
;;;
;;; ABSTRACT:
;;; 
;;;  Analyze a message trace and produce a report of timings.
;;;
;;; $Revision: 1.2 $
;;; $Date: 1993/06/13 23:26:23 $
;;; $Author: rich $
;;;
;;; REVISION HISTORY
;;;
;;;  $Log: timingReport.lisp,v $
;;; Revision 1.2  1993/06/13  23:26:23  rich
;;; Added comments at the front of each file.
;;; Fixed prefix for log entries.
;;; Made changes needed for vx works style global variables.
;;;

(defparameter *module-name-alist* '(("GAIT PLANNER" "Gait Planner")
				    ("Footfall Planner" "Footfall")
				    ("l.r.p." "LRP") ("CONTROLLER" "Controller")
				    ("IQM_Perceptron" "IQM")))

(defparameter *ignore-msgs* '("bodyPositionQuery" "positionQuery" "addChildRef"
			      "findParentRef" "findAnteRefByName" "findPrevChild"
			      "findFirstChild" "findNextChild" "referenceData"))

(defun rename-modules (timings)
  (dolist (module (timings-modules timings))
    (let ((new-name (second (assoc (module-name module) *module-name-alist*
				   :TEST #'STRING-EQUAL))))
      (when new-name (setf (module-name module) new-name)))))

(defun timingReport1 (logFile &key displayFile utilizationFile messageFile 
			      date start end includeTopLevel timings)
  (unless timings
    (setq timings (analyze-timings logFile :ignoreMsgs *ignore-msgs*
				   :includeTopLevel includeTopLevel)))
  (rename-modules timings)
  (unless start
    ;; Find first "bodyMoveCommand" to the Controller
    (let ((controller (get-module "Controller" "bodyMoveCommand"
				  (timings-modules timings))))
      (do ((msgs (reverse (module-active controller)) (cdr msgs)))
	  ((or (null msgs) start))
	(when (string-equal (block-name (first msgs)) "bodyMoveCommand")
	  (setq start (time-string (block-start (first msgs))))))))
  (unless end
    ;; Find last "checkPosition" query to the "Gait Monitor" module
    (let ((gait-planner (get-module "Gait Monitor" "checkPosition"
				  (timings-modules timings))))
      (do ((msgs (module-active gait-planner) (cdr msgs)))
	  ((or (null msgs) end))
	(when (string-equal (block-name (first msgs)) "checkPosition")
	  (setq end (time-string (or (block-end (first msgs))
				     (block-start (first msgs)))))))))
  (let ((title (concatenate 'STRING "NASA Demo Run " date)))
    (when displayFile
      (display-timings timings displayFile :TITLE title
		       ;; :ORIENTATION :VERTICAL
		       :SECONDS-PER-TICK 60 :SECONDS-PER-LABEL 300
		       :MODULES-TO-DISPLAY '("Controller" "Gait Planner" 
					     "Gait Monitor" "LRP" "LTM" "ISM")
		       :START start :END end))
    (when utilizationFile 
      (report-utilization timings 
			  :FILE (unless (eql utilizationFile T) utilizationFile)
			  :START start :END end :TITLE title))
    (when messageFile
      (report-messages timings :FILE (unless (eql messageFile T) messageFile)
		       :START start :END end :TITLE title))
    timings))

(defun timingReport (logFile date)
  (timingReport1 logFile
		 :date date
		 :includeTopLevel NIL
		 :displayFile (concatenate 'STRING logFile ".ps")
		 :messageFile (concatenate 'STRING logFile ".msgs")
		 :utilizationFile (concatenate 'STRING logFile ".utl")))
