/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1993 Richard Goodwin & Joseph O'Sullivan. All rights reserved.
 *
 * FILE: xfCallbacks.c
 *
 * ABSTRACT:
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/supervise/inspect_callbacks.c,v $
 * $Revision: 1.7 $
 * $Date: 97/05/26 19:55:48 $
 * $Author: josullvn $
 *
 * REVISION HISTORY:
 *
 * $Log:	inspect_callbacks.c,v $
 * Revision 1.7  97/05/26  19:55:48  josullvn
 * Running inspect without supervise would cause inspect to crash, due to
 * jl moving the syncwithsupervise to early in the mainline. Added a flag
 * x11Running, to protect this condition.
 * 
 * Revision 1.6  1997/05/26 19:36:29  josullvn
 * Fixed filter bug (pressing cancel crashes inspect)
 * Fixed select bug (was only updating select menu once)
 *
 * Revision 1.5  1997/05/20 14:36:58  josullvn
 * When an error messages happens, process buttons now flash red.
 *
 * Revision 1.4  1997/05/20 05:51:38  josullvn
 * chooseCallback was printing to the wrong location
 *
 * Revision 1.3  1997/05/20 05:47:51  josullvn
 * fixed bogus newline under sunOS in browser window, and an error in Simulator.rc
 *
 * Revision 1.2  1997/05/20 05:19:17  josullvn
 * new flag -silent for silent operation, better message when readline fails,
 * cleaned up readlines linktime list, quit in inspect closes xterms properly
 *
 * Revision 1.1  1997/05/18 01:31:55  josullvn
 * Reverted back to the old format for storing the sources, ie
 * not in a directory hierarchy. Also updated the help system in inspect.
 *
 * Revision 1.1  1997/05/13 18:01:24  josullvn
 * Amended nanny and xfMiniRunConsole to supervise and inspect.
 * Numerous bugs fixes, but chiefly...
 *  Updated the xforms display.
 *  respond correctly to CTRL-C's.
 *  new resource file format partially available as -oldload foo.rc
 *  fixed signal handling - a change to devUtils disables prior interrupt handlers.
 * 			Not a good thing methinks...
 *  fixed fd handling. Now superviseCatchChild explicitly handles closed fd's.
 *  reduced message traffic, pings only sent when necessary
 *
 * inspect has been cleaned up, slimmed down. Communication and
 * resource management stuff in common. Still a few TODO things...
 *  Doesn't handle blank key strokes (ie just hitting carriage return).
 *  VT100 compliance is limited. As in, still can't play hangman or do vi, or
 *  anything that uses the secondary graphics window. Don't know why really, look
 *  at.... http://sv1pc44.cs.york.ac.uk/vt100_2.html
 *  Basic curses should be OK
 *
 * Revision 1.4  1997/05/02 16:40:34  robocomp
 * Cleaned up to work on Linux (Joseph & Reid)
 *
 * Revision 1.3  1997/05/01 18:09:38  robocomp
 * Changes (mostly by Joseph) to get nanny/console running on Linux, and
 *   running with version 0.86 of libforms.
 *
 * Revision 1.2  1997/01/21 11:19:24  reids
 * When switching processes, don't quiet a process if there is a script
 *   conditional that depends on output from that process.
 * Don't add -wunitialized without the -o flag.
 *
 * Revision 1.1  96/07/29  05:03:23  josullvn
 * Bloody hell. This commiting is a bit different than under Xavier. Short
 * story is cleaned up some purify bugs, and also made changes to nanny
 * which should make it a bit better - Improving performance over multiple
 * machines, explict quietening of nondisplayed processes, replacing of
 * runConsole with xfMiniConsole, which is multithreaded, vt102 compilant,
 * adds a uniform emacs-like command line editing feature, better on small
 * screens and otherwise fab.
 * 
 * Revision 1.5  1996/07/10  17:19:50  reids
 * Replaced the "Destroy->" menu (which didn't really work right) with a
 *   "Choose->" menu for picking which process to view (Can do "destroy" with
 *   a "choose" and a "kill").
 *
 * Revision 1.4  1996/06/28  14:07:42  reids
 * Fixed quite a few bugs -- with graphics, interaction with script, and
 *   killing processes
 *
 * Revision 1.3  1996/03/29  15:57:43  reids
 * Consolidated the common code between xCallbacks and xfCallbacks, and
 *   xRunConsole and xfRunConsole.
 * Added a way to add macro definitions to resource files ("define: <x> <y>").
 * Fixed a bug that was causing the xfRunConsole to crash when a task was
 *   restarted.
 *
 ****************************************************************************/
#include "tca/libc.h"
#include "tca/devUtils.h"
#include "tca/stdinDev.h"

#include "Common.h"

#include "forms.h"
#include "messages.h"
#include "messageHandle.h"

#include "inspect.h"
#include "inspect_xterm.h"
#include "callbacks.h"
#include "script.h"

extern int x11Running;

/* Forward references */
static void DoProcess(FL_OBJECT *obj, long index);


void inputCallback(int fd, long chars_available)
{
  static char userString[DEFAULT_LINE_LENGTH+1];
  int numRead=0, i;
  
  bzero(userString, DEFAULT_LINE_LENGTH+1);
  numRead = devReadN(fd, userString, 
		     MIN(chars_available,DEFAULT_LINE_LENGTH));

  if ((numRead==2) && userString[0]=='')
    sendBreak();
  else {
    /* Send to supervise... */
    userString[strlen(userString)+1]='\0';
    userString[strlen(userString)]='\n';
    runCommandProcess(consoleProc[currentProc].name, userString);
  }
}
/*
 * DESCRIPTION:
 *
 */
void closeConsoleCallback(FL_OBJECT *obj, long foo)
{
  fl_hide_form(xfConsole->xfMiniConsole);
  close_xterm();
  safeExit();
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void reloadCallback(FL_OBJECT *obj, long foo)
{
  const char *fname, *fname2;
  char *restName;
  static char scriptDirectory[100] = "\0";

  fname = fl_show_file_selector("Resource File To Load", scriptDirectory,
				"*.rc", "Simulator.rc");

  if (fname) {
    /* Update the directory name */
    restName = strrchr(fname, '/');
    if (restName) {
      restName[0] = '\0';
      strcpy(scriptDirectory, fname);
      restName[0] = '/';
    }

    reloadResource(fname);
  }
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void syncCallback(FL_OBJECT *obj, long foo)
{
  syncWithSupervise();
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void filterCallback(FL_OBJECT *obj, long foo)
{
  char *newFilter;

  newFilter = (char *)fl_show_input("Enter string to filter upon:",
				    (filterString != NULL ? filterString : ""));
  if (newFilter == NULL) /* cancel */
    return;

  setFilterString(newFilter);
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void messageClearBufferCallback(FL_OBJECT *obj, long foo)
{
  fl_clear_browser(xfConsole->messageBrowser);
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void killCallback(FL_OBJECT *obj, long foo)
{
  
  if ((currentProc != -1) && 
      fl_show_question("The kill operation can't be undone.\nDo you wish to proceed?", 0))
    killCurrentProcess();
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void restartCallback(FL_OBJECT *obj, long foo)
{
  if ((currentProc != -1) && 
      fl_show_question("The restart operation can't be undone.\nDo you wish to proceed?", 0)) 
    restartCurrentProcess();
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void breakCallback(FL_OBJECT *obj, long foo)
{
  sendBreak();
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void processClearBufferCallback(FL_OBJECT *obj, long foo)
{
  char buffer[256];
  sprintf(buffer, "Clearing... [2J[0;0H");

  EditAppend(buffer, strlen(buffer));
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void runCallback(FL_OBJECT *obj, long foo)
{
  if (!fl_get_menu_maxitems(obj)) {
    getProcessList("Run");
  } else if (fl_get_menu(obj) != -1) {
    runNewProcess((char *)fl_get_menu_text(obj));
  }
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void chooseCallback(FL_OBJECT *obj, long foo)
{
  if (!fl_get_menu_maxitems(obj)) {
    getProcessList("Select");
  } else if (fl_get_menu(obj) != -1) {
    char buffer[256];

    sprintf(buffer, "Choosing %s\n", fl_get_menu_text(obj));

    MessagePrintf(buffer);
    DoProcess(NULL, convertProcessNameToId(fl_get_menu_text(obj)));
  }
}

/*
 * DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void shutdownCallback(FL_OBJECT *obj, long foo)
{
  if( !nConsoleProcs ||
      fl_show_question("You have running processes.\nShutdown will kill them\nDo you wish to proceed?",0)) {
    fl_hide_form(xfConsole->xfMiniConsole);
    close_xterm();
    shutdownProcess();
  }
}

void set_xterm_title(char *name)
{
  char buffer[256];
  sprintf(buffer, "]1;inspect[%s]]2;inspect[%s]",
	  name, name);

  EditAppend(buffer, strlen(buffer));
}

/* 
 * 
 */
void DoProcess(FL_OBJECT *obj, long index)
{
  int i;

  for (i=0; i<nConsoleProcs; i++)
    if (fl_get_button(consoleProc[i].graphics->button)) {
      if (!consoleProc[i].errorReported)
	fl_set_button(consoleProc[i].graphics->button, 0);
      fl_activate_object(consoleProc[i].graphics->button);
    }
     
  for (i=0; i<nConsoleProcs; i++) {
    /* Don't bother quieting it if it is the chosen process */
    /* Keep process verbose if a script is waiting for it's output */
    if (!(i == index || scriptWaitingFor(consoleProc[i].name)))
      sendQuiet(consoleProc[i].name);
  }

  currentProc = index;
  fl_set_object_color(consoleProc[index].graphics->button,FL_COL1,FL_YELLOW);
  fl_set_button(consoleProc[index].graphics->button, 1);
  consoleProc[index].errorReported = 0;

  fl_activate_object(xfConsole->killButton);
  fl_activate_object(xfConsole->restartButton);
  fl_activate_object(xfConsole->breakButton);

  fl_deactivate_object(consoleProc[index].graphics->button);

  set_xterm_title(consoleProc[currentProc].name);
  processClearBufferCallback(NULL, 0);
  fl_freeze_form(xfConsole->xfMiniConsole);

  sendLoud(consoleProc[currentProc].name);
  startFlushing(consoleProc[currentProc].name);
}

void EditAppend(unsigned char *text, int n)
{
  int nbytes, nwritten;
  int ret;

  nbytes = n; nwritten = 0;
  do {
    if ((ret= write(xterm_socket, &(text[nwritten]), nbytes-nwritten)) == -1)
      {
	WARNING2("Failed to write %d bytes", nbytes-nwritten);
	switch (errno) {
	case EINTR:
	case EIO: 
	case EAGAIN:
	  break;
	default:
	  return; /* Ooooh ohhh - probably need to spawn a new xterm.. */
	}
      }
    nwritten += ret;
  } while (nbytes != nwritten);
  if (dumpFd != -1) {
    nbytes = n; nwritten = 0;
    do {
      if ((ret= write(dumpFd, &(text[nwritten]), nbytes-nwritten)) == -1)
	{
	  WARNING2("Failed to write %d bytes", nbytes-nwritten);
	  switch (errno) {
	  case EINTR:
	  case EIO: 
	  case EAGAIN:
	    break;
	  default:
	    return; /* Ooooh ohhh - probably need to spawn a new xterm.. */
	  }
	}
      nwritten += ret;
    } while (nbytes != nwritten);
  }
}

static void addProcessButton1 (int index, const char *name)
{
  FL_OBJECT *obj;

  /* If the size of the console window is changed, update these
     constants accordingly.. */     
  consoleProc[index].graphics->button = 
    obj = fl_add_lightbutton(FL_PUSH_BUTTON, 15+60*index, 60, 60, 25, name);
  fl_set_object_boxtype(obj, FL_SHADOW_BOX);
  fl_set_object_lsize(obj, FL_TINY_SIZE);
  fl_set_object_callback(obj, DoProcess, index);
}

void addProcessButton (const char *name)
{
  fl_addto_form(xfConsole->xfMiniConsole);
  addProcessButton1(nConsoleProcs, name);
  fl_end_form();
}

void destroyProcessButton1 (int index)
{
  fl_hide_object(consoleProc[index].graphics->button);
  fl_delete_object(consoleProc[index].graphics->button);
  fl_free_object(consoleProc[index].graphics->button);
  consoleProc[index].graphics->button = NULL;
}

/*
** need this wrapper in order to update the form - otherwise,
** the button will not actually be updated.
*/ 
void destroyProcessButton (int index)
{
  fl_addto_form(xfConsole->xfMiniConsole);
  destroyProcessButton1(index);
  fl_end_form();
}

/*
** Draw all the process buttons.
** Where index has just been destroyed. 
** Used to just redraw objects from index onwards, but now redraw all menu items.
*/
void resetProcessButtons (int index)
{
  int i;

  fl_addto_form(xfConsole->xfMiniConsole);
  /* This forces the frame to be cleared */
  /* fl_delete_object(xfConsole->processFrame); */
  for (i=0; i<nConsoleProcs; i++) {
    destroyProcessButton1(i);
  }
  /* This forces the frame to be cleared */
  /*fl_add_object(xfConsole->xfMiniConsole, 
		xfConsole->processFrame); */
  for (i=0; i<nConsoleProcs; i++) {
    addProcessButton1(i, consoleProc[i].name);
  }

  attachProgramMenu(); /* redo program menu */

  fl_end_form();

  processClearBufferCallback(NULL, 0);

  fl_deactivate_object(xfConsole->killButton);
  fl_deactivate_object(xfConsole->restartButton);
  fl_deactivate_object(xfConsole->breakButton);
}

BOOLEAN highlightProcess(char *name)
{
  int ntmp;

  /* Figure out which of consoleProcs we wish to reset */
  if ((ntmp = convertProcessNameToId(name)) == -1) return;

  if (!consoleProc[ntmp].errorReported)
  {
    fl_set_object_color(consoleProc[ntmp].graphics->button,FL_COL1,FL_RED);
    fl_set_button(consoleProc[ntmp].graphics->button, 1);
    consoleProc[ntmp].errorReported = 1;
  }
}

/* Lowlevel functions */
/*	Function Name: Feep
 *	Description: feeps the bell.
 *	Arguments: none.
 *	Returns: none.
 */

void Feep(void)
{
  /* XBell(CurDpy, 0); */
}

void WFlush(void)
{
  /* XFlush(CurDpy); */
}
/*
 * DESCRIPTION:
 *  prints string to message browser, which is forced to the bottom...
 * INPUTS:
 *
 * OUTPUTS:
 */
void MessagePrintf (const char * str)
{
  if (x11Running)
    fl_addto_browser_chars(xfConsole->messageBrowser, str);
  else
    fprintf(stderr, str);
}

static void setScrollPosition (void)
{
  fl_set_slider_value(xfConsole->statusSlider, getPosition());
}

static void updateStatus(Pointer call_back, Pointer client_data)
{
  updatePosition();
  setScrollPosition();
  ProcessXFHandler(call_back, client_data);
}

/*
 * DESCRIPTION:
 * Starts our little Knightrider scanner going, with the message "waiting message"
 * 
 */
void activateStatus (const char *message)
{
  if (!x11Running) return;

  /* So that Status isn't hidden by the above update, we write it explicitly. */
  fl_set_object_label(xfConsole->statusLogo, "Status");
  fl_set_object_label(xfConsole->statusLabel, message);
  fl_set_object_label(xfConsole->statusLogo, " Status");

  initPosition();
  setScrollPosition();

  devStartPolling(stdin_device, &pollTime, (Handler) updateStatus, NULL);
}

/* DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void deactivateStatus (const char *message)
{
  if (!x11Running) return;

  /* So that Status isn't hidden by the above update, we write it explicitly. */
  fl_set_object_label(xfConsole->statusLogo, "Status");
  fl_set_object_label(xfConsole->statusLabel, message);
  fl_set_object_label(xfConsole->statusLogo, " Status");

  devStopPolling(stdin_device);
  devStartPolling(stdin_device, &pollTime, (Handler)ProcessXFHandler, NULL);

  initPosition();
  setScrollPosition();
}

/* Dynamic menu handling */
/* DESCRIPTION:
 *
 * INPUTS:
 *
 * OUTPUTS:
 */
void attachProgramMenu(void)
{
  char *buf, program[128];
  int i;

  fl_clear_menu(xfConsole->runMenu);
#if 0
  fl_clear_menu(xfConsole->destroyMenu);
#endif
  fl_clear_menu(xfConsole->chooseMenu);
  buf = messageExtractKey(possiblePrograms, program);
  for (i=0; i<numPossiblePrograms; i++) 
    {
      fl_addto_menu(xfConsole->runMenu, program);
#if 0
      fl_addto_menu(xfConsole->destroyMenu, program);
#endif
      if (convertProcessNameToId(program) != -1) {
	fl_addto_menu(xfConsole->chooseMenu, program);
      }
      buf = messageExtractKey(buf, program);
    }
}

/*****************************************************************************
 ****************************************************************************/

void setSuspendResume(const char *title)
{
   /* reids script stuff. Xforms could do this nicer - but don't know
    * where reid is going with this all.
    */
  fl_clear_menu(xfConsole->scriptMenu);

  fl_addto_menu(xfConsole->scriptMenu, "Run...");
  fl_addto_menu(xfConsole->scriptMenu, "Rerun");
  fl_addto_menu(xfConsole->scriptMenu, title);
  fl_addto_menu(xfConsole->scriptMenu, "Stop");
}

static void rerunScriptCallback (FL_OBJECT *obj, long foo)
{
  rerunScript();
}

/* Used for both suspend and resume */
static void suspendScriptCallback (FL_OBJECT *obj, long index)
{
  suspendResume(fl_get_menu_text(obj));
}

static void runScriptCallback(FL_OBJECT *obj, long index)
{
  const char *fname;
  char *restName;
  static char scriptDirectory[100] = "\0";

  fname = fl_show_file_selector("Script File To Load", scriptDirectory,
				"*.script", "foo.script");
  if (fname) {
    /* Update the directory name */
    restName = strrchr(fname, '/');
    if (restName) {
      restName[0] = '\0';
      strcpy(scriptDirectory, fname);
      restName[0] = '/';
    }

    if (!scriptLoad((char *)fname)) {
      fl_show_alert("File not found or syntax error in parsing", "", "", 1);
    }
  }
}

static void stopScriptCallback (FL_OBJECT *obj, long index)
{
  stopScript();
}

void scriptCallback(FL_OBJECT *obj, long foo)
{
  static char   buf[128];

  if (!fl_get_menu_maxitems(obj))
    {
      /* If we have no items, that means we have no supervise connect yet...  */
      Feep();
      MessagePrintf("[CONSOLE] script menu weirdness...\n");
    }
  else
    {
      if (fl_get_menu(obj) != -1)
	{
	  if (!strcmp(fl_get_menu_text(obj), SUSPEND)) 
	    suspendScriptCallback(obj,foo);
	  else if (!strcmp(fl_get_menu_text(obj), RESUME)) 
	    suspendScriptCallback(obj,foo);
	  else if (!strcmp(fl_get_menu_text(obj), "Run...")) 
	    runScriptCallback(obj,foo);
	  else if (!strcmp(fl_get_menu_text(obj), "Rerun")) 
	    rerunScriptCallback(obj,foo);
	  else if (!strcmp(fl_get_menu_text(obj), "Stop")) 
	    stopScriptCallback(obj,foo);
	  else {
	    MessagePrintf("[CONSOLE] script menu weirdness (2)...\n");
	  }
	}
    }
}

/* 
**
*/
void ProcessXFHandler(Pointer call_back, Pointer client_data)
{
  FL_OBJECT *obj;
  char   buf[DEFAULT_LINE_LENGTH];
  static int ping_timeout=0;

  /* Check that we are still talking... */
  if (!ConnectedSupervise() && ping_timeout++ > 20) {
    ping_timeout = 0;
    syncWithSupervise();
  }

  if (flushing) {
    if (flushing_count++ > 20) {
      flushing=FALSE; flushing_count = 0;
      unfreeze();
    }
  }

  obj = fl_check_forms();
}


void unfreeze (void)
{
  fl_unfreeze_form(xfConsole->xfMiniConsole);
}

struct _CONSOLE_GRAPHICS *createConsoleGraphics (void)
{
  return (CONSOLE_GRAPHICS_PTR)malloc(sizeof(CONSOLE_GRAPHICS_TYPE));
}

void clearConsoleGraphics (consoleProcPtr consoleProc)
{
  bzero(consoleProc->graphics, sizeof(CONSOLE_GRAPHICS_TYPE));
}

void copyConsoleGraphics (consoleProcPtr from, consoleProcPtr to)
{
  *to->graphics = *from->graphics;
}
