/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1993 Richard Goodwin & Joseph O'Sullivan. All rights reserved.
 *
 * FILE: inspect_help.c
 *
 * ABSTRACT:
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/supervise/inspect_help.c,v $
 * $Revision: 1.1 $
 * $Date: 97/05/18 01:31:56 $
 * $Author: josullvn $
 *
 * REVISION HISTORY:
 *
 *
 ****************************************************************************/
#include "tca/libc.h"
#include "tca/devUtils.h"
#include "tca/stdinDev.h"

#include "forms.h"

#include "supervise.h"

/*
** Here is how to format the help display.
**
** Whenever the line starts with the symbol @ the next letter indicates the 
** special characteristic associated with this line. The following 
** possibilities exist at the moment: 
        f               Fixed width font.
        n               Normal (Helvetica) font.
        t               Times-Roman like font.
        b               Boldface. Modifier
        i               Italic. Modifier
        l               Large (new size = FL_LARGE_SIZE).
        m               Medium (new size = FL_MEDIUM_SIZE).
        s               Small (new size = FL_SMALL_SIZE).
        L               Large (current size += 6)
        M               Medium (current size += 4)
        S               Small (current size -= 2).
        c               Centered.
        r               Right aligned.
        _               Draw underlined text.
        -               An engraved separator. Text following - is ignored.
        C               The next number indicates the color index of this line.
        N               Non-selectable line (in selectable browsers).
        @               Regular @ character.
**
*/
#define CREDITS "\n\
@L@b@cInspect\n@-" \
"\n\t\t Released : " SUPERVISE_VERSION_DATE \
"\n\t\t Commited : " SUPERVISE_COMMIT_DATE \
"\n\t\t Compiled : " __DATE__ " "  __TIME__ \
"\n\n\
@L@i  Joseph O'Sullivan \t josullvn@cs.cmu.edu \n\
\n\
@L@i  Rich Goodwin \t\t rich@cs.cmu.edu \n\
\n\
@L@i  Reid Simmons \t\t reids@cs.cmu.edu\n\
\n\
\n\
\tDesigned with XForms c0.86 by T. C. Zhao &  Mark Overmars\n"

#define OVERVIEW "\n\
@L@b@cOverview\n\
@-\n\
\n\
With complex multi person robot projects, the standard run time set up\n\
nonetheless is too reliant upon people being coached in handling the setup\n\
and debugging of subsystem processes. SUPERVISE is a system for extracting\n\
much of this knowledge, codifying it, and then managing the inevitable \n\
pitfalls of multiple processes running in the real world (code failure etc).\n\
\n\
A common philiosophy used in robot projects is to start up multiple xterms,\n\
connected to appropriate machines. For a typical runtime situation, several\n\
such windows were necessary. Most people don't know nor wish to know what each\n\
of these processes did. Instead, you only care about these processes when you\n\
wish to control them, or want to be notified that something has gone\n\
wrong. Monitoring the extraneous windows proves to be distracting and confusing\n\
in practice.\n\
\n\
Supervise/Inspect address these problems, with the idea being to have a single\n\
process on each machine (\"supervise\") that starts up and monitors the processes\n\
needed to run the robot.  The supervise has all the information needed to give\n\
the programs the correct arguments and knows which modules depend on other\n\
programs being run first.  Or course, since the supervise starts these programs\n\
and manages all the IO, you can't directly see what is going on.  That is where\n\
inspect comes in.  The inspect program connects to the supervise server and\n\
allows you to interact with any of the running programs (see the output, type\n\
new input etc), on any of the remote machines.\n\
\n\
Low-level knowledge about individual module has been removed from hands of\n\
the end-users. The responsibility has been placed in the hands of the\n\
original module author to provide resources with correct dependency\n\
information to SUPERVISE, which will then detect when the module is required,\n\
and ensure that it will stay running. \n\
\n\
INSPECT simplifies interaction with a complex system. Instead of having\n\
to monitor extraneous modules, the end-user can now concentrate on a specific\n\
process, with SUPERVISE handling any failures, and in worst cases \n\
highlighting them to the end-user.\n\
\n"

#define SUPERVISE "\n\
@L@b@cSupervise\n\
@-\n\
\n\
look at the documentation for details on supervise\n\
However, don't run inspect without also running supervise.\n\
If you do so, inspect will hog CPU looking for supervise."

#define INSPECT "\n\
@L@b@cInspect\n\
@-\n\
When run, INSPECT displays two windows. The basic console window, and \n\
an xterm. \n\
\n\
@-\n\
@L@b@cConsole Window\n\
@-\n\
The console window can be divided into 4 sections. \n\
\n\
@L@b@iProcess information:\n\
\n\
@bRun:\n\
This will present a Menu list. From this list select a program for SUPERVISE to\n\
run (all dependents processes will automatically be run). \n\
\n\
@bSelect:\n\
This will present a Menu list. From this list select a program for interaction\n\
via INSPECT's xterm. This is intended for situations when there are too many\n\
processes running to fit on the console display.\n\
\n\
@bScript:\n\
Begin executing a script (see Script help).\n\
\n\
@bKill:\n\
Kill the currently selected program. It will not be restarted, until explicitly\n\
called due to Run again.\n\
\n\
@bRestart:\n\
Kill the currently selected program, then restart it.\n\
\n\
@bBreak:  \n\
A CTRL-C signal will be sent to the currently selected process.\n\
\n\
@L@b@iExectuting Processes:\n\n\
The list of currently available processes is next presented. Clicking\n\
one will cause it to be the currently selected process, and will the\n\
text widget will allow for interaction with it. If an error occurs in\n\
a process, the button will turn red until the error is examined.\n\
\n\
@L@b@iMessage information:\n\n\
\n\
@btext widget: \n\
This displays error signals from the running processes, from various \n\
SUPERVISE's (labelled with machine name) and from INSPECT itself. \n\
\n\
@bSTATUS indicator:\n\
This tells how current communication with supervise is going. The scroll bar \n\
will continue moving until a command is acknowledge to have completed\n\
successfully, or failed. \n\
\n\
@bFilter: \n\
Will restrict messages to those containing a string, eg those containing\n\
\"[xavier]\", or maybe more usefully \"Arrgh, out of memory\".\n\
\n\
@bClear Buffer:\n\
Clear current messages in this text widget\n\
\n\
@L@b@iConsole commands:\n\n\
@bHelp:      \n\
You're reading it!\n\
\n\
@bReload:\n\
Not implemented yet - will reload the resource file SUPERVISE is utilizing.\n\
\n\
@bShutdown:    \n\
Shut down all executing processes, and close INSPECT\n\
\n\
@bClose Console:\n\
Close INSPECT, without affecting running programs.\n\
\n\
@-\n\
@L@b@cXterm Window\n\
@-\n\
The xterm window is partially vt100 compliant. Basic curses capabilities\n\
is supported, but no secondary graphics operations. \n\
\n\
The name of the xterm window will change to reflect the currently selected\n\
process. If no process is selected, the name will be \"inspect[waiting...]\".\n\
\n" 

#define SCRIPT  "\n\
@L@b@cBNF\n\
@-\n\
\n\
<script>    ::= \t { <commment> | <statement> }*\n\
<statement> ::= \t { <action> | <conditional> } [<commment>]\n\
<action>    ::= \t { <run-action> | <kill-action> | <send-action> |\n\
	       \t  <suspend-action> | <continue-action> }\n\
<conditional> ::= \t <condition> <consequent>\n\
<consequent>  ::= { <statement> | <block> }\n\
<block>       ::= BEGIN: <statement>* END:\n\
<condition>   ::= { <when-condition> | <at-condition> | <every-condition> }\n\
\n\
<run-action>      ::= RUN: <process>\n\
<kill-action>     ::= KILL: <process>\n\
<suspend-action>  ::= SUSPEND: <process>\n\
<continue-action> ::= CONTINUE: <process>\n\
<send-action>     ::= SEND: <process> <line>\n\
\n\
<when-condition>  ::= { WHEN: <process> <line> | WHENREADY: <process> }\n\
<at-condition>    ::= AT: <time>\n\
<every-condition> ::= { WHENEVERY: <process> <line> : ATEVERY: <time> }\n\
\n\
<time>          ::= { <relative-time> | <absolute-time> }\n\
<relative-time> ::= +[[<hr>:]min]:sec\n\
<absolute-time> ::= [[<month>-]<day>] [[[<hr>:]min]:sec]\n\
\n\
<process> ::= alphanumeric string\n\
<line>    ::= \"double-quoted string of any characters\"\n\
<comment> ::= '#' followed by any characters, up to <cr>\n\
\n\
<hr>    ::= { 0 | 1 | 2 | .. | 23 }\n\
<sec>   ::= <min> ::= { 00 | 01 | 02 | .. | 59 }\n\
<month> ::= { JAN | FEB | .. | DEC }\n\
<day>   ::= { 1 | 2 | .. | 31 }\n\
\n\
@L@b@cSEMANTICS:\n\
@-\n\
Statements can be either active, inactive or completed.  All top-level\n\
statements in a script file are intially active.  Active <action> statements\n\
are executed sequentially, and then marked as \"completed\".  Active\n\
<conditional> statements are checked concurrently.  When the <condition> of a\n\
conditional statement holds (either the time has arrived or the process\n\
outputs the desired line), then the <consequent> of the <conditional> is made\n\
active (for <block>s, all statements within the block are made active).\n\
After this, <when-condition> and <at-condition> statements are marked\n\
\"completed\", while <every-condition> statements remain active.\n\
\n\
A <send-action> sends the line of text to the given process (stdin).\n\
The <run-action>, <kill-action>, <suspend-action> and <continue-action> all do\n\
the obvious things to the given process (it is assumed that nanny correctly\n\
handles the cases where a process that is not running is asked to be\n\
killed/suspended/continued, and where a running process is asked to be run).\n\
\n\
<time> conditions are figured with respect to the time a statement becomes\n\
active.  <absolute-time> conditions hold at the given time.  For \"incomplete\"\n\
times (e.g., giving only min:sec), the time is considered to hold at a\n\
completion of the time. For example, ATEVERY: 15:00 holds at a quarter past\n\
every hour.\n\
\n\
<line>s are surrounded by double-quotes.  To include a double-quote\n\
within a line, precede it with a backslash (\").  To include a backslash,\n\
precede it with another backslash (\\).  <line>s may span multiple lines\n\
(using NEWLINE's or \n's).  See test.d for examples\n\
\n\
Comments are indicated with a \'#\'.  All characters on a line following the\n\
pound sign are ignored.\n\
\n\
@L@b@cEXAMPLES:\n\
@-\n\
@b1. Start up a gdb process:\n\
 RUN: gdbCTR\n\
 WHENREADY: gdbCTR\n\
 SEND: gdbCTR \"r -x -2\"\n\
\n\
@b2. Say \"hi\" to Greg every 30 seconds:\n\
 RUN: CTS\n\
 WHENREADY: CTS\n\
 ATEVERY: +:30\n\
 SEND: CTS \"say Hi Greg\"\n\
\n\
@b3. Wait until noon, then say \"hi\" to Greg every 30 seconds:\n\
 AT: 12:00:00\n\
 BEGIN:\n\
  RUN: CTS\n\
  WHENREADY: CTS\n\
  ATEVERY: +:30\n\
  SEND: CTS \"say Hi Greg\"\n\
 END:\n\
\n\
@b4. Tell central to resend messages, and kill CTR after three bumps.\n\
 RUN: central\n\
 RUN: CTR\n\
 # Note: Don't have to put a \"WHENREADY\" central, because CTR depends on central\n\
 WHENREADY: CTR\n\
 BEGIN:\n\
  WHENEVERY: central \"Resend messages?\"\n\
  SEND: central \"Y\"\n\
\n\
  WHEN: CTR \"BUMP RECOVERY\"\n\
   WHEN: CTR \"BUMP RECOVERY\"\n\
    WHEN: CTR \"BUMP RECOVERY\"\n\
     KILL: CTR                 # Kill only after the third bump\n\
 END:\n\
\n"

#define TODO    "\n\
@L@b@cBugs/Tips\n\
@-\n\
\n\
Currently, doesn\'t handle incorrect resource command lines well. That is, \n\
if you specifiy a program that is not runnable, supervise gets screwed up. \n\
\n\
Doesn\'t handle blank key strokes (ie just hitting carriage return).\n\
\n\
VT100 compliance is limited. As in, still can\'t play hangman or do vi, or \n\
anything that uses the secondary graphics window. Don\'t know why really,\n\
need to look in detail at.... http://sv1pc44.cs.york.ac.uk/vt100_2.html\n\
(Basic curses should be OK)\n\
\n\
Note that if INSPECT acts up, it is quite trivial to shut it, using either \n\
\"Close Console\" or just CTRL-Cing and starting it up again. No data will \n\
be lost, as all data is store on the supervisers. So, this should not alter \n\
running processes but may clear a bad state. If after restarting, you don\'t \n\
have access to any of the running processes, try executing a dummy process \n\
(I frequently have something such as tcsh in my resource file) to nudge \n\
the SUPERVISE processes. \n\
\n\
If all fails and SUPERVISE crashes, do a ps in your system since it may have\n\
left stray processes running. You should be notified of such processes when\n\
you next run SUPERVISE, and running SUPERVISE with the -clean option will\n\
automatically remove them.\n\
However, if they are not removed, they will prevent INSPECT and SUPERVISE\n\
running correctly, since they will lock the sockets.\n\
\n\
The knightrider type notification bar in INSPECT is only semi-implemented,\n\
and so should not be trusted.\n\
\n\
@L@b@cFuture Work\n\
@-\n\
Increasing fault tolerance is possible by providing alternatives/allowing\n\
for process migration. For example, if a process was running upon a machine\n\
which suffered hard disk failure, it would be possible to code a list of\n\
alternative machines to use as backup. More realistically, processes such as\n\
navigate don\'t care where they are run, and could be shoved around for load\n\
balancing purposes.\n\
\n\
The message communication mechanism isn't all it could be, and has a lot of\n\
overhead for each message. \n\
\n\
Report any bugs to josullvn@cs.cmu.edu" 

typedef struct {
  char *name;
  char *body;
} helpFormEntryType;

#define HELP_SECTIONS 6

static helpFormEntryType helpFormEntry[HELP_SECTIONS] = {
  {"Credits", CREDITS},
  {"Overview", OVERVIEW},
  {"Supervise", SUPERVISE},
  {"Inspect", INSPECT},
  {"Scripting", SCRIPT},
  {"Todo", TODO}
};
  
static FL_FORM *helpform;
static FL_OBJECT *helpbr;

static void dismiss_help_form(FL_OBJECT *o, long foo)
{
  fl_hide_form(helpform);
  fl_free_form(helpform);
}

static int dismiss_help_form_wrapper(FL_OBJECT *o, long foo)
{
  dismiss_help_form(o,foo);
  return FL_IGNORE;
}

static void help_select_section(FL_OBJECT *o, long section)
{
  fl_freeze_form(helpform);
  fl_clear_browser(helpbr);
  fl_addto_browser_chars(helpbr, helpFormEntry[section].body);
  fl_set_browser_topline(helpbr, 1);
  fl_unfreeze_form(helpform);
}

/*
** Sets up the section buttons, returning
** a pointer to the first section.
*/
FL_OBJECT *help_add_sections(void)
{
  FL_OBJECT *obj;
  FL_OBJECT *ret = NULL;
  int i;

  for (i=0; i<HELP_SECTIONS; i++) 
    {
      obj = fl_add_button(FL_RADIO_BUTTON,10+(i*60),315,60,25,
			  helpFormEntry[i].name);
      /*fl_set_object_boxtype(obj,FL_SHADOW_BOX);*/
      fl_set_object_lsize(obj,FL_TINY_SIZE);
      fl_set_object_callback(obj, help_select_section, i);
      if (ret == NULL)
	ret = obj;
    }
  return ret;
}

/*
** DESCRIPTION:
**
*/
void helpCallback(FL_OBJECT *o, long foo)
{
  FL_OBJECT *obj;
  FL_OBJECT *first_button;

  helpform = fl_bgn_form(FL_NO_BOX,550,350);
  obj = fl_add_box(FL_UP_BOX,0,0,550,350,"");
  fl_set_object_color(obj,FL_SLATEBLUE,FL_COL1);
  helpbr = obj = fl_add_browser(FL_NORMAL_BROWSER,10,10,530,295,"");

  obj = fl_add_button(FL_NORMAL_BUTTON,480,315,60,25,"Dismiss");
  fl_set_object_boxtype(obj,FL_SHADOW_BOX);
  fl_set_object_lsize(obj,FL_TINY_SIZE);
  fl_set_object_callback(obj, dismiss_help_form, 0);
  first_button = help_add_sections();
  fl_end_form();

  fl_set_browser_fontstyle(helpbr,FL_FIXED_STYLE);

  fl_set_form_atclose(helpform, dismiss_help_form_wrapper, NULL);

  fl_show_form(helpform,FL_PLACE_FREE,FL_FULLBORDER,"Help");

  fl_set_button(first_button,1);
  fl_call_object_callback(first_button);
}
