/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1993 Richard Goodwin & Joseph O'Sullivan. All rights reserved.
 *
 * FILE: xfRunConsole.c
 *
 * ABSTRACT:
 *
 * Arrrgh. Couldn't stand runConsole. Rewrote, and simplfied.
 * use fdesign to redesign
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/supervise/inspect_xterm.c,v $
 * $Revision: 1.3 $
 * $Date: 97/05/22 08:16:28 $
 * $Author: josullvn $
 *
 * REVISION HISTORY:
 * $Log:	inspect_xterm.c,v $
 * Revision 1.3  97/05/22  08:16:28  josullvn
 * Mods from jl@cs - remove superviseCatchChild race condition, inspect now
 * immediately connects with supervise, and improve the checking of running
 * processes at startup
 * 
 * Revision 1.2  1997/05/20 05:19:17  josullvn
 * new flag -silent for silent operation, better message when readline fails,
 * cleaned up readlines linktime list, quit in inspect closes xterms properly
 *
 * Revision 1.1  1997/05/18 01:31:57  josullvn
 * Reverted back to the old format for storing the sources, ie
 * not in a directory hierarchy. Also updated the help system in inspect.
 *
 * Revision 1.1  1997/05/13 18:01:26  josullvn
 * Amended nanny and xfMiniRunConsole to supervise and inspect.
 * Numerous bugs fixes, but chiefly...
 *  Updated the xforms display.
 *  respond correctly to CTRL-C's.
 *  new resource file format partially available as -oldload foo.rc
 *  fixed signal handling - a change to devUtils disables prior interrupt handlers.
 * 			Not a good thing methinks...
 *  fixed fd handling. Now superviseCatchChild explicitly handles closed fd's.
 *  reduced message traffic, pings only sent when necessary
 *
 * inspect has been cleaned up, slimmed down. Communication and
 * resource management stuff in common. Still a few TODO things...
 *  Doesn't handle blank key strokes (ie just hitting carriage return).
 *  VT100 compliance is limited. As in, still can't play hangman or do vi, or
 *  anything that uses the secondary graphics window. Don't know why really, look
 *  at.... http://sv1pc44.cs.york.ac.uk/vt100_2.html
 *  Basic curses should be OK
 *
 * Revision 1.3  1997/05/05 21:12:35  robocomp
 * Use smaller font (and window size) for on-board (laptop) console.
 *
 * Revision 1.2  1997/05/01 18:09:39  robocomp
 * Changes (mostly by Joseph) to get nanny/console running on Linux, and
 *   running with version 0.86 of libforms.
 *
 * Revision 1.1  1996/07/29 05:03:28  josullvn
 * Bloody hell. This commiting is a bit different than under Xavier. Short
 * story is cleaned up some purify bugs, and also made changes to nanny
 * which should make it a bit better - Improving performance over multiple
 * machines, explict quietening of nondisplayed processes, replacing of
 * runConsole with xfMiniConsole, which is multithreaded, vt102 compilant,
 * adds a uniform emacs-like command line editing feature, better on small
 * screens and otherwise fab.
 *
 * Revision 1.4  1996/06/28  14:07:43  reids
 * Fixed quite a few bugs -- with graphics, interaction with script, and
 *   killing processes
 *
 * Revision 1.3  1996/03/29  15:57:45  reids
 * Consolidated the common code between xCallbacks and xfCallbacks, and
 *   xRunConsole and xfRunConsole.
 * Added a way to add macro definitions to resource files ("define: <x> <y>").
 * Fixed a bug that was causing the xfRunConsole to crash when a task was
 *   restarted.
 *
 *****************************************************************************/
#include "tca/libc.h"
#include "tca/stdinDev.h"
#include <sys/signal.h>

#include "messages.h"

#include "inspect_xterm.h"
#include "callbacks.h"


int     xterm_socket = -1;
int     xterm_pid = 0;
DEV_PTR xterm_device = NULL;


/* DESCRIPTION:
 * 
 * Now create an xterm for program display. Use cat of a socket to 
 * write data to that xterm, use bin/readline to read users input
 * 
 * PRECONDS:
 *
 * POSTCONDS:
 * 
 */
int xterm_comm_socket(struct sockaddr *name)
{
  int  s; 

  if ((s = socket(PF_UNIX,SOCK_STREAM,0)) == -1) {
    perror("socket");
    exit(1);
  }

  if (bind(s,name,sizeof(* name)) == -1) {
    perror("bind");
    exit(1);
  }

  /* backlogs?  one client at a time, so no pending connections */
  if (listen(s,0) == -1) {
    perror("listen");
    exit(1);
  }

  return s;
}


/* DESCRIPTION:
 * 
 * PRECONDS:
 *
 * POSTCONDS:
 * 
 */
void close_xterm()
{
  char   name[14];  /* This is all the bytes we get... */
  int    pid;

  pid = getpid();
  sprintf(name,"/tmp/.xc%d", pid);
  unlink(name);
  kill (xterm_pid, 9);
}

/* DESCRIPTION: 
 * 
 * PRECONDS:
 * 
 * POSTCONDS:
 *  
 */
void mungeReadlineBinary(char *binary, char *argv0)
{
  int i,fd;
  BOOLEAN dirPresent=FALSE;
  char *lastLoc = binary;
  struct stat   buf;

  strcpy(binary, argv0);
  for (i=0; i<strlen(binary); i++)
    if (binary[i] == '/') {
      dirPresent = TRUE;
      lastLoc = &(binary[i]);
     }

  if (dirPresent) {
    lastLoc[1]='\0';
    strcat(binary,"readline");
  } else 
    /* If the filename used as argv[0] does not contain directory information,
     * then readline must be in the path...
     */
    strcpy(binary,"readline");

  if (stat(binary, &buf) == -1)
    {
      WARNING2("\n'readline' not found\nIt must be at the same location as %s\n",argv0);
      WARNING2("Also, please make sure a complete path name to %s was presented\n",argv0);
      exit (-1);
    }

  if (!(buf.st_mode & S_IXUSR))
    {
      WARNING2("\n%s is not executable\n", binary);
      exit (-1);
    }
}

/* DESCRIPTION:
 * 
 * Now create an xterm for program display. Use cat of a socket to 
 * write data to that xterm, use bin/readline to read users input
 * 
 * PRECONDS:
 *
 * POSTCONDS:
 */
int spawn_xterm(int argc, char **argv)
{
  int    i, s, ss, pid, namelen; 
  char   readline_binary[1024];
  struct sockaddr         name;

  DEBUG1("plop_xterm\n");

  mungeReadlineBinary(readline_binary, argv[0]);

  pid = getpid();
  bzero((char *)&name,sizeof(name));
  name.sa_family = AF_UNIX;
  sprintf(name.sa_data,"/tmp/.xc%d", pid);

  xterm_socket = -1;
  s = xterm_comm_socket(&name);
  
  switch (xterm_pid = fork()) {
  case -1:
    WARNING1("FORK FAILURE\n");  perror("fork");
    unlink(name.sa_data);
    return 0;
  case 0:			/* This is the child process executing */
    {
      /* Close the rest of the open files */
      for (i=3; i<NOFILE; i++) {
	close(i); /* Ignoring errors merrily */
      }
#if defined(linux)
      execl ("/usr/X11R6/bin/xterm", 
#else
      execl ("/usr/local/bin/xterm", 
#endif
	     "xterm", 
	     "-rv",         /* I like rv*/
	     "-title", "inspect - waiting...",
	     "-sb",         /* scroll bar good */
#ifdef 0SMALL_WINDOW
	     /* do not do this here - do it with Xdefaults */
	     "-fn", "5x7", /* Use smaller font for on-board laptops */
#endif
	     "-e", 
	     readline_binary,
	     name.sa_data,
	     NULL);
      perror("XTERM"); 
      kill(pid, 9); /* Kill the parent also... */
      exit(-1);
    }
  default:		/* This is the parent process executing */
    /* Wait for the xterm to come up ... */
    namelen = sizeof name;
    xterm_socket = accept(s,(struct sockaddr *)&name,&namelen);
    if (xterm_device == NULL)
      xterm_device = devCreateDev("Xterm device",
				  DEV_OUTPUTHND, inputCallback,
				  DEV_LISTENING, LISTENING | TALKING,
				  DEV_CLOSE_ON_ZERO, FALSE,
				  NULL);
    devConnectDev(xterm_device,xterm_socket);
  }
  return 1;
}
 

/*
 * DESCRIPTION:
 *  If a interrupt happens, we want to kill that damn xterm, if it exists, before exiting
 *
 * INPUTS:
 *  
 */
static void quit(int code)
{
  WARNING1("Interrupt signal received. Cleaning up and Quiting\n");
  close_xterm();
  exit (0);
}

/* DESCRIPTION:
** Handles character typed from stdin while supervise is running.
** Type h for a list of commands.
*/
static void stdin_inputHnd(int fd, long chars_available)
{
  static char buffer[DEFAULT_LINE_LENGTH+1];
  int numRead=0;
  char *buffPtr;
  
  if (silent) return;

  bzero(buffer, DEFAULT_LINE_LENGTH+1);
  
  numRead = devReadN(fd, buffer,
		     MIN(chars_available,DEFAULT_LINE_LENGTH));
  for(buffPtr = buffer; ((*buffPtr == ' ') && (buffPtr != '\0')); buffPtr++);
  if (strstr(buffer,"q")) {
    close_xterm();
    exit(0);
  } else if (strstr(buffer,"h")) {
    printf("h: Help, q: Quit");
    printf("\n");
  }
  printf("> ");fflush(stdout);
}

void xRunConsole(int argc, char **argv)
{
  static char buf[DEFAULT_LINE_LENGTH];
  struct timeval clickPollTime = {0, 100}; /* Every 1/10 second */

  fl_initialize(&argc, argv, "inspect",0,0);

  stdin_connect(stdin_inputHnd); 
  
  /*
   * Add a polling thingy to the stdin_device. Really should
   * create a new device using devCreate, but this is the most convient
   * method. Every "pollTime", pollHandler will be called.
   */
  devStartPolling(stdin_device, &clickPollTime, ProcessXFHandler, NULL);

  xfConsole = create_form_xfMiniConsole();

  fl_deactivate_object(xfConsole->killButton);
  fl_deactivate_object(xfConsole->restartButton);
  fl_deactivate_object(xfConsole->breakButton);

  /* fl_set_browser_fontsize(xfConsole->messageBrowser,FL_TINY_SIZE); */

  setSuspendResume("Suspend");

  deactivateStatus("Ready");

  fl_show_form(xfConsole->xfMiniConsole,
	       FL_PLACE_MOUSE, FL_FULLBORDER, "inspect");

  spawn_xterm(argc, argv);
  signal(SIGINT,quit);
}
