/*****************************************************************************
 * PROJECT: TCA
 *
 * (c) Copyright 1996 Reid Simmons. All rights reserved.
 *
 * FILE: scriptParse.h
 *
 * ABSTRACT: Type definitions for running script files
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/supervise/scriptParse.h,v $
 * $Revision: 1.1 $
 * $Date: 97/05/18 01:32:01 $
 * $Author: josullvn $
 *
 * REVISION HISTORY:
 *
 * $Log:	scriptParse.h,v $
 * Revision 1.1  97/05/18  01:32:01  josullvn
 * Reverted back to the old format for storing the sources, ie
 * not in a directory hierarchy. Also updated the help system in inspect.
 * 
 * Revision 1.1  1997/05/13 18:01:27  josullvn
 * Amended nanny and xfMiniRunConsole to supervise and inspect.
 * Numerous bugs fixes, but chiefly...
 *  Updated the xforms display.
 *  respond correctly to CTRL-C's.
 *  new resource file format partially available as -oldload foo.rc
 *  fixed signal handling - a change to devUtils disables prior interrupt handlers.
 * 			Not a good thing methinks...
 *  fixed fd handling. Now superviseCatchChild explicitly handles closed fd's.
 *  reduced message traffic, pings only sent when necessary
 *
 * inspect has been cleaned up, slimmed down. Communication and
 * resource management stuff in common. Still a few TODO things...
 *  Doesn't handle blank key strokes (ie just hitting carriage return).
 *  VT100 compliance is limited. As in, still can't play hangman or do vi, or
 *  anything that uses the secondary graphics window. Don't know why really, look
 *  at.... http://sv1pc44.cs.york.ac.uk/vt100_2.html
 *  Basic curses should be OK
 *
 * Revision 1.2  1996/02/05 15:57:05  reids
 * Added scripting capabilities to nanny -- using lex/bison to parse script
 *  files according to the script.bnf standard.
 * Integrated support for running/stopping/suspending scripts into the
 *  runConsole window.
 *
 * Revision 1.1  1996/01/22  21:31:42  reids
 * Fixed the way stdout is handled, using pseudo-terminals, so that it preserves
 *   the line-buffering mode of the real TTY.
 * Added support for running scripts (not really integrated, yet, but initial
 *   tests are working).
 *
 ****************************************************************/

#ifndef INC_SCRIPT_PARSE_H
#define INC_SCRIPT_PARSE_H

#define INDENT_INCREMENT (1)

/* Needed for script & runConsole */
void setLocalTimeFromAbsTime(struct tm *localTime, ABS_TIME_PTR absTime);
extern SCRIPT_LIST_PTR scriptParse (FILE *scriptFile);
extern void scriptPrint (SCRIPT_LIST_PTR script);

/* Needed for the bison/yacc parser */
extern SCRIPT_STATEMENT_PTR scriptCreateAction (SCRIPT_STATEMENT_ENUM type,
						char *process);
extern SCRIPT_STATEMENT_PTR scriptCreateSend (char *process, char *line);
extern SCRIPT_STATEMENT_PTR scriptCreateWhenReady (char *process);
extern SCRIPT_STATEMENT_PTR scriptCreateWhen (char *process, BOOLEAN continual,
					      char* line);
extern SCRIPT_STATEMENT_PTR scriptCreateAt (BOOLEAN continual, 
					    SCRIPT_TIME_TYPE time);
extern void scriptInitTime(SCRIPT_TIME_PTR time);
extern time_t relTimeFromAbs(ABS_TIME_PTR time);

extern SCRIPT_LIST_PTR currentScript;

#endif /* INC_SCRIPT_PARSE_H */
