/*****************************************************************************
 * PROJECT: Xavier
 *
 * (c) Copyright 1993 Richard Goodwin & Joseph O'Sullivan. All rights reserved.
 *
 * FILE: supervise.h
 *
 * ABSTRACT:
 *
 * $Source: /afs/cs.cmu.edu/project/TCA/Master/tcaV8/tools/supervise/supervise.h,v $
 * $Revision: 1.13 $
 * $Date: 97/05/26 19:55:49 $
 * $Author: josullvn $
 *
 * REVISION HISTORY:
 *
 * $Log:	supervise.h,v $
 * Revision 1.13  97/05/26  19:55:49  josullvn
 * Running inspect without supervise would cause inspect to crash, due to
 * jl moving the syncwithsupervise to early in the mainline. Added a flag
 * x11Running, to protect this condition.
 * 
 * Revision 1.12  1997/05/26 19:36:30  josullvn
 * Fixed filter bug (pressing cancel crashes inspect)
 * Fixed select bug (was only updating select menu once)
 *
 * Revision 1.11  1997/05/26 18:22:17  josullvn
 * CTR was missing from dependancy list.
 *
 * Revision 1.10  1997/05/26 18:19:32  josullvn
 * fixed bug in supervise_processes in checking the ps list for executables
 *
 * Revision 1.9  1997/05/22 08:16:28  josullvn
 * Mods from jl@cs - remove superviseCatchChild race condition, inspect now
 * immediately connects with supervise, and improve the checking of running
 * processes at startup
 *
 * Revision 1.8  1997/05/20 14:38:40  josullvn
 * error in Xavier.rc
 *
 * Revision 1.7  1997/05/20 14:36:59  josullvn
 * When an error messages happens, process buttons now flash red.
 *
 * Revision 1.6  1997/05/20 05:51:38  josullvn
 * chooseCallback was printing to the wrong location
 *
 * Revision 1.5  1997/05/20 05:47:51  josullvn
 * fixed bogus newline under sunOS in browser window, and an error in Simulator.rc
 *
 * Revision 1.4  1997/05/20 05:23:40  josullvn
 * Some cvs quirks with the last commit
 *
 * Revision 1.3  1997/05/20 05:19:19  josullvn
 * new flag -silent for silent operation, better message when readline fails,
 * cleaned up readlines linktime list, quit in inspect closes xterms properly
 *
 * Revision 1.2  1997/05/19 23:19:57  josullvn
 * Doh - forgot to commit inspect_gui.fd
 * Fixed a X11 bug in messageMakeExec
 * 
 * Revision 1.1  1997/05/18 01:32:02  josullvn
 * Reverted back to the old format for storing the sources, ie
 * not in a directory hierarchy. Also updated the help system in inspect.
 *
 * Revision 1.1  1997/05/13 18:02:23  josullvn
 * Amended nanny and xfMiniRunConsole to supervise and inspect.
 * Numerous bugs fixes, but chiefly...
 *  Updated the xforms display.
 *  respond correctly to CTRL-C's.
 *  new resource file format partially available as -oldload foo.rc
 *  fixed signal handling - a change to devUtils disables prior interrupt handlers.
 * 			Not a good thing methinks...
 *  fixed fd handling. Now superviseCatchChild explicitly handles closed fd's.
 *  reduced message traffic, pings only sent when necessary
 *
 * signal catching and general cleaning up were the key changes here.
 *
 * Revision 1.24  1997/05/02 16:40:29  robocomp
 * Cleaned up to work on Linux (Joseph & Reid)
 *
 * Revision 1.23  1997/01/21 11:19:20  reids
 * When switching processes, don't quiet a process if there is a script
 *   conditional that depends on output from that process.
 * Don't add -wunitialized without the -o flag.
 *
 * Revision 1.22  96/08/05  16:08:04  rich
 * Added comments to endifs.
 * 
 * Revision 1.21  1996/07/29  05:59:30  josullvn
 * Sigh. All the xfConsole files should only be updated using fdesign from
 * xforms. DO NOT EDIT. (Someone had done so...)
 *
 * Revision 1.20  1996/07/29  05:43:00  josullvn
 * Forgot to make the necessary changes to xfConsole et all for the new version
 * of xforms
 *
 * Revision 1.19  1996/07/29  05:03:08  josullvn
 * Bloody hell. This commiting is a bit different than under Xavier. Short
 * story is cleaned up some purify bugs, and also made changes to nanny
 * which should make it a bit better - Improving performance over multiple
 * machines, explict quietening of nondisplayed processes, replacing of
 * runConsole with xfMiniConsole, which is multithreaded, vt102 compilant,
 * adds a uniform emacs-like command line editing feature, better on small
 * screens and otherwise fab.
 *
 * Revision 1.18  1996/07/25  22:25:21  rich
 * Fixed uninitialized memory references.
 *
 * Revision 1.17  1996/07/18  02:23:06  rich
 * Fixed some memory problems found with purify.
 *
 * Revision 1.16  1996/07/10  17:19:41  reids
 * Replaced the "Destroy->" menu (which didn't really work right) with a
 *   "Choose->" menu for picking which process to view (Can do "destroy" with
 *   a "choose" and a "kill").
 *
 * Revision 1.15  1996/07/10  02:10:54  reids
 * Enable "define"s to be multi-word (contain whitespace).
 *
 * Revision 1.14  1996/06/28  14:07:23  reids
 * Fixed quite a few bugs -- with graphics, interaction with script, and
 *   killing processes
 *
 * Revision 1.13  1996/03/29  15:57:15  reids
 * Consolidated the common code between xCallbacks and xfCallbacks, and
 *   xRunConsole and xfRunConsole.
 * Added a way to add macro definitions to resource files ("define: <x> <y>").
 * Fixed a bug that was causing the xfRunConsole to crash when a task was
 *   restarted.
 *
 * Revision 1.12  1996/03/18  22:59:02  reids
 * Added way to load script file as command line option.
 * Fixed the way the browser handled process output that was not
 *   terminated by a <cr>.
 * Made the "run script" callback remember the scriptDirectory.
 *
 * Revision 1.11  1996/02/20  11:40:32  josullvn
 * Made extensive changes:
 *    added xfRunConsole, an xforms based interface which was quick
 *    To change the interface, from the nanny dir run
 *    /afs/cs/user/josullvn/xforms/DESIGN/fdesign xfConsole
 *    to get going, and made debugging easier.
 *    added tons of comments.
 *    Debug spews a lot more info back, so just compile
 *    with DEBUG=DEBUG if you really need it.
 *    I've added some timeouts to the code, which will
 *    probably be tweaked later. There are in xfCallbacks.c
 *    Its become better at keeping processes going, and
 *    so its harder to quit. Using shutdown can be
 *    painful. Killing the processes individually is
 *    probably better. Need to do a reverse dependancy when quitting.
 *    Multiple machines are again poor. It doesn't wait sufficently
 *    for dependencies - so you need to run things explicitly...
 *
 * Revision 1.10  1996/02/18  21:17:47  reids
 * Put script stuff back in;
 *   Try to fix pseudo-terminal handling for UNIX and LINUX;
 *   Resize dialog windows when text gets too long.
 *
 * Revision 1.9  1996/02/14  22:15:13  rich
 * Use flex rather than lex because linux does not have lex.
 *
 * Revision 1.8  1996/02/12  00:46:14  rich
 * Scripting does not compile, removed.
 *
 * Revision 1.7  1996/02/10  16:52:17  rich
 * Made private functions static and fixed some forward declarations.
 *
 * Revision 1.6  1996/02/07  00:28:12  rich
 * Add prefix to VERSION_DATE and COMMIT_DATE.
 *
 * Revision 1.5  1996/02/05  15:56:50  reids
 * Added scripting capabilities to nanny -- using lex/bison to parse script
 *  files according to the script.bnf standard.
 * Integrated support for running/stopping/suspending scripts into the
 *  runConsole window.
 *
 * Revision 1.4  1996/02/01  04:04:13  rich
 * Generalized updateVersion and added recursion.
 *
 * Revision 1.3  1996/02/01  02:03:20  reids
 * Cleaned up/standardized command line option parsing and help messages.
 *
 * Revision 1.2  1996/01/31  22:51:55  reids
 * Added automatic updating of (micro) version control numbers
 *
 * Revision 1.1  1995/12/17  20:25:44  rich
 * Moved Nanny to the tca release.
 *
 * Revision 1.6  1995/07/15  22:40:51  josullvn
 * Had deleted the line which announce new processes by mistake
 *
 * Revision 1.5  1995/07/15  07:13:11  josullvn
 * ProcessDevices is _not_ reentrant - this was causing lots of problems as it
 * turns out. Restructing my blocking code fixed it.
 * Also, added a notion of dead - that is telling when something has
 * finally died.
 *
 * Revision 1.4  1995/07/13  17:04:34  josullvn
 * Added version info
 *
 * Revision 1.3  1995/07/11  22:59:16  josullvn
 * Needed to retain the concept of who sent a message, and so had to
 * restructure all the message passing. Also, made sure we don't
 * overwrite any of the parameters due to reentry problems.
 *
 * Revision 1.2  1995/07/11  11:21:34  josullvn
 * A lot more changes to message passing to handle multiple nannys better.
 * This basically involved removing fdclient, and being smarter about what
 * nannys are in the system
 *
 * Revision 1.1  1995/05/15  17:16:48  rich
 * Added nanny.h and moved external declarations to the .h files.
 *
 *
 *****************************************************************************/

#ifndef SUPERVISE_H_INCLUDED
#define SUPERVISE_H_INCLUDED

extern char *rcFileName;

extern char **environment;

#define SUPERVISE_VERSION_MAJOR  1
#define SUPERVISE_VERSION_MINOR  4
#define SUPERVISE_VERSION_MICRO  15
#define SUPERVISE_VERSION_DATE "May-26-97"
#define SUPERVISE_COMMIT_DATE "$Date: 97/05/26 19:55:49 $"

#endif /* SUPERVISE_H_INCLUDED */
