########################################################################
######################## CANVAS MANIPULATION ###########################

set scroll_offset 0

#-----------------------------------------------------------------------
# Canvas_Change_Time
# If the canvas is scaled or shifted then the time corresponding to
# the leftmost visible side of the canvas needs to be determined.  
#-----------------------------------------------------------------------
proc Canvas_Change_Time {} {
    global scroll_offset CANVAS_WIDTH DISPLAY_TIME Scale_Factor
    global CANVAS_SHOW_WIDTH

    set b [lindex [.a.c.canvas xview] 0]
    set scroll_offset [expr $CANVAS_WIDTH*$b]
    if ($DISPLAY_TIME) {
	.space.lefttime configure \
		-text [Time_String [expr $scroll_offset/$Scale_Factor]]
	.space.righttime configure -text [Time_String \
		[expr ($scroll_offset+$CANVAS_SHOW_WIDTH)/$Scale_Factor]]
    }
}


#-----------------------------------------------------------------------
# Whenever the canvas is scrolled the scroll_offset variable needs to
# be updated.
#-----------------------------------------------------------------------
proc CanvasScrollX { a b { c d } } {

    if ([string compare $a moveto]==0) {
	.a.c.canvas xview $a $b
    } else {
	.a.c.canvas xview $a $b $c
    }
    
    Canvas_Change_Time
}


#-----------------------------------------------------------------------
# Whenever the canvas is scrolled the scroll_offset variable needs to
# be updated.
#-----------------------------------------------------------------------
proc CanvasScrollY { a b { c d } } {

    if ([string compare $a moveto]==0) {
	.a.c.canvas yview $a $b
    } else {
	.a.c.canvas yview $a $b $c
    }
}


#-----------------------------------------------------------------------
# Canvas_Left_Edge returns the position within the canvas that is
# currently at the leftmost visible side of the display.
#-----------------------------------------------------------------------

proc Canvas_Left_Edge {} {
    global CANVAS_WIDTH Scale_Factor

    set current_view [.a.c.canvas xview]
    return [expr [lindex $current_view 0] * $CANVAS_WIDTH / $Scale_Factor]
}


#-----------------------------------------------------------------------
# Canvas_Right_Edge returns the position within the canvas that is
# currently at the rightmost visible side of the display.
#-----------------------------------------------------------------------

proc Canvas_Right_Edge {} {
    global CANVAS_WIDTH Scale_Factor

    set current_view [.a.c.canvas xview]
    return [expr [lindex $current_view 1] * $CANVAS_WIDTH / $Scale_Factor]
}


#-----------------------------------------------------------------------
# Canvas_Center_Around
#-----------------------------------------------------------------------
proc Canvas_Center_Around { position } {
    global CANVAS_WIDTH Scale_Factor CANVAS_SHOW_WIDTH

    set x [expr ($position-$CANVAS_SHOW_WIDTH/2)*$Scale_Factor/$CANVAS_WIDTH]
    
    if ($x<0) { set x 0 }
    if ($x>1) { set x 1 }

    Canvas_Moveto $x
}


#-----------------------------------------------------------------------
# Canvas_Moveto
#-----------------------------------------------------------------------
proc Canvas_Moveto { position } {
    .a.c.canvas xview moveto $position
    Canvas_Change_Time
}


#-----------------------------------------------------------------------
# Canvas_Scroll
#-----------------------------------------------------------------------
proc Canvas_Scroll { amt unit } {
    .a.c.canvas xview scroll $amt $unit
    Canvas_Change_Time
}

#-----------------------------------------------------------------------
# Unselect_Current
#-----------------------------------------------------------------------
proc Unselect_Current {} {
    global CurrentShowAll current_highlight

    set CurrentShowAll "0"

    .a.c.canvas delete module_call_arrows
}

#-----------------------------------------------------------------------
# Unselect
#-----------------------------------------------------------------------
proc Unselect {} {
    global current_highlight

    Unselect_Current
    set current_highlight 0
    .info.top.tippy.unselect configure -state disabled
}

#-----------------------------------------------------------------------
# Grow_Canvas
#-----------------------------------------------------------------------
proc Grow_Canvas { x_amt y_amt } {
    global CANVAS_WIDTH CANVAS_HEIGHT place

    set CANVAS_WIDTH [expr $CANVAS_WIDTH+$x_amt]
    set CANVAS_HEIGHT [expr $CANVAS_HEIGHT+$y_amt]
    set foo [list 0 0 $CANVAS_WIDTH $CANVAS_HEIGHT]
    $place configure -scrollregion $foo
}
