/*
 * File: filei.c
 * Author: Greg Whelan, CMU
 * Purpose: Some simple functions to read from a log file
 *
 */

#include <sys/file.h>
#ifdef REDHAT_52
#include <sys/ioctl.h>
#endif
#include "tools/Standard.h"
#include "tools/List.h"
#include "tools/Array.h"
#include "tools/MsgData.h"
#include "tools/TaskTree.h"
#include "tools/MsgData.h"
#include "tools/Parser.h"
#include "filei.h"

FILE *global_log_file;

/* ------------------- local prototypes ---------------------------- */

static int ReadFromFile(FILE *fd, String buffer, 
			String *startPos, String *endPos);
static int ReadFromBuffer(String my_string, String buffer, 
			  String *startPos, String *endPos);
static BOOLEAN CommandInBuffer(String buffer, String startPos, String endPos);

/* ----------------------------------------------------------------- */


BOOLEAN OpenFile(String file_name)
{   
  global_log_file = fopen(file_name,"r");
  return (global_log_file != NULL);
}

int ReadNlFromFile(FILE *fd, String my_string)
{
  static char buffer[MAX_LONG_MESSAGE+1];
  static char *startPos = buffer; 
  static char *endPos = buffer; 
  int num_read;
  
  if (!CommandInBuffer(buffer, startPos, endPos)) {
    num_read = ReadFromFile(fd, buffer, &startPos, &endPos);
    if (num_read == 0) 
      return(EOF);
  }
  return(ReadFromBuffer(my_string, buffer, &startPos, &endPos));
}


/******************************************************************************
 *
 * FUNCTION: long numChars(sd)
 *
 * DESCRIPTION:
 * Find out how many characters are available to be read.
 *
 * INPUTS: 
 * int sd;
 *
 * OUTPUTS: 
 *
 * NOTES:
 *
 *****************************************************************************/

static long numChars(int sd)
{
  long available=0;
#if defined(__svr4__) /* SUNOS5 */
  off_t currentLoc;
  
  currentLoc = lseek(sd, 0L, SEEK_CUR);
  if (currentLoc >= 0) {
    available = lseek(sd, 0L, SEEK_END) - currentLoc;
    lseek(sd, currentLoc, SEEK_SET);
    return available;
  } else if (errno != ESPIPE)
    return -1;
  else
#endif /* SUNOS5 */
    if (ioctl(sd, FIONREAD, &available) == 0)
      return available;
    else
      return -1;
}

static int ReadFromFile(FILE *fd, String buffer, 
			String *startPos, String *endPos)
{
  int num_read=0;
  int chars_to_read;
  long available=0;
  
  available = numChars(fileno(fd));
  chars_to_read = MIN(MAX_LONG_MESSAGE - (*endPos - *startPos),
		      available);
  if (chars_to_read > 0) {
    num_read = read(fileno(fd), *endPos, chars_to_read);
    *endPos += num_read;
  }
  return(num_read);
}


static int ReadFromBuffer(String my_string, String buffer, 
			   String *startPos, String *endPos)
{
  String lineEnd;
  
  lineEnd = *startPos;
  while (*lineEnd != '\n') {
    lineEnd++;
    if (lineEnd == *endPos) {
      lineEnd = NULL;
      break;
    }
  }
  if (lineEnd != NULL) {
    *lineEnd = '\0';
    strcpy(my_string, *startPos);
    *startPos = lineEnd+1;
  }
  /* Fix up the buffer. Throw out any consumed lines.*/
  if (*startPos != buffer)
    { /* slide it back and wait for more characters */
      bcopy(*startPos, buffer, (*endPos - *startPos));
      *endPos = buffer + (*endPos - *startPos);
      *startPos = buffer;
    }
  return(1);
}


static BOOLEAN CommandInBuffer(String buffer, String startPos, String endPos)
{
  String lineEnd;
  
  if (startPos == endPos)
    return(False);
  lineEnd = startPos;
  while (*lineEnd != '\n') {
    lineEnd++;
    if (lineEnd == endPos)
      return (False);
  }
  return (True);
}
