/*****************************************************************
 * File: main.c
 * Author: Greg Whelan, CMU
 * Purpose: Main for TView.
 * 
 ****************************************************************/

#include <stdio.h>
#include "tools/Standard.h"
#include "tools/List.h"
#include "tools/Array.h"
#include "tools/MsgData.h"
#include "tools/TaskTree.h"
#include "tools/MsgData.h"
#include "tools/Parser.h"
#include "filei.h"
#include "tview.h"
#include "tclsetup.h"

/* --------------------- Local prototypes ----------------------- */

static void displayVersion(void);
static void printHelp(void);
static void parseCommandLineOptions (int argc, char **argv);

/* --------------------- Global variables ----------------------- */

int show_prompt;
int APP_DEBUG = 0;
int screen_size=2;
int file_selected=0;
char filename[1024];

/* --------------------------- Code ----------------------------- */

static void displayVersion(void)
{
  printf("tview %d.%d.%d (log parser %d.%d.%d)\n",
	 TVIEW_VERSION_MAJOR, TVIEW_VERSION_MINOR, TVIEW_VERSION_MICRO,
	 TPARSER_VERSION_MAJOR, TPARSER_VERSION_MINOR, TPARSER_VERSION_MICRO);
  printf(" Released : %s\n", TVIEW_VERSION_DATE);
  printf(" Commited : %s\n", TVIEW_COMMIT_DATE);
  printf(" Compiled : %s %s\n", __DATE__, __TIME__);
  fflush(stdout);
}

static void printHelp(void)
{
  printf("-h : Print this message\n");
  printf("-v : Print version information\n");
  printf("-f <logfile>: View the logfile\n");
  printf("-m : Display inform messages (default is not to display them)\n");
}

static void parseCommandLineOptions (int argc, char **argv)
{
  int i;

  for (i=1; i<argc; i++) {
    if (!strcmp(argv[i], "-h")) {
      printHelp();
      exit(0);
    } else if (!strcmp(argv[i], "-v")) {
      displayVersion();
      exit(0);
    } else if (!strcmp(argv[i], "-d")) {
      APP_DEBUG=1;
      show_prompt=1;
    } else if (!strcmp(argv[i], "-s")) {
      screen_size = 1;
    } else if (!strcmp(argv[i], "-f")) {
      i++;
      if (!OpenFile(argv[i])) {
	fprintf(stderr, "File %s doesn't exist\n", argv[i]);
	exit(-1);
      } else {
	sprintf(filename, "set filename %s", argv[i]);
	file_selected=1;
      }
    } else if (!strcmp(argv[i],"-m")) {
      _process_info_msg = TRUE;
    } else {
      fprintf(stderr, "Unknown option %s\n", argv[i]);
    }
  }
}

void main (int argc, char **argv)
{
  char tview_directory[1024];
  char willow_directory[1024];
  char *tview_env_var;
  char *willow_env_var;

  parseCommandLineOptions(argc, argv);

  /* If the envirnment variable TVIEW_DIRECTORY is defined then
     that variable specifies where to find the source Tcl files,
     otherwise use the macro defined at compilation time */
  tview_env_var=getenv("TVIEW_DIRECTORY");

  if (!tview_env_var) {
    /* the constant TVIEW_DIRECTORY is set as a define in the makefile
       for the tview application. if the user doesn't have an environment
       variable overriding that setting, use that definition. */
    strcpy(tview_directory, TVIEW_DIRECTORY);
  } else {
    strcpy(tview_directory, tview_env_var);
  }

  willow_env_var=getenv("WILLOW_DIRECTORY");

  if (!willow_env_var) {
    /* the constant WILLOW_DIRECTORY is set as a define in the makefile
       for the tview application. if the user doesn't have an environment
       variable overriding that setting, use that definition. */
    strcpy(willow_directory, WILLOW_DIRECTORY);
  } else {
    strcpy(willow_directory, willow_env_var);
  }

  tview_startup();
  init_grafx(argc, argv, filename, "1", 0, 
	     tview_directory, willow_directory, show_prompt);
}
