# -------------------------------------------------------------------- #
#
# Tview -- TCA Communication Graphic Tool
# Main Tcl Source
#
# -------------------------------------------------------------------- #

wm title . "Tview 3.0"

set result [expr [string length $settings_file]>0]

if ($FILE_SELECTED) {
    # remove any path information from the filename (for use in displaying
    # the filename above the graphical display).
    set fname [string range $filename \
	    [expr [string last "/" $filename]+1] \
	    [string length $filename]]

    # create the log file name to search for:
    
    # loc is the location of the "." in the filename
    set loc [expr [string last "." $filename]-1]
    if ($loc<0) { set loc [string length $filename] }
    # raw_filename is the filename excluding the ".log", if present.
    # ie. for "foo.log" raw_filename is "foo"
    set raw_filename [string range $filename 0 $loc]

    if ($result) {
    } else {
	set settings_file [string range $filename 0 $loc]
	append settings_file ".set"
    }
} else {
    set fname "No File Opened"
}

# --------------------------------------------------------------------

source $SOURCE_DIRECTORY/options.tcl
source $SOURCE_DIRECTORY/help.tcl
source $SOURCE_DIRECTORY/connect.tcl

source $WILLOW_DIRECTORY/nodes.tcl
source $SOURCE_DIRECTORY/fonts.tcl

########################################################################
########################  Graphical Layout #############################

frame .mbar -relief raised -bd 2
frame .space -bd 2
pack .mbar -fill x
pack .space -fill x

# load menu
source $SOURCE_DIRECTORY/menu.tcl

label .space.lefttime -text ""
pack .space.lefttime -side left -padx 120

# create a label above the canvas that shows the filename (the
# filename variable should be set by the C code in tcl-framework.c)
label .space.filename -relief groove -text $fname -padx 5
pack .space.filename -side left -padx 115

label .space.righttime -text ""
pack .space.righttime -side right -padx 35


#######################################################################
####################### More Graphical Layout #########################

frame .info
frame .info.top
frame .info.bot

frame .info.top.time
frame .info.top.tippy

button .info.top.tippy.runback -text "<<" -command RunBackCommand \
	-state disabled
pack .info.top.tippy.runback -side left

button .info.top.tippy.back -text "Back" -command BackCommand \
	-state disabled
pack .info.top.tippy.back -side left

button .info.top.tippy.step -text "Step" -command StepCommand
pack .info.top.tippy.step -side left

button .info.top.tippy.run -text ">>" -command RunCommand
pack .info.top.tippy.run -side left

button .info.top.tippy.unselect -text "Unselect" -command Unselect \
	-state disabled
pack .info.top.tippy.unselect -side left

button .info.top.tippy.follow -text "Scroll Lock" -command FollowIncoming
pack .info.top.tippy.follow -side left

label .info.top.time.tlabel -text "--"
#pack .info.top.time -side bottom
pack .info.top.time.tlabel -side right

pack .info.top.tippy -side top
pack .info.top.time -side bottom

frame .info.bot.mini
label .info.bot.mini.command
label .info.bot.mini.buf
pack .info.bot.mini -side bottom
pack .info.bot.mini.command -side left
pack .info.bot.mini.buf -side left

pack .info.bot -side bottom
pack .info.top -side top -padx 150

pack .info -side bottom -pady 20

frame .lbo
pack .lbo -side bottom

frame .pan
pack .pan -side bottom

#-----------------------------------------------------------------------
# Scrolled_Canvas
#
# procedure to create a scrolled canvas and display it.
#-----------------------------------------------------------------------

proc Scrolled_Canvas { c width height region } {
	frame $c
	canvas $c.canvas -width $width -height $height \
		-scrollregion $region \
		-xscrollcommand [list $c.xscroll set] \
		-yscrollcommand [list $c.yscroll set] \
		-background grey \
		-relief sunken
	scrollbar $c.xscroll -orient horizontal \
		-command CanvasScrollX
	scrollbar $c.yscroll -orient vertical \
		-command CanvasScrollY
	pack $c.xscroll -side bottom -fill x
	pack $c.yscroll -side right -fill y
	pack $c.canvas -side left -fill both -expand true
	pack $c -side right -fill both -expand true
	return $c.canvas
}

#########################################################################
########################## CANVAS LAYOUT ################################

frame .a

# Create a  canvas to be used for the module labels:
# frame .a.mod
#canvas .a.mod.names -width 100 -height $CANVAS_SHOW_HEIGHT
#pack .a.mod.names -padx 4 -side left -fill y

set foo [list 0 0 $CANVAS_WIDTH $CANVAS_HEIGHT]

Scrolled_Canvas .a.c $CANVAS_SHOW_WIDTH $CANVAS_SHOW_HEIGHT $foo

pack .a -side top
#pack .a.mod -side bottom
#pack .a.mod.names

.a.c.canvas configure -rel sunken

set place .a.c.canvas

# NO PANNER YET
if (0) {

canvas .pan.panner -width $CANVAS_SHOW_WIDTH -height 25 -bg grey
pack .pan.panner -side bottom
.pan.panner create rectangle 2 2 25 26 -fill lightgrey -tags "panner" \
	-outline white -width 2

set pannerX 0
set pannerY 0

.pan.panner bind panner <Button-1> {
    set pX %x
    set pY %y
}

.pan.panner bind panner <B1-Motion> {
    set dx [expr %x-$pX]
    set dy [expr %y-$pY]

    if ([expr $dx+$pannerX]<0) { set dx [expr -$pannerX] }
    if ([expr $dx+$pannerX]>775) { set dx [expr 775-$pannerX] }
    if ([expr $dy+$pannerY]<0) { set dy [expr -$pannerY] }
    if ([expr $dy+$pannerY]>0) { set dy [expr 0-$pannerY] }

    .pan.panner move panner $dx $dy
    incr pannerX $dx
    incr pannerY $dy
    set pX %x
    set pY %y
    CanvasScrollX moveto [expr $pX / 800.0]
}

}

proc resize {} {
    set foo [.pan.panner coords panner]
    set x1 [lindex $foo 0]
    set y1 [lindex $foo 1]
    set x2 [lindex $foo 2]
    set y2 [lindex $foo 3]
    
    .pan.panner coords panner $x1 $y1 [expr $x2+1] $y2
}

# canvas functions
source $SOURCE_DIRECTORY/canvas.tcl

# log window
source $SOURCE_DIRECTORY/logwindow.tcl

# tview stuff
source $SOURCE_DIRECTORY/tview.tcl
