/* --------------------------------------------------------------- */
/* record.c                                                        */
/* Support for back-step in Tview                                  */
/* May 22, 1997                                                    */
/* --------------------------------------------------------------- */

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "tools/Standard.h"
#include "tools/List.h"
#include "tools/Array.h"
#include "tools/MsgData.h"
#include "tools/TaskTree.h"
#include "tools/MsgData.h"
#include "tools/Parser.h"
#include "filei.h"
#include "tview.h"
#include <willow.h>
#include "tclsetup.h"
#include "uistuff.h"
#include "record.h"
#include "tools/Array.h"
#include "tools/List.h"


/* ----------------------------------------------------------------- */

#define My_Array_Type ListCell

ListCell My_Array_New(void);
Record_Type *My_Array_Data(My_Array_Type cell);
ListCell My_Array_Next(My_Array_Type cell);
ListCell My_Array_Prev(My_Array_Type cell);
ListCell My_Array_Insert(My_Array_Type cell, Record_Type *rt);

ListCell My_Array_New()
{
  ListCell foo;

  foo = (ListCell)(malloc(sizeof(_ListCell)));
  ListCellInit(foo);

  return (foo);
}

Record_Type *My_Array_Data(My_Array_Type cell)
{
  return (Record_Type *)(cell->item);
}

ListCell My_Array_Next(My_Array_Type cell)
{
  return (ListForwardN(cell, 1));
}

ListCell My_Array_Prev(My_Array_Type cell)
{
  return (ListBackwardN(cell, 1));
}

ListCell My_Array_Insert(My_Array_Type cell, Record_Type *rt)
{
  ListCell n;

  n = My_Array_New();
  n->item = rt;
  ListInsertRight(cell, n);

  return (n);
}

/* ----------------------------------------------------------------- */

extern Willow_Tree_Type global_tree;
extern Tcl_Interp *local_ti;

My_Array_Type Undo_Record;

int undo_count=0;
int undo_index=0;
int action_count=0;

/* ---------------------- local prototypes ------------------------- */

void Insert_Node(Task task);
void Delete_Node(Task task);

/* ----------------------------------------------------------------- */

void Initialize_Recording()
{
  /* the undo record is a resizable array */
  Undo_Record = My_Array_New();
}

void Record_Action(Task task, Undo_Type type, taskState from, taskState to)
{
  Record_Type *New_Item;

  New_Item = (Record_Type *)(malloc(sizeof(Record_Type)));

  New_Item->task = task;
  New_Item->undo_type = type;
  New_Item->undo_from_state = from;
  New_Item->undo_to_state = to;
  New_Item->actions = 0;

  Undo_Record = My_Array_Insert(Undo_Record, New_Item);

  /* action_count is the number of actions for a particular step */
  action_count++;
  /* undo_count is the total number of histories in the undo_record */
  undo_count++;
  /* undo_index is the history we're currently using/constructing */
  undo_index = undo_count;
}

void End_Record_Action(int logline)
{
  Record_Type *rt;

  rt = My_Array_Data(Undo_Record);
  if (rt) {
    rt->actions = action_count;
    rt->logline = logline;
  }
  /* reset the counter */
  action_count = 0;
}

/* Undo_Step is called when a "Back" command is issued from Tview */

int Undo_Step()
{
  Task task;
  int n;
  Record_Type *undo_element;

  /* get the number of individual steps for this undo */
  n = My_Array_Data(Undo_Record)->actions;
  Highlight_Log_Line(My_Array_Data(Undo_Record)->logline);

  do {

    undo_element = My_Array_Data(Undo_Record);
    task = undo_element->task;
    
    if (undo_element->undo_type==UNDO_INSERT) {
      /* the original action was an insert, so we delete */
      Delete_Node(task);
    } else if (undo_element->undo_type==UNDO_DELETE) {
      /* the original action was a delete, so we insert */
      task->state = undo_element->undo_from_state;
      Insert_Node(task);
    } else {
      Update_State_Node(task, undo_element->undo_from_state);
      task->state = undo_element->undo_from_state;
    }

    Undo_Record = My_Array_Prev(Undo_Record);
    undo_index--;

  } while (--n > 0);

  return (undo_index > 0);
}

void Redo_Step()
{
  Task task;
  int actions;
  Record_Type *undo_element;

  do {
    Undo_Record = My_Array_Next(Undo_Record);
    undo_element = My_Array_Data(Undo_Record);
    task = undo_element->task;
    actions = undo_element->actions;

    if (undo_element->undo_type==UNDO_DELETE) {
      /* the original action was a delete, so we delete */
      Delete_Node(task);
    } else if (undo_element->undo_type==UNDO_INSERT) {
      /* the original action was a insert, so we insert */
      task->state = undo_element->undo_to_state;
      Insert_Node(task);
    } else {
      Update_State_Node(task, undo_element->undo_to_state);
      task->state = undo_element->undo_to_state;
    }

    undo_index++;

  } while (actions==0);

  Highlight_Log_Line(undo_element->logline);
}
