#include <stdio.h>
#include <stdlib.h>
#include "tca/libc.h"
#include <tcl.h>
#include <tk.h>
#include <string.h>
#include <math.h>
#include <willow.h>
#include "tview.h"
#include "uistuff.h"
#include "tclsetup.h"
#include "record.h"

#define STARTUP_FILE "main.tcl"

static char *tcl_RcFileName =  NULL;
char *s_filename;
char zoom_init_str[255];
char startup_string[255];
static char set_dir_string[1024];
static char set_wil_string[1024];

Tcl_Interp *tcl_interp;
static Tk_Window tcl_mainwindow;

extern int screen_size;

extern int undo_count;
extern int undo_index;

/* -------------------- local prototypes ----------------------------- */

int Step_Forward(ClientData, Tcl_Interp *, int, char *[]);
int Change_Size(ClientData, Tcl_Interp *, int, char *[]);
int Widget_CB(ClientData, Tcl_Interp *, int, char *[]);
int C_Initialize(ClientData, Tcl_Interp *, int, char *[]);
int C_Redraw_Tree(ClientData, Tcl_Interp *, int, char *[]);
int C_Toggle_Layout(ClientData, Tcl_Interp *, int, char *[]);
int C_Draw_Nodes(ClientData, Tcl_Interp *, int, char *[]);
int C_Set_Child_Space(ClientData, Tcl_Interp *, int, char *[]);
int C_Set_Sibling_Space(ClientData, Tcl_Interp *, int, char *[]);
int C_Toggle_Message(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[]);
int C_Step_Back(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);
int C_Redo_Step(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[]);

/* ------------------------------------------------------------------- */

int Step_Forward(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  if (undo_index == undo_count) {
    sprintf(interp->result, "%d", One_Tree_Update_Step(interp));
  } else {
    Redo_Step();
    sprintf(interp->result, "%d", 1);
  }

  return TCL_OK;
}

int Change_Size(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  smaller();

  return TCL_OK;
}

int Widget_CB(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  int ptr;

  sscanf(argv[1], "%d", &ptr);
  Select_Widget((void *)ptr);
  return TCL_OK;
}

int C_Initialize(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  tview_initialize(interp);
  return TCL_OK;
}

int C_Redraw_Tree(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  Redraw_Display();
  return TCL_OK;
}

int C_Toggle_Layout(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  toggle_layout();
  return TCL_OK;
}

int C_Draw_Nodes(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  toggle_draw_nodes();
  return TCL_OK;
}

int C_Step_Back(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  sprintf(interp->result, "%d", Undo_Step());
  return TCL_OK;
}

int C_Redo_Step(ClientData clientData, Tcl_Interp *interp,
		 int argc, char *argv[])
{
  Redo_Step();
  return TCL_OK;
}

int C_Set_Child_Space(ClientData clientData, Tcl_Interp *interp,
		      int argc, char *argv[])
{
  int size;

  sscanf(argv[1], "%d", &size);
  Set_Child_Space(size);
  return TCL_OK;
}

int C_Set_Sibling_Space(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[])
{
  int size;

  sscanf(argv[1], "%d", &size);
  Set_Sibling_Space(size);
  return TCL_OK;
}

int C_Toggle_Message(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[])
{
  int task_type;

  sscanf(argv[1], "%d", &task_type);
  toggle_message(task_type);
  return TCL_OK;
}

int init_grafx(int argc, char **argv, char *set_filename, char *zoom,
	       int zoom_found, char *source_directory, 
	       char *willow_directory, int tcl_prompt)
{
  int c=1; /* bogus hack because we don't want Tk to see the command */
	   /* line arguments (they're not for Tk) */
  
  s_filename=strdup(set_filename);

  if (zoom_found) { sprintf(zoom_init_str,"set Scale_Factor %s", zoom); } 
  else { sprintf(zoom_init_str,"set Scale_Factor -1"); } 

  sprintf(set_dir_string, "set SOURCE_DIRECTORY %s", source_directory);
  sprintf(set_wil_string, "set WILLOW_DIRECTORY %s", willow_directory);
  sprintf(startup_string, "source %s/%s", source_directory, STARTUP_FILE);

  if (tcl_prompt) {
    /* For debugging purposes this will result in a wish prompt */
    Tk_Main(c, argv, (Tcl_AppInitProc*) Tcl_AppInit);
  } else {
    /* For normal usage, use the Tk main event loop (no prompt) */
    tcl_interp = Tcl_CreateInterp();

#if(TCL_MAJOR_VERSION==8)
    tcl_mainwindow = Tk_MainWindow(tcl_interp);
#else
    tcl_mainwindow = Tk_CreateMainWindow(tcl_interp, (char *)NULL,
					 argv[0], argv[0]);
#endif
    
    Tcl_AppInit(tcl_interp);
    Tk_MainLoop();
  }

  return 0;
}

/*
 * Tcl_AppInit is called from Tcl_Main
 * after the Tcl interpreter has been created,
 * and before the script file
 * or interactive command loop is entered.
 */
int Tcl_AppInit(Tcl_Interp *interp)
{
  char tcl_command[100];

  /* Initialize packages. Tcl_Init sets up the Tcl library facility. */
  if (Tcl_Init(interp) == TCL_ERROR) {
    fprintf(stderr,"Tcl_Init ERROR\n");
    return TCL_ERROR;
  }
  if (Tk_Init(interp) == TCL_ERROR) {
    fprintf(stderr,"Tk_Init ERROR\n");
    return TCL_ERROR;
  }

  /* Application-specific commands */
  Tcl_CreateCommand(interp, "C_Step", Step_Forward,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Step_Back", C_Step_Back,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Redo_Step", C_Redo_Step,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Change_Size", Change_Size,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  
  Tcl_CreateCommand(interp, "C_Widget_Callback", Widget_CB,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Initialize", C_Initialize,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Redraw_Tree", C_Redraw_Tree,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Toggle_Message", C_Toggle_Message,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Toggle_Layout", C_Toggle_Layout,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
  
  Tcl_CreateCommand(interp, "C_Draw_Nodes", C_Draw_Nodes,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Set_Child_Space", C_Set_Child_Space,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "C_Set_Sibling_Space", C_Set_Sibling_Space,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Add_Willow_Commands(interp);
  
  /*
   * Define startup filename. This file is read in
   * case the program is run interactively.
   */
  tcl_RcFileName = STARTUP_FILE;

  My_Tcl_Eval(interp, s_filename);
  My_Tcl_Eval(interp, zoom_init_str);
  My_Tcl_Eval(interp, set_dir_string);
  My_Tcl_Eval(interp, set_wil_string);
  My_Tcl_Eval(interp, "set settings_file \"\"");
  My_Tcl_Eval(interp, "set FILE_SELECTED 1");
  sprintf(tcl_command, "set GEOMETRY_SIZE %d", screen_size);
  My_Tcl_Eval(interp, tcl_command);
  My_Tcl_Eval(interp, startup_string);

  return TCL_OK;
}


int My_Tcl_Eval(Tcl_Interp *interp, char *s)
{
  int result;

  result = Tcl_Eval(interp, s);

  if (result == TCL_ERROR) {
    fprintf(stderr, "Tcl Error: %s\n", interp->result);
  }
  
  return result;
}
