/* --------------------------------------------------------------- */
/* TView Version 3.0                                               */
/* Greg Whelan, whelan@cs.cmu.edu                                  */
/* Carnegie Mellon University                                      */
/* April 1997                                                      */
/* --------------------------------------------------------------- */

#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include "tools/Standard.h"
#include "tools/List.h"
#include "tools/Array.h"
#include "tools/MsgData.h"
#include "tools/TaskTree.h"
#include "tools/MsgData.h"
#include "tools/Parser.h"
#include "filei.h"
#include "tview.h"
#include <willow.h>
#include "tclsetup.h"
#include "uistuff.h"
#include "record.h"

Willow_Tree_Type global_tree;
Tcl_Interp *local_ti;
int message_processed;
int display_type[25];    /* this currently is not being used */
int del_when_done[25];
extern int APP_DEBUG;
int msg_count;
int log_lines;
Task root;

/* ---------------------- local prototypes ------------------------ */

int Get_Line_From_Log_File(FILE *file, char *msg, MsgData *msgData, 
		      Tcl_Interp *interp);
void Update_GUI_Log(char *msg, MsgData *msgData);
int Logged_Message_Type(taskType type);
void Redraw_Node_Handler(void *tnode_ptr);
int Delete_Node_Test(Task task, int new_state);
void Check_For_Bogus_Node(Task task);

/* ---------------------- Task Tree Management -------------------- */

static void InsertTaskAsRootUI(Task root_task)
{
  Tview_Node tnode;
  Willow_Node wnode;
  char tcl_command[100];
  int ptr;
  int x, y;

  /* keep track of the root task for nodes which have no parent */
  root = root_task;
  /* set the id to -1 */
  root_task->id = -1;

  /* create the new tree (done in Tcl for flexible and easy config) */
  sprintf(tcl_command, "Tview_Create_Tree");
  My_Tcl_Eval(local_ti, tcl_command);
  /* the return result is a pointer to a Willow tree */
  sscanf(local_ti->result, "%d", &ptr);
  global_tree = (Willow_Tree_Type)ptr;

  /* register our redraw-callback handler */
  Willow_Set_C_Callback(global_tree, Redraw_Node_Handler);

  /* create the new widget to be inserted as root of tree */
  Tview_Create_Node(root_task, &tnode);
  root_task->additional = (void *)tnode;
  wnode = tnode->wnode;
  /* we insert that widget into the tree data structure */
  Willow_Insert_Node(global_tree, wnode, 0);
  /* we now create the display of the tree */
  Willow_Draw_Tree(global_tree, &x, &y);
}

void Redraw_Node_Handler(void *tnode_ptr)
{
  char tcl_command[100];
  Tview_Node tnode;
  Willow_Node new_wnode;
  char description[250];

  tnode = (Tview_Node)tnode_ptr;

  if (!(tnode->selected)) {
    Task_Type_String(tnode->task, description);
    sprintf(tcl_command, "%s %s %d %d",
	    description, (tnode->task)->name, 
	    (tnode->task)->state, (int)tnode);
  } else {
    Task_Description(tnode->task, (tnode->task)->state, description);
    sprintf(tcl_command, "Create_Selected \"%s\" %d %d",
	    description, (tnode->task)->state, (int)tnode);
  }

  Make_Node(tcl_command, &new_wnode, tnode);
  /* replace the node that's there with this new node */
  Willow_Modify_Node(global_tree, tnode->wnode, new_wnode);
}


/* this is a hack -- get rid of it eventually */
void Check_For_Bogus_Node(Task task)
{
  Task node;
  int match = 0;
  
  node = task->creator->first_child;

  while (node && !match) {
    if (!(match = ((node->id == task->id) && (node!=task))))
      node = node->sibiling;
  }
  
  if (match) fprintf(stderr, "Found a bogus node\n");
  Delete_Node(node);
}

static void InsertTaskAsChildUI(Task parent, Task task)
{
  /* see function Onne_Tree_Update_Step for message_processed expln */
  message_processed = 1;
  /* check for bogus node -- if it's there delete it */
  /* Check_For_Bogus_Node(task); */
  /* create the new widget to be inserted into the tree */
  Insert_Node(task);
  /* record the action for backwards step capability */
  Record_Action(task, UNDO_INSERT, 0, task->state);
}

void Task_Type_String(Task task, char *str)
{
  switch (task->type) {
  case task_QUERY:
    strcpy(str, "Create_Query");
    break;

  case task_GOAL:
    strcpy(str, "Create_Goal");
    break;

  case task_COMMAND:
    strcpy(str, "Create_Command");
    break;

  case task_INFORM:
    strcpy(str, "Create_Inform");
    break;

  default:
    strcpy(str, "Create_Inform");
    break;
  }
}

void Tview_Create_Node(Task task, Tview_Node *node_ptr)
{
  char tcl_command[100];
  char task_type_str[20];

  Task_Type_String(task, task_type_str);
  /* allocate space for the Tview node data structure */
  *node_ptr = (Tview_Node)malloc(sizeof(struct Tview_Node_Struct));
  /* the node is initially not selected (not showing extra info) */
  (*node_ptr)->selected = 0;
  (*node_ptr)->task = task;

  /* this Tcl command will create the graphical object */
  sprintf(tcl_command, "%s %s %d %d", task_type_str, 
	  task->name, task->state, (int)(*node_ptr));
  Make_Node(tcl_command, &((*node_ptr)->wnode), *node_ptr);
}


static void ModifyTaskUI(Task task, taskState new_state, int time)
{
  /* see function One_Tree_Update_Step for message_processed expln */
  message_processed = 1;

  /* check that the task has an associated widget */
  if (task->additional) {

    /* When some tasks are completed they are removed from the tree */
    if (Delete_Node_Test(task, new_state)) {
      Delete_Node(task);
      /* it may now be possible to delete the parent */
      if (task->creator->state == task_COMPLETED) {
	ModifyTaskUI(task->creator, task_COMPLETED, time);
      }
      Record_Action(task, UNDO_DELETE, task->state, new_state);
    } else {
      /* task is not removed from tree, update the color */
      Update_State_Node(task, new_state);
      Record_Action(task, UNDO_MODIFY, task->state, new_state);
    }
  } else {
    /* even though there is no graphic we have to record the state
       change so that a back-step displays the correct information. */
    Record_Action(task, UNDO_MODIFY, task->state, new_state);
  }
}

/* When a task is completed (or killed) this function determines
   whether the node in the task tree graphics should be deleted. The
   boolean array del_when_done determines which task-types are
   deleted. If a node has children that still appear in the tree then
   it cannot be deleted. */

int Delete_Node_Test(Task task, int new_state)
{
  int result;

  result = (((new_state == task_COMPLETED)||(new_state == task_KILLED))
	    &&(del_when_done[task->type]));

  /* don't delete a node if it has undeleted children */
  if (((Tview_Node)(task->additional))->wnode->children) result = 0;

  return (result);
}

static void ModifyTask2UI(Task task, taskState2 new_state, int time)
{
  fprintf(stderr, "!! ModifyTask2UI should be called !!\n");
  /* exit(0);*/
}

/* --------------------------------------------------------------- */

int Get_Line_From_Log_File(FILE *file, char *msg, MsgData *msgData, 
		      Tcl_Interp *interp)
{
  if (!file)
      return (0);

  /* is this really neccessary ??? */
  *msg = '\0';
  
  /* we loop until a valid message has been read from the log file */
  if (ReadNlFromFile(file, msg) == EOF) {
    return (0);
  }
  
  *msgData = ParseMsg(msg);
  
  if (APP_DEBUG) {
    fprintf(stderr, "\nReadNl Got : %s\n", msg);
    fprintf(stderr, "(*msgData)->type is %d\n", (*msgData)->type);
  }

  return(1);
}

int Logged_Message_Type(taskType type)
{
  switch (type) {
  case QUERY:
  case GOAL:
  case COMMAND:
  case INFORM:
  case REPLY:
  case FAILURE:
  case SUCCESS:
  case BROADCAST:
  case TEMP_CONSTRAINT:
  case POINT_CONSTRAINT:
  case KILLED:
  case EXCEPTION:
  case INTERVAL_MONITOR:
    return 1;
    break;
  case TYPE_NULL:
  default:
    return 0;
  }
}

void Update_GUI_Log(char *msg, MsgData *msgData)
{
  char tcl_command[MAX_LONG_MESSAGE+50];

  /* check if the message is one that results in an event on the tree */
  if (Logged_Message_Type((*msgData)->type)) {
    sprintf(tcl_command, ".lbo.lbox insert end \"%d: %s\"", 
	    msg_count, msg);
    msg_count++;
  } else {
    /* should number of leading spaces be adjusted? */
    sprintf(tcl_command, ".lbo.lbox insert end \"   %s\"", 
	    msg);
  }
  log_lines++;

  Tcl_Eval(local_ti, tcl_command);
  sprintf(tcl_command, "scrollListbox");
  Tcl_Eval(local_ti, tcl_command);

#if(0)
  
  if (display_all) {
    /* shouldn't this go after the update_listbox_vars ? */
    local_update_listbox(interp);
    
    update_listbox_vars(msg, (int)(*msgData)->type);
  } else if (((*msgData)->type!=TYPE_NULL) && 
	     ((*msgData)->type!=MODULE_CONNECT) && 
	     ((*msgData)->type!=MODULE_DISCONNECT)) {
    update_listbox_vars(msg, (int)(*msgData)->type);
  } else {
    /* no show */
    listbox_msgt=-1;
  }
#endif
}

/* One_Tree_Update_Step is called from Tcl. It returns:
            1 = data successfully read
	    2 = no data available */

int One_Tree_Update_Step(Tcl_Interp *interp)
{
  char msg[MAX_LONG_MESSAGE];
  MsgData msgData;
  int result;

  message_processed = 0;

  /* this will read in lines from the log file until it reaches one
     that effects the tree -- or until the end of the log file is
     reached. */

  do {
    /* get a line from the log file, and parse it */
    result = Get_Line_From_Log_File(global_log_file, msg, 
				    &msgData, interp);

    /* if there was data to read from the log file then process it */
    if (result) {
      /* Depending on the type of the given message we may throw it into
	 the log window in the GUI */
      Update_GUI_Log(msg, &msgData);

      /* This does a lot of work */
      TaskTreeProcessNewMessage(msgData);
      if (message_processed) End_Record_Action(log_lines-1);
    } else {
      message_processed = 2;
    }

  } while (message_processed == 0);

  return (message_processed);
}

void tview_initialize(Tcl_Interp *interp)
{
  msg_count = 0;
  log_lines = 0;
  local_ti = interp;

  /* del_when_done determines whether the particular type is removed
     from the graphics tree when it completes or is killed */
  del_when_done[task_QUERY] = 1;
  del_when_done[task_BROADCAST] = 1;
  del_when_done[task_GOAL] = 0;
  del_when_done[task_COMMAND] = 0;
  del_when_done[task_EXCEPTION] = 0;
  del_when_done[task_INTERVAL_MONITOR] = 0;
  del_when_done[task_POINT_MONITOR] = 0;
  del_when_done[task_INFORM] = 1;
  del_when_done[task_EXCEPTION] = 1;

  /* Initialize the data structures neccessary for the step-back
     feature. See the file record.c */
  Initialize_Recording();

  /* TaskTreeInitialize needs to be done AFTER the graphics has been
     initialized because it will create the root node, which results
     in InsertTaskAsRootUI being called, which requires graphics to be
     initialized. */
  TaskTreeInitialize(0);
}

int tview_startup()
{
  setInsertTaskRootFn(InsertTaskAsRootUI);
  setInsertTaskChildFn(InsertTaskAsChildUI);
  setModifyTaskFn(ModifyTaskUI);
  setModifyTask2Fn(ModifyTask2UI);
  
  return(0);
}
