# -------------------------------------------------------------------- #
#
# Tview -- TCA Task Tree Graphical Tool
#
# -------------------------------------------------------------------- #

# This file should provide the basic functions for drawing
# the Task Tree (using the Willow Tcl/Tk tree library)

proc Willow_Connect_Nodes { canvas x1 y1 x2 y2 } {
    $canvas create line $x1 $y1 $x2 $y2
}

proc Willow_Move_Connector { canvas id x1 y1 x2 y2 } {
    $canvas coords $id $x1 $y1 $x2 $y2
}

proc Willow_Delete_Connector { canvas id } {
    $canvas delete $id
}

####################################################################

proc Create_Query { name state id } {
    global can task_color

    set color $task_color($state)
    set result [Create_Oval $can $name 0 0 -color $color]
    Create_Node_Callback [lindex $result 0] $id
    return $result
}

proc Create_Goal { name state id } {
    global can task_color

    set color $task_color($state)
    set result [Create_Smooth $can $name 0 0 -color $color]
    Create_Node_Callback [lindex $result 0] $id
    return $result
}

proc Create_Command { name state id } {
    global can task_color

    set color $task_color($state)
    set result [Create_Rectangle $can $name 0 0 -color $color]
    Create_Node_Callback [lindex $result 0] $id
    return $result
}

proc Create_Inform { name state id } {
    global can task_color

    set color $task_color($state)
    set result [Create_Angled $can $name 0 0 -color $color]
    Create_Node_Callback [lindex $result 0] $id
    return $result
}

proc Create_Selected { name state id } {
    global can task_color READ_FONT

    set color $task_color($state)
    set result [Create_Rectangle $can $name 0 0 -color $color \
	    -font $READ_FONT]
    Create_Node_Callback [lindex $result 0] $id
    return $result
}

####################################################################

proc Change_Node_State { node_id new_state } {
    global can task_color

    Change_Node_Background_Color $can $node_id $task_color($new_state)
}

####################################################################

# Node_Callback is called when the user clicks on a particular node in
# the tree.

# is having many individual bindings slower than having a single
# binding which does more work once it is actually called?

proc Create_Node_Callback { widget_id internal_id } {
    global can

    set command "C_Widget_Callback $internal_id"
    $can bind $widget_id <Button-1> $command

}

####################################################################

set can $place

proc Tview_Create_Tree { } {
    global can INITIAL_ORIENTATION INITIAL_X INITIAL_Y

    set tree [Willow_Create_Tree $can \
	    $INITIAL_ORIENTATION $INITIAL_X $INITIAL_Y]

    return $tree
}

####################################################################

# Update_Canvas_Size takes the width and height of the tree and will
# adjust the size of the canvas (ie. the scrollable area) so that the
# tree fits snug into that space.

proc Update_Canvas_Size { x y } {
    global CANVAS_SHOW_WIDTH CANVAS_SHOW_HEIGHT
    global CANVAS_WIDTH CANVAS_HEIGHT

    if (($x<$CANVAS_SHOW_WIDTH)&&($y<$CANVAS_SHOW_HEIGHT)) {
	return
    }
    
    if ($x<$CANVAS_SHOW_WIDTH) {
	set x $CANVAS_SHOW_WIDTH
    }

    if ($y<$CANVAS_SHOW_HEIGHT) {
	set y $CANVAS_SHOW_HEIGHT
    }

    Grow_Canvas [expr $x-$CANVAS_WIDTH] [expr $y-$CANVAS_HEIGHT]
}

proc valid_message_type { msg_type } {
    global QUERY_MESSAGE GOAL_MESSAGE COMMAND_MESSAGE INFORM_MESSAGE
    global EXCEPTION_MESSAGE KILLED_MESSAGE INTERVAL_MESSAGE POINT_MESSAGE 
    global FIRE_MESSAGE REPLY_MESSAGE FAILURE_MESSAGE SUCCESS_MESSAGE
    global BYPASS_MESSAGE PCONSTRAINT_MESSAGE RETRY_MESSAGE WILL_KILL_MESSAGE
    global TCONSTRAINT_MESSAGE WARNING_MESSAGE CONNECT_MESSAGE
    global DISCONNECT_MESSAGE BROADCAST_MESSAGE


    debugp "valid_message_type \"$msg_type\""
    
    if ([string compare $msg_type ""]==0) { return 0 }
    
    if ($msg_type==$QUERY_MESSAGE||$msg_type==$GOAL_MESSAGE||$msg_type==$COMMAND_MESSAGE||$msg_type==$INFORM_MESSAGE||$msg_type==$EXCEPTION_MESSAGE||$msg_type==$KILLED_MESSAGE||$msg_type==$INTERVAL_MESSAGE||$msg_type==$POINT_MESSAGE||$msg_type==$FIRE_MESSAGE||$msg_type==$REPLY_MESSAGE||$msg_type==$FAILURE_MESSAGE||$msg_type==$SUCCESS_MESSAGE||$msg_type==$BYPASS_MESSAGE||$msg_type==$PCONSTRAINT_MESSAGE||$msg_type==$RETRY_MESSAGE||$msg_type==$WILL_KILL_MESSAGE||$msg_type==$WARNING_MESSAGE||$msg_type==$CONNECT_MESSAGE||$msg_type==$DISCONNECT_MESSAGE||$msg_type==$BROADCAST_MESSAGE) {
	return 1 }
	
	return 0
}

#-----------------------------------------------------------------------
# Step
#
# Does the following:
# 1) Read in parsed data
# 2) Handle that message
# 
#-----------------------------------------------------------------------

set back_disabled 1

proc Step {} {
    global back_disabled

    set in [C_Step]
    if ($back_disabled) {
	.info.top.tippy.back configure -state active
	.info.top.tippy.runback configure -state active
	set back_disbled 0
    }
    update
    return $in
}

set Comview_Running 0
# set Comview_Running [expr 1 - [catch { send comview Ping }]]
#if ($Comview_Running) { send comview set Tview_Running 1 }

proc LocalStep {} {
    global Comview_Running

    if ($Comview_Running) { send comview Step }
    return [Step]
}

#-----------------------------------------------------------------------
# StepCommand is the procedure that is executed when the "Run" button is 
# pressed.
#-----------------------------------------------------------------------

set RunOK 0

proc StepCommand {} {
    global RunOK

    # if we're currently Running this will stop that:
    set RunOK 0
    LocalStep
}

proc BackCommand {} {
    global RunOK back_disabled

    # if we're currently Running this will stop that:
    set RunOK 0
    set result [C_Step_Back]
    if ($result==0) {
	.info.top.tippy.back configure -state disabled
	.info.top.tippy.runback configure -state disabled
	set back_disabled 1
    }
}

#-----------------------------------------------------------------------
# RunCommand is the procedure that is executed when the "Run" button is 
# pressed.
#-----------------------------------------------------------------------

proc RunCommand {} {
    global RunOK
    set RunOK [expr 1-$RunOK]
    if ($RunOK==1) { Run }
}

proc RunBackCommand {} {
    global RunOK
    set RunOK [expr 1-$RunOK]
    if ($RunOK==1) { RunBack }
}

#-----------------------------------------------------------------------
# 
# Run
#
#-----------------------------------------------------------------------

proc Run {} {
    global RunOK

    .info.top.tippy.run configure -bg #11f033
    .info.top.tippy.run configure -activebackground #33ff44

    for {} {($RunOK==1)} {} {
	set run_result [LocalStep]
	if ($run_result==2) { 
	    .info.top.tippy.run configure -bg lightyellow
	    .info.top.tippy.run configure -activebackground #deff10
	}
    }
    
    .info.top.tippy.run configure -bg #d9d9d9
    .info.top.tippy.run configure -activebackground #ececec
}

proc RunBack {} {
    global RunOK

    .info.top.tippy.runback configure -bg #11f033
    .info.top.tippy.runback configure -activebackground #33ff44

    for {} {($RunOK==1)} {} {
	set run_result [C_Step_Back]
	update
	if ($run_result==0) {
	    .info.top.tippy.back configure -state disabled
	    .info.top.tippy.runback configure -state disabled
	    set back_disabled 1
	    set RunOK 0
	}
    }
    
    .info.top.tippy.runback configure -bg #d9d9d9
    .info.top.tippy.runback configure -activebackground #ececec
}

#-----------------------------------------------------------------------
# 
# Stepn is only a development/debugging function
#
#-----------------------------------------------------------------------

proc Stepn {n} {
    for {set loop 1} {$loop < $n} { incr loop } {
	LocalStep
    }
}

proc Ping {} {
    return 1
}

#-----------------------------------------------------------------------
# Do any neccessary post-graphic-startup initialization in C
C_Initialize
