#include <stdio.h>
#include <string.h>
#include <tcl.h>
#include <tk.h>
#include "tools/Standard.h"
#include "tools/List.h"
#include "tools/Array.h"
#include "tools/MsgData.h"
#include "tools/TaskTree.h"
#include "tools/MsgData.h"
#include "tools/Parser.h"
#include "filei.h"
#include "tview.h"
#include <willow.h>
#include "tclsetup.h"
#include "uistuff.h"

extern Tcl_Interp *local_ti;
extern Willow_Tree_Type global_tree;
extern int del_when_done[16];
extern Task root;

/* ---------------------- local prototypes ------------------------ */


void smaller()
{
  /* this requires all nodes to be redrawn */

}

void Select_Widget(void *x)
{
  char tcl_command[100];
  Tview_Node tnode;
  Willow_Node new_wnode;
  int width, height;
  char description[250];

  tnode = (Tview_Node)x;

  /* if the widget is already selected, then revert to original */
  if (tnode->selected) {
    Task_Type_String(tnode->task, description);
    sprintf(tcl_command, "%s %s %d %d",
	    description, (tnode->task)->name, 
	    (tnode->task)->state, (int)tnode);
  } else {
    /* the node is selected, show extra token info */
    Task_Description(tnode->task, (tnode->task)->state, description);
    sprintf(tcl_command, "Create_Selected \"%s\" %d %d",
	    description, (tnode->task)->state, (int)tnode);
  }

  Make_Node(tcl_command, &new_wnode, tnode);
  /* replace the node that's there with this new node */
  Willow_Modify_Node(global_tree, tnode->wnode, new_wnode);
  
  /* we now update the display of the tree */
  Willow_Draw_Tree(global_tree, &width, &height);

  /* update the size of the canvas based on new size of tree */
  Update_Canvas_Size(global_tree, width, height);

  /* invert the state of the selection */
  tnode->selected = (1 - tnode->selected);
}

void Update_Selected_Widget(Tview_Node tnode, taskState state)
{
  char tcl_command[100];
  Willow_Node new_wnode;
  int width, height;
  char description[250];

  /* the node is selected, show extra token info */
  Task_Description(tnode->task, state, description);
  sprintf(tcl_command, "Create_Selected \"%s\" %d %d",
	  description, state, (int)tnode);
  Make_Node(tcl_command, &new_wnode, tnode);
  /* replace the node that's there with this new node */
  Willow_Modify_Node(global_tree, tnode->wnode, new_wnode);
	
  /* we now update the display of the tree */
  Willow_Draw_Tree(global_tree, &width, &height);
	
  /* update the size of the canvas based on new size of tree */
  Update_Canvas_Size(global_tree, width, height);
}

/* first arg is tcl command to create node */
void Make_Node(char *tcl_command, Willow_Node *new_wnode, 
	       Tview_Node tnode)
{
  char *widget_id;
  int width, height, x1, x2, y1, y2;
  char *object_result;

  My_Tcl_Eval(local_ti, tcl_command);
  /* the return result is a list which includes the widget id */
  object_result = strdup(local_ti->result);
  
  /* now extract the 3 results that are returned as a list */
  sprintf(tcl_command, "lindex [list %s] 0", object_result);
  My_Tcl_Eval(local_ti, tcl_command);
  widget_id = strdup(local_ti->result);
  sprintf(tcl_command, "lindex [list %s] 1", object_result);
  My_Tcl_Eval(local_ti, tcl_command);
  sscanf(local_ti->result, "%d", &width);
  sprintf(tcl_command, "lindex [list %s] 2", object_result);
  My_Tcl_Eval(local_ti, tcl_command);
  sscanf(local_ti->result, "%d", &height);

  /* this assumes the object is centered about (0,0) */
  x2 = (width / 2);
  y2 = (height / 2);
  x1 = -x2;
  y1 = -y2;

  /* create a node for the widget and insert the node into the tree */
  Willow_Create_Node(global_tree, x1, y1, x2, y2, widget_id, 
		     new_wnode, (void *)tnode);
}

void Task_Description(Task task, taskState state, char *text)
{
  sprintf(text, "%s    \n%s {%d}    \n",
	  TaskTypeString(task->type), task->name, task->id);
  
  if (task->state2 != task_NORMAL) {
    strcat(text, TaskState2String(task->state2));
    strcat(text, "    \n");
  }
  strcat(text, TaskStateString(state));
  strcat(text, "    \n");
  strcat(text, task->module);
  strcat(text, "\n");
  /* 
     strcat(text, TemporalConstraints(task,&(constraintsText[0])));
     */
}

void Redraw_Display()
{
  int height, width;

  Willow_Redraw_Tree(global_tree);

  /* we now update the display of the tree */
  Willow_Draw_Tree(global_tree, &width, &height);
  
  /* update the size of the canvas based on new size of tree */
  Update_Canvas_Size(global_tree, width, height);
}

void toggle_layout()
{
  int width, height;

  global_tree->orientation = (1 - global_tree->orientation);
  /* redraw the tree */
  Willow_Draw_Tree(global_tree, &width, &height);

  /* update the size of the canvas based on new size of tree */
  Update_Canvas_Size(global_tree, width, height);
}

void toggle_draw_nodes()
{
}

void toggle_message(int task_type)
{
  del_when_done[task_type] =  1 - del_when_done[task_type];
}

void Update_Canvas_Size(Willow_Tree_Type tree, int width, int height)
{
  char tcl_command[100];
  int x, y;

  if (tree->orientation == WILLOW_VERTICAL) {
    x = width;
    y = height;
  } else {
    y = width;
    x = height;
  }

  sprintf(tcl_command, "Update_Canvas_Size %d %d", x+50, y+50);
  My_Tcl_Eval(local_ti, tcl_command);
}

void Set_Child_Space(int size)
{
  Willow_Set_Child_Space(global_tree, size);
}

void Set_Sibling_Space(int size)
{
  Willow_Set_Sibling_Space(global_tree, size);
}


void Delete_Node(Task task)
{
  Willow_Node wnode;
  int width, height;

  /* get the willow node from the tree node */
  wnode = (Willow_Node)((Tview_Node)task->additional)->wnode;
  /* task must be removed from the tree */
  Willow_Delete_Node(global_tree, wnode);
  /* we now update the display of the tree */
  Willow_Draw_Tree(global_tree, &width, &height);
  /* the task no longer needs the Tview node pointer */
  task->additional = 0;
}

void Insert_Node(Task task)
{
  Task parent;
  Tview_Node tnode;
  Willow_Node parent_node;
  Willow_Node wnode;
  int width, height;

  Tview_Create_Node(task, &tnode);
  task->additional = (void *)tnode;
  wnode = tnode->wnode;

  parent = task->creator;
  if (!(parent->additional)) {
    parent = root;
  }

  parent_node = (Willow_Node)(((Tview_Node)(parent->additional))->wnode);
    
  /* we insert that widget into the tree data structure. the hairy
       third argument is simply obtaining the parent's Willow node. */
  Willow_Insert_Node(global_tree, wnode, parent_node);
  /* we now update the display of the tree, getting new h & w */
  Willow_Draw_Tree(global_tree, &width, &height);
  /* update the size of the canvas based on new size of tree */
  Update_Canvas_Size(global_tree, width, height);
}

void Update_State_Node(Task task, taskState state)
{
  Willow_Node wnode;
  char tcl_command[100];

  if (((Tview_Node)task->additional)->selected) {
    Update_Selected_Widget((Tview_Node)task->additional, state);
  } else {
    /* get the willow node from the tree node */
    wnode = (Willow_Node)((Tview_Node)task->additional)->wnode;
    /* update the background color of the node */
    sprintf(tcl_command, "Change_Node_State %s %d",
	    wnode->widget_id, state);
    My_Tcl_Eval(local_ti, tcl_command);
  }
}

void Highlight_Log_Line(int line_number)
{
  char tcl_command[100];

  sprintf(tcl_command, "Highlight_Log_Line %d",
	  line_number);
  My_Tcl_Eval(local_ti, tcl_command);
}
