# node.tcl
#
# This file provides the following functions:
#    Create_Oval, Create_Rectangle, Create_Angled, Create_Smooth
#

source $WILLOW_DIRECTORY/keyword.tcl

set PORTHOLE_FONT "-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-iso*-*"
set READ_FONT     "-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-iso*-*"

set PORTHOLE_FONT "-adobe-helvetica-medium-r-normal--*-100-*-*-*-*-iso*-*"
set READ_FONT     "-adobe-helvetica-medium-r-normal--*-100-*-*-*-*-iso*-*"

set DEFAULT_FONT $PORTHOLE_FONT

set X_PAD 15
set Y_PAD 6
#set INTERNODE_SPACE 0
set INTERNODE_SPACE 4

####################################################################

proc Set_Font_Size { s } {
    global DEFAULT_FONT X_PAD

    set DEFAULT_FONT "-adobe-helvetica-medium-r-normal--*-$s-*-*-*-*-iso*-*"
    if ($s<90) { set X_PAD 10 } else { set X_PAD 15 }
    if ($s<40) { set X_PAD 5 }
    C_Redraw_Tree
}

set UNIQUE_ID_COUNT 0

proc Unique_Identifier { } {
    global UNIQUE_ID_COUNT
    
    incr UNIQUE_ID_COUNT
    return uid$UNIQUE_ID_COUNT
}

# why is this here?
if (0) {

proc Create_Node_Binding { canvas tag } {

    set command "$canvas itemconfigure current -width 2"

    $canvas bind $tag <Any-Enter> $command

    $canvas bind $tag <Button-1> {
	set item [$canvas find withtag current]
	Widget_Select $item
    }

    $canvas bind $tag <Any-Leave> {
	$place itemconfigure current -width 1
    }
}

}

proc Widget_Select { item } {
    global itemcode
    .info.top.tippy.unselect configure -state normal
    C_Widget_Callback $itemcode($item)
}

proc Create_Text { can name x y read_font id args } {
    global DEFAULT_FONT X_PAD Y_PAD

    set font $DEFAULT_FONT
    Keyword_Args $args [list -font font]
    
    set text [$can create text $x $y \
	    -text $name -font $font \
	    -tag $id]

    set b [$can bbox $text]
    set width [expr [lindex $b 2] - [lindex $b 0] + $X_PAD]
    set height [expr [lindex $b 3] - [lindex $b 1] + $Y_PAD]

    return [list $width $height $text]
}


####################################################################

# Currently these do not create canvas objects in the same manner as
# genuine canvas objects are created. Is there any benefit to doing it
# that way?

proc Create_Oval { can name x y args } {
    global node_background

    set fill_color blue
    Keyword_Args $args [list -color fill_color]
    
    # This identifier will be used to tag all the components of a node
    # (the text and the oval background)
    set id [Unique_Identifier]

    set text [Create_Text $can $name $x $y 0 $id]
    set width [lindex $text 0]
    set height [lindex $text 1]
    set text [lindex $text 2]

    set x [expr $x - $width/2]
    set y [expr $y - $height/2]

    if ([llength $args]>0) {
	# assumes [lindex $args 0]=="-color"
	set fill_color [lindex $args 1]
    }

    set item [$can create oval $x $y \
	    [expr $x+$width] \
	    [expr $y+$height] \
	    -fill $fill_color \
	    -tag $id]

    # create the standard bindings for the node
    Create_Node_Bindings $can $item $text

    # keep track of the background
    set node_background($id) $item

    $can raise $text $item

    # what really needs to be returned is an identifier for
    # the composite object (text + oval)
    return [list $id $width $height]
}

proc Create_Rectangle { can name x y args } {
    global node_background DEFAULT_FONT

    set fill_color blue
    set font $DEFAULT_FONT
    Keyword_Args $args [list -color fill_color -font font]

    # This identifier will be used to tag all the components of a node
    # (the text and the rectangular background)
    set id [Unique_Identifier]

    set text [Create_Text $can $name $x $y 0 $id -font $font]
    set width [lindex $text 0]
    set height [lindex $text 1]
    set text [lindex $text 2]

    set x [expr $x - $width/2]
    set y [expr $y - $height/2]

    set item [$can create rectangle $x $y \
	    [expr $x+$width] \
	    [expr $y+$height] \
	    -fill $fill_color \
	    -tag $id]

    # create the standard bindings for the node
    Create_Node_Bindings $can $item $text
    
    # keep track of the background
    set node_background($id) $item

    $can raise $text $item

    return [list $id $width $height]
}

proc Create_Smooth { can name x y args } {
    global node_background

    set fill_color blue
    Keyword_Args $args [list -color fill_color]

    set id [Unique_Identifier]
    set text [Create_Text $can $name $x $y 0 $id]
    set width [lindex $text 0]
    set height [lindex $text 1]
    set text [lindex $text 2]

    set smooth 20
    set smooth [expr $height]

    set x [expr $x - $width/2]
    set y [expr $y - $height/2]

    set item [$can create polygon \
	    $x $y \
	    [expr $x+$smooth] $y \
	    [expr $x+$width-$smooth] $y \
	    [expr $x+$width] $y \
	    [expr $x+$width] [expr $y+$smooth] \
	    [expr $x+$width] [expr $y+$height-$smooth] \
	    [expr $x+$width] [expr $y+$height] \
	    [expr $x+$width-$smooth] [expr $y+$height] \
	    [expr $x+$smooth] [expr $y+$height] \
	    $x [expr $y+$height] \
	    $x [expr $y+$height-$smooth] \
	    $x [expr $y+$smooth] \
	    $x $y \
	    -smooth t \
	    -fill $fill_color \
	    -outline black \
	    -tag $id]

    # create the standard bindings for the node
    Create_Node_Bindings $can $item $text

    # keep track of the background
    set node_background($id) $item

    $can raise $text $item

    return [list $id $width $height]
}

proc Create_Angled { can name x y args } {
    global node_background

    set fill_color blue
    Keyword_Args $args [list -color fill_color]

    set smooth 10

    set id [Unique_Identifier]
    set text [Create_Text $can $name $x $y 0 $id]
    set width [lindex $text 0]
    set height [lindex $text 1]
    set text [lindex $text 2]

    set x [expr $x - $width/2]
    set y [expr $y - $height/2]

    if ([llength $args]>0) {
	# assumes [lindex $args 0]=="-color"
	set fill_color [lindex $args 1]
    }

    set item [$can create polygon \
	    [expr $x+$smooth] $y \
	    [expr $x+$width-$smooth] $y \
	    [expr $x+$width] [expr $y+($height/2)] \
	    [expr $x+$width-$smooth] [expr $y+$height] \
	    [expr $x+$smooth] [expr $y+$height] \
	    $x [expr $y+($height/2)] \
	    [expr $x+$smooth] $y \
	    -fill $fill_color \
	    -outline black \
	    -tag $id]

    # create the standard bindings for the node
    Create_Node_Bindings $can $item $text

    # keep track of the background
    set node_background($id) $item

    $can raise $text $item

    return [list $id $width $height]
}

####################################################################

proc Change_Node_Background_Color { can id new_color } {
    global node_background

    $can itemconfigure $node_background($id) -fill $new_color
}

proc Create_Node_Bindings { can bg_id text_id } {

    set command "$can itemconfigure $bg_id -width 2"
    $can bind $bg_id <Any-Enter> $command
    $can bind $text_id <Any-Enter> $command

    set command "$can itemconfigure $bg_id -width 1"
    $can bind $bg_id <Any-Leave> $command
    $can bind $text_id <Any-Leave> $command
}