/* tcl-interface.c */
/* Willow -- A tree layout package for Tcl/Tk */

#include <tcl.h>
#include <tk.h>
#include <string.h>
#include "willow.h"

int C_Willow_Create_Tree(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[]);
int C_Willow_Insert(ClientData clientData, Tcl_Interp *interp,
		    int argc, char *argv[]);
int C_Willow_Draw_Tree(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[]);
int C_Willow_Delete(ClientData clientData, Tcl_Interp *interp,
		    int argc, char *argv[]);
int C_Willow_Replace(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[]);

/* ---------------------------------------------------------------- */

void Add_Willow_Commands(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp, "Willow_Create_Tree", C_Willow_Create_Tree,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "Willow_Insert", C_Willow_Insert,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "Willow_Delete", C_Willow_Delete,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "Willow_Draw_Tree", C_Willow_Draw_Tree,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

  Tcl_CreateCommand(interp, "Willow_Replace", C_Willow_Replace,
		    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
}

/* ---------------------------------------------------------------- */

/* This is the stub that calls the actual Willow_Create_Tree after
   converting all of Tcl's arguments (strings) to the appropriate
   type. */

int C_Willow_Create_Tree(ClientData clientData, Tcl_Interp *interp,
		       int argc, char *argv[])
{
  Willow_Tree_Type tree;
  int orientation, x, y;

  if (argc != 5) {
    interp->result = "Wrong number of arguments to Willow_Create_Tree";
    return TCL_ERROR;
  }

  /* should check that there are a sufficient number of arguments */
  sscanf(argv[2], "%d", &orientation);
  sscanf(argv[3], "%d", &x);
  sscanf(argv[4], "%d", &y);

  Willow_Create_Tree(interp, strdup(argv[1]), orientation, x, y,
		     &tree);

  /* the Tcl return result is a pointer to the new tree */
  sprintf(interp->result, "%d", (int)tree);
  return TCL_OK;
}

int C_Willow_Insert(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[])
{
  int ptr;
  Willow_Tree_Type tree;
  Willow_Node parent, new_node;
  int x1, y1, x2, y2;

  if (argc != 8) {
    interp->result = "Wrong number of arguments to Willow_Create_Tree";
    return TCL_ERROR;
  }

  sscanf(argv[1], "%d", &ptr);
  tree = (Willow_Tree_Type)ptr;
  sscanf(argv[2], "%d", &ptr);
  parent = (Willow_Node)ptr;
  sscanf(argv[4], "%d", &x1);
  sscanf(argv[5], "%d", &y1);
  sscanf(argv[6], "%d", &x2);
  sscanf(argv[7], "%d", &y2);

  Willow_Create_Node(tree, x1, y1, x2, y2, strdup(argv[3]),
		     &new_node, (void *)NULL);

  Willow_Insert_Node(tree, new_node, parent);

  /* the Tcl return result is a pointer to the new node */
  sprintf(interp->result, "%d", (int)new_node);
  return TCL_OK;
}

int C_Willow_Replace(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[])
{
  int ptr;
  Willow_Node old_node, new_node;
  Willow_Tree_Type tree;
  int x1, y1, x2, y2;

  if (argc != 8) {
    interp->result = "Wrong number of arguments to Willow_Replace";
    return TCL_ERROR;
  }

  sscanf(argv[1], "%d", &ptr);
  tree = (Willow_Tree_Type)ptr;
  sscanf(argv[2], "%d", &ptr);
  old_node = (Willow_Node)ptr;
  sscanf(argv[4], "%d", &x1);
  sscanf(argv[5], "%d", &y1);
  sscanf(argv[6], "%d", &x2);
  sscanf(argv[7], "%d", &y2);

  Willow_Create_Node(tree, x1, y1, x2, y2, strdup(argv[3]),
		     &new_node, (void *)NULL);

  Willow_Modify_Node(tree, old_node, new_node);

  /* the Tcl return result is a pointer to the new node */
  sprintf(interp->result, "%d", (int)new_node);
  return TCL_OK;
}

int C_Willow_Draw_Tree(ClientData clientData, Tcl_Interp *interp,
		     int argc, char *argv[])
{
  int ptr;
  Willow_Tree_Type tree;
  int x, y;

  sscanf(argv[1], "%d", &ptr);
  tree = (Willow_Tree_Type)ptr;

  Willow_Draw_Tree(tree, &x, &y);
  /* the Tcl return result is the dimensions of the tree */
  sprintf(interp->result, "%d %d", x, y);

  return TCL_OK;
}

int C_Willow_Delete(ClientData clientData, Tcl_Interp *interp,
		  int argc, char *argv[])
{
  int ptr;
  Willow_Tree_Type tree;
  Willow_Node node;

  if (argc != 3) {
    interp->result = "Wrong number of arguments to Willow_Create_Tree";
    return TCL_ERROR;
  }

  sscanf(argv[1], "%d", &ptr);
  tree = (Willow_Tree_Type)ptr;
  sscanf(argv[2], "%d", &ptr);
  node = (Willow_Node)ptr;

  Willow_Delete_Node(tree, node);

  return TCL_OK;
}
