#ifndef _WILLOW_H
#define _WILLOW_H

#define MAX_TREE_DEPTH 255
#define SIBLING_SPACING 4
#define CHILD_SPACING 20

#define WILLOW_VERTICAL 0
#define WILLOW_HORIZONTAL 1

typedef struct Willow_Node_Struct *Willow_Node;

struct Willow_Node_Struct {

  char *widget_id;
  char *connector_id;
  int height;              /* these are the x, y dimensions of the */
  int width;               /* node, irrespective of tree orientation */
  int children_dimension;
  int max;
  
  int current_x;
  int current_y;

  Willow_Node sibling;
  Willow_Node children;
  Willow_Node last_child; /* used only for insertion of a node */

  Willow_Node prev_sibling; /* these two needed for O(1) deletion */
  Willow_Node parent;

  void *additional;      /* user fills this in for redraw callback */
};

typedef struct Willow_Tree {

  Tcl_Interp *interp;
  char *canvas_name;
  int max_level_size[MAX_TREE_DEPTH];
  Willow_Node root_node;
  int orientation;

  int child_space;
  int sibling_space;
  int max_depth;

  int canvas_x;     /* (x,y) offset to display the tree */
  int canvas_y;

  int callback_language;    /* 0 == Tcl, 1 == C */
  char *tcl_callback_fn;
  void (* c_callback_fn)(void *);

} *Willow_Tree_Type;


/* ------------------- Function Prototypes ------------------------ */

void Willow_Create_Tree(Tcl_Interp *interp, char *canvas_name, 
			int orientation, int x_offset, int y_offset, 
			Willow_Tree_Type *tree);
void Willow_Draw_Tree(Willow_Tree_Type t, int *x, int *y);
void Willow_Redraw_Tree(Willow_Tree_Type t);
void Willow_Destroy_Tree(Willow_Tree_Type t);
void Willow_Create_Node(Willow_Tree_Type tree, 
			int x1, int y1, int x2, int y2,
			char *widget_id, Willow_Node *n,
			void *additional);
void Willow_Insert_Node(Willow_Tree_Type t, Willow_Node n,
			Willow_Node parent);
void Willow_Delete_Node(Willow_Tree_Type t, Willow_Node n);
void Willow_Modify_Node(Willow_Tree_Type t, Willow_Node old_node,
			Willow_Node new_node);
void Willow_Set_Tcl_Callback(Willow_Tree_Type tree, char *command);
void Willow_Set_C_Callback(Willow_Tree_Type tree, 
			   void (* c_callback_fn)(void *));
void Willow_Set_Child_Space(Willow_Tree_Type tree, int space);
void Willow_Set_Sibling_Space(Willow_Tree_Type tree, int space);

/* This requires Tcl... should tcl.h be included here? */

void Add_Willow_Commands(Tcl_Interp *interp);

#endif /* _WILLOW_H */

